@@ -106,37 +106,40 @@ class EQConstraintComp(ExplicitComponent):
                                use_mult, mult_name, mult_val, normalize, add_constraint, ref, ref0,
                                adder, scaler, **kwargs)
 
-    def setup(self):
+    def _post_configure(self):
         """
         Define the independent variables, output variables, and partials.
         """
+        # set static mode to False because we are doing things that would normally be done in setup
+        self._static_mode = False
+
         for name, options in iteritems(self._output_vars):
 
             meta = self.add_output(name, **options['kwargs'])
 
-            n = meta['size']
+            shape = meta['shape']
 
             for s in ('lhs', 'rhs', 'mult'):
                 if options['{0}_name'.format(s)] is None:
                     options['{0}_name'.format(s)] = '{0}:{1}'.format(s, name)
 
             self.add_input(options['lhs_name'],
-                           val=np.ones(n),
+                           val=np.ones(shape),
                            units=options['eq_units'])
 
             self.add_input(options['rhs_name'],
-                           val=options['rhs_val'] * np.ones(n),
+                           val=options['rhs_val'] * np.ones(shape),
                            units=options['eq_units'])
 
             if options['use_mult']:
                 self.add_input(options['mult_name'],
-                               val=options['mult_val'] * np.ones(n),
+                               val=options['mult_val'] * np.ones(shape),
                                units=None)
 
-            self._scale_factor = np.ones(n)
-            self._dscale_drhs = np.ones(n)
+            self._scale_factor = np.ones(shape)
+            self._dscale_drhs = np.ones(shape)
 
-            ar = np.arange(n)
+            ar = np.arange(np.prod(shape))
             self.declare_partials(of=name, wrt=options['lhs_name'], rows=ar, cols=ar, val=1.0)
             self.declare_partials(of=name, wrt=options['rhs_name'], rows=ar, cols=ar, val=1.0)
 
@@ -147,6 +150,10 @@ class EQConstraintComp(ExplicitComponent):
                 self.add_constraint(name, equals=0., ref0=options['ref0'], ref=options['ref'],
                                     adder=options['adder'], scaler=options['scaler'])
 
+        self._static_mode = True
+
+        super(EQConstraintComp, self)._post_configure()
+
     def compute(self, inputs, outputs):
         """
         Calculate the output for each equality constraint.
@@ -227,15 +234,18 @@ class EQConstraintComp(ExplicitComponent):
                 mult = inputs[mult_name]
 
                 # Partials of output wrt mult
-                partials[name, mult_name] = lhs * self._scale_factor
+                deriv = lhs * self._scale_factor
+                partials[name, mult_name] = deriv.flatten()
             else:
                 mult = 1.0
 
             # Partials of output wrt rhs
-            partials[name, rhs_name] = (mult * lhs - rhs) * self._dscale_drhs - self._scale_factor
+            deriv = (mult * lhs - rhs) * self._dscale_drhs - self._scale_factor
+            partials[name, rhs_name] = deriv.flatten()
 
             # Partials of output wrt lhs
-            partials[name, lhs_name] = mult * self._scale_factor
+            deriv = mult * self._scale_factor
+            partials[name, lhs_name] = deriv.flatten()
 
     def add_eq_output(self, name, eq_units=None, lhs_name=None, rhs_name=None, rhs_val=0.0,
                       use_mult=False, mult_name=None, mult_val=1.0, normalize=True,
