@@ -66,10 +66,13 @@ class MuxComp(ExplicitComponent):
         """
         self._vars[name] = {'val': val, 'shape': shape, 'units': units, 'desc': desc, 'axis': axis}
 
-    def setup(self):
+    def _post_configure(self):
         """
         Declare inputs, outputs, and derivatives for the demux component.
         """
+        # set static mode to False because we are doing things that would normally be done in setup
+        self._static_mode = False
+
         opts = self.options
         vec_size = opts['vec_size']
 
@@ -117,6 +120,10 @@ class MuxComp(ExplicitComponent):
 
                 self.declare_partials(of=var, wrt=in_name, rows=rs, cols=cs, val=1.0)
 
+        self._static_mode = True
+
+        super(MuxComp, self)._post_configure()
+
     def compute(self, inputs, outputs):
         """
         Mux the inputs into the appropriate outputs.
