@@ -3,10 +3,13 @@
 from __future__ import division
 
 from collections import OrderedDict, Counter, defaultdict
+
+# note: this is a Python 3.3 change, clean this up for OpenMDAO 3.x
 try:
     from collections.abc import Iterable
 except ImportError:
     from collections import Iterable
+
 from itertools import product
 from six import string_types, iteritems, itervalues
 
@@ -222,6 +225,24 @@ class Component(System):
                                                                 self._approx_schemes):
             raise RuntimeError("%s: num_par_fd is > 1 but no FD is active." % self.msginfo)
 
+        self._static_mode = True
+
+        if self.options['distributed']:
+            if self._distributed_vector_class is not None:
+                self._vector_class = self._distributed_vector_class
+            else:
+                simple_warning("The 'distributed' option is set to True for Component %s, "
+                               "but there is no distributed vector implementation (MPI/PETSc) "
+                               "available. The default non-distributed vectors will be used."
+                               % pathname)
+                self._vector_class = self._local_vector_class
+        else:
+            self._vector_class = self._local_vector_class
+
+    def _post_configure(self):
+        """
+        Do any remaining setup that had to wait until after final user configuration.
+        """
         # check here if declare_coloring was called during setup but declare_partials
         # wasn't.  If declare partials wasn't called, call it with of='*' and wrt='*' so we'll
         # have something to color.
@@ -236,19 +257,7 @@ class Component(System):
                                "using default metadata and method='%s'." % (self.msginfo, method))
                 self.declare_partials('*', '*', method=method)
 
-        self._static_mode = True
-
-        if self.options['distributed']:
-            if self._distributed_vector_class is not None:
-                self._vector_class = self._distributed_vector_class
-            else:
-                simple_warning("The 'distributed' option is set to True for Component %s, "
-                               "but there is no distributed vector implementation (MPI/PETSc) "
-                               "available. The default non-distributed vectors will be used."
-                               % pathname)
-                self._vector_class = self._local_vector_class
-        else:
-            self._vector_class = self._local_vector_class
+        super(Component, self)._post_configure()
 
     def _setup_var_data(self, recurse=True):
         """
@@ -307,7 +316,8 @@ class Component(System):
                 abs2prom[type_][abs_name] = prom_name
 
                 # Compute allprocs_discrete (metadata for discrete vars)
-                self._var_allprocs_discrete[type_][abs_name] = val
+                self._var_allprocs_discrete[type_][abs_name] = v = val.copy()
+                del v['value']
 
         self._var_allprocs_abs2prom = abs2prom
 
@@ -376,7 +386,6 @@ class Component(System):
         if self._use_derivatives:
             self._var_sizes['nonlinear'] = self._var_sizes['linear']
 
-        # for a component, all vars are 'owned'
         self._owned_sizes = self._var_sizes['nonlinear']['output']
 
         self._setup_global_shapes()
@@ -517,25 +526,24 @@ class Component(System):
         if tags is not None and not isinstance(tags, (str, list)):
             raise TypeError('The tags argument should be a str or list')
 
-        metadata = {}
-
         # value, shape: based on args, making sure they are compatible
-        metadata['value'], metadata['shape'], src_indices = ensure_compatible(name, val, shape,
-                                                                              src_indices)
-        metadata['size'] = np.prod(metadata['shape'])
+        value, shape, src_indices = ensure_compatible(name, val, shape, src_indices)
+        distributed = self.options['distributed']
 
-        # src_indices: None or ndarray
-        if src_indices is None:
-            metadata['src_indices'] = None
-        else:
+        metadata = {
+            'value': value,
+            'shape': shape,
+            'size': np.prod(shape),
+            'src_indices': None,
+            'flat_src_indices': flat_src_indices,
+            'units': units,
+            'desc': desc,
+            'distributed': distributed,
+            'tags': make_set(tags),
+        }
+
+        if src_indices is not None:
             metadata['src_indices'] = np.asarray(src_indices, dtype=INT_DTYPE)
-        metadata['flat_src_indices'] = flat_src_indices
-
-        metadata['units'] = units
-        metadata['desc'] = desc
-        metadata['distributed'] = self.options['distributed']
-
-        metadata['tags'] = make_set(tags)
 
         # We may not know the pathname yet, so we have to use name for now, instead of abs_name.
         if self._static_mode:
@@ -693,37 +701,24 @@ class Component(System):
         if tags is not None and not isinstance(tags, (str, set, list)):
             raise TypeError('The tags argument should be a str, set, or list')
 
-        metadata = {}
-
         # value, shape: based on args, making sure they are compatible
-        metadata['value'], metadata['shape'], _ = ensure_compatible(name, val, shape)
-        metadata['size'] = np.prod(metadata['shape'])
-
-        # units, res_units: taken as is
-        metadata['units'] = units
-        metadata['res_units'] = res_units
-
-        # desc: taken as is
-        metadata['desc'] = desc
+        value, shape, _ = ensure_compatible(name, val, shape)
 
         if lower is not None:
-            lower = ensure_compatible(name, lower, metadata['shape'])[0]
+            lower = ensure_compatible(name, lower, shape)[0]
             self._has_bounds = True
         if upper is not None:
-            upper = ensure_compatible(name, upper, metadata['shape'])[0]
+            upper = ensure_compatible(name, upper, shape)[0]
             self._has_bounds = True
 
-        metadata['lower'] = lower
-        metadata['upper'] = upper
-
         # All refs: check the shape if necessary
         for item, item_name in zip([ref, ref0, res_ref], ['ref', 'ref0', 'res_ref']):
             if not isscalar(item):
                 it = atleast_1d(item)
-                if it.shape != metadata['shape']:
+                if it.shape != shape:
                     raise ValueError("{}: When adding output '{}', expected shape {} but got "
                                      "shape {} for argument '{}'.".format(self.msginfo, name,
-                                                                          metadata['shape'],
+                                                                          shape,
                                                                           it.shape, item_name))
 
         if isscalar(ref):
@@ -745,13 +740,23 @@ class Component(System):
         ref0 = format_as_float_or_array('ref0', ref0, flatten=True)
         res_ref = format_as_float_or_array('res_ref', res_ref, flatten=True)
 
-        metadata['ref'] = ref
-        metadata['ref0'] = ref0
-        metadata['res_ref'] = res_ref
+        distributed = self.options['distributed']
 
-        metadata['distributed'] = self.options['distributed']
-
-        metadata['tags'] = make_set(tags)
+        metadata = {
+            'value': value,
+            'shape': shape,
+            'size': np.prod(shape),
+            'units': units,
+            'res_units': res_units,
+            'desc': desc,
+            'distributed': distributed,
+            'tags': make_set(tags),
+            'ref': ref,
+            'ref0': ref0,
+            'res_ref': res_ref,
+            'lower': lower,
+            'upper': upper,
+        }
 
         # We may not know the pathname yet, so we have to use name for now, instead of abs_name.
         if self._static_mode:
@@ -818,6 +823,37 @@ class Component(System):
 
         return metadata
 
+    def _update_dist_src_indices(self, abs_in2out):
+        """
+        Set default src_indices on distributed components for any inputs where they aren't set.
+
+        Parameters
+        ----------
+        abs_in2out : dict
+            Mapping of connected inputs to their source.  Names are absolute.
+        """
+        if not self.options['distributed']:
+            return
+
+        iproc = self.comm.rank
+        abs2meta = self._var_abs2meta
+        sizes = np.zeros(self.comm.size, dtype=INT_DTYPE)
+        tmp = np.zeros(1, dtype=INT_DTYPE)
+
+        for iname in self._var_allprocs_abs_names['input']:
+            if iname in abs2meta and iname in abs_in2out:
+                if abs2meta[iname]['src_indices'] is None:
+                    metadata = abs2meta[iname]
+                    tmp[0] = metadata['size']
+                    self.comm.Allgather(tmp, sizes)
+                    offset = np.sum(sizes[:iproc])
+                    end = offset + sizes[iproc]
+                    simple_warning("{}: Component is distributed but input '{}' was added without "
+                                   "src_indices. Setting "
+                                   "src_indices to range({}, {}).".format(self.msginfo, iname,
+                                                                          offset, end))
+                    metadata['src_indices'] = np.arange(offset, end, dtype=INT_DTYPE)
+
     def _approx_partials(self, of, wrt, method='fd', **kwargs):
         """
         Inform the framework that the specified derivatives are to be approximated.
