@@ -3,10 +3,13 @@ from __future__ import division
 
 import os
 from collections import Counter, OrderedDict, defaultdict
+
+# note: this is a Python 3.3 change, clean this up for OpenMDAO 3.x
 try:
     from collections.abc import Iterable
 except ImportError:
     from collections import Iterable
+
 from itertools import product, chain
 from numbers import Number
 import inspect
@@ -78,6 +81,10 @@ class Group(System):
         Mapping of local subsystem names to their corresponding System.
     _approx_subjac_keys : list
         List of subjacobian keys used for approximated derivatives.
+    _setup_procs_finished : bool
+        Flag to check if setup_procs is complete
+    _has_distrib_vars : bool
+        If True, this Group contains distributed variables.
     """
 
     def __init__(self, **kwargs):
@@ -104,6 +111,8 @@ class Group(System):
         self._transfers = {}
         self._discrete_transfers = {}
         self._approx_subjac_keys = None
+        self._setup_procs_finished = False
+        self._has_distrib_vars = False
 
         # TODO: we cannot set the solvers with property setters at the moment
         # because our lint check thinks that we are defining new attributes
@@ -290,11 +299,7 @@ class Group(System):
             if subsys.matrix_free:
                 self.matrix_free = True
 
-        self._static_mode = False
-        try:
-            self.configure()
-        finally:
-            self._static_mode = True
+        self.configure()
 
     def _setup_procs(self, pathname, comm, mode, prob_options):
         """
@@ -321,6 +326,8 @@ class Group(System):
         self.options._parent_name = self.msginfo
         self.recording_options._parent_name = self.msginfo
 
+        self._setup_procs_finished = False
+
         if self._num_par_fd > 1:
             info = self._coloring_info
             if comm.size > 1:
@@ -376,9 +383,7 @@ class Group(System):
             self._subsystems_myproc = [self._subsystems_allprocs[ind] for ind in sub_inds]
 
             # Define local subsystems
-            if np.sum([minp for minp, _, _ in proc_info]) <= comm.size:
-                self._subsystems_myproc_inds = sub_inds
-            else:
+            if not (np.sum([minp for minp, _, _ in proc_info]) <= comm.size):
                 # reorder the subsystems_allprocs based on which procs they live on. If we don't
                 # do this, we can get ordering mismatches in some of our data structures.
                 new_allsubs = []
@@ -390,19 +395,19 @@ class Group(System):
                             new_allsubs.append(self._subsystems_allprocs[ind])
                             seen.add(ind)
                 self._subsystems_allprocs = new_allsubs
-                sub_idxs = {s.name: i for i, s in enumerate(self._subsystems_allprocs)}
-
-                # since the subsystems_allprocs order changed, we also have to update
-                # subsystems_myproc_inds
-                self._subsystems_myproc_inds = [sub_idxs[s.name] for s in self._subsystems_myproc]
         else:
             sub_comm = comm
             self._subsystems_myproc = self._subsystems_allprocs
-            self._subsystems_myproc_inds = list(range(len(self._subsystems_myproc)))
             sub_proc_range = (0, 1)
 
         # Compute _subsystems_proc_range
         self._subsystems_proc_range = [sub_proc_range] * len(self._subsystems_myproc)
+        self._subsystems_inds = inds = {}
+
+        # need to set pathname correctly even for non-local subsystems
+        for i, s in enumerate(self._subsystems_allprocs):
+            inds[s.name] = i
+            s.pathname = '.'.join((self.pathname, s.name)) if self.pathname else s.name
 
         self._local_system_set = set()
 
@@ -414,18 +419,24 @@ class Group(System):
             subsys._use_derivatives = self._use_derivatives
             subsys._solver_info = self._solver_info
             subsys._recording_iter = self._recording_iter
-
-            if self.pathname:
-                subsys._setup_procs('.'.join((self.pathname, subsys.name)), sub_comm, mode,
-                                    prob_options)
-            else:
-                subsys._setup_procs(subsys.name, sub_comm, mode, prob_options)
+            subsys._setup_procs(subsys.pathname, sub_comm, mode, prob_options)
 
         # build a list of local subgroups to speed up later loops
         self._subgroups_myproc = [s for s in self._subsystems_myproc if isinstance(s, Group)]
 
         self._loc_subsys_map = {s.name: s for s in self._subsystems_myproc}
 
+        self._setup_procs_finished = True
+
+    def _post_configure(self):
+        """
+        Do any remaining setup that had to wait until after final user configuration.
+        """
+        for subsys in self._subsystems_myproc:
+            subsys._post_configure()
+
+        super(Group, self)._post_configure()
+
     def _check_child_reconf(self, subsys=None):
         """
         Check if any subsystem has reconfigured and if so, perform the necessary update setup.
@@ -517,9 +528,10 @@ class Group(System):
             allprocs_counters = {}
             for type_ in ['input', 'output']:
                 allprocs_counters[type_] = np.zeros(nsub_allprocs, INT_DTYPE)
-                for subsys, isub in zip(self._subsystems_myproc, self._subsystems_myproc_inds):
+                for subsys in self._subsystems_myproc:
                     comm = subsys.comm if subsys._full_comm is None else subsys._full_comm
                     if comm.rank == 0 and vec_name in subsys._rel_vec_names:
+                        isub = self._subsystems_inds[subsys.name]
                         allprocs_counters[type_][isub] = \
                             len(subsys._var_allprocs_relevant_names[vec_name][type_])
 
@@ -527,7 +539,8 @@ class Group(System):
             if self.comm.size > 1:
                 gathered = self.comm.allgather(allprocs_counters)
                 allprocs_counters = {
-                    type_: np.zeros(nsub_allprocs, INT_DTYPE) for type_ in ['input', 'output']}
+                    type_: np.zeros(nsub_allprocs, INT_DTYPE) for type_ in ['input', 'output']
+                }
                 for myproc_counters in gathered:
                     for type_ in ['input', 'output']:
                         allprocs_counters[type_] += myproc_counters[type_]
@@ -538,9 +551,10 @@ class Group(System):
             for type_ in ['input', 'output']:
                 subsystems_var_range[vec_name][type_] = {}
 
-                for subsys, isub in zip(self._subsystems_myproc, self._subsystems_myproc_inds):
+                for subsys in self._subsystems_myproc:
                     if vec_name not in subsys._rel_vec_names:
                         continue
+                    isub = self._subsystems_inds[subsys.name]
                     start = np.sum(allprocs_counters[type_][:isub])
                     subsystems_var_range[vec_name][type_][subsys.name] = (
                         start, start + allprocs_counters[type_][isub]
@@ -762,8 +776,8 @@ class Group(System):
                     sizes_in = sizes[type_][iproc, :].copy()
                     self.comm.Allgather(sizes_in, sizes[type_])
 
-            has_distrib_vars = self.comm.allreduce(n_distrib_vars) > 0
-            if (has_distrib_vars or not np.all(self._var_sizes[vec_names[0]]['output']) or
+            self._has_distrib_vars = self.comm.allreduce(n_distrib_vars) > 0
+            if (self._has_distrib_vars or not np.all(self._var_sizes[vec_names[0]]['output']) or
                     not np.all(self._var_sizes[vec_names[0]]['input'])):
                 if self._distributed_vector_class is not None:
                     self._vector_class = self._distributed_vector_class
@@ -781,7 +795,7 @@ class Group(System):
                     for rank in range(self.comm.size):
                         if sizes[rank, i] > 0:
                             owns[name] = rank
-                            if type_ is 'output' and not abs2meta[name]['distributed']:
+                            if type_ == 'output' and not abs2meta[name]['distributed']:
                                 self._owned_sizes[rank + 1:, i] = 0  # zero out all dups
                             break
 
@@ -798,7 +812,8 @@ class Group(System):
         if self._use_derivatives:
             self._var_sizes['nonlinear'] = self._var_sizes['linear']
 
-        self._setup_global_shapes()
+        if self.comm.size > 1:
+            self._setup_global_shapes()
 
     def _setup_global_connections(self, recurse=True, conns=None):
         """
@@ -866,15 +881,29 @@ class Group(System):
             # (not traceable to a connect statement, so provide context)
             if (prom_out not in allprocs_prom2abs_list_out and
                     prom_out not in self._var_allprocs_discrete['output']):
-                raise NameError(
-                    "%s: Output '%s' does not exist for connection in '%s' from '%s' to '%s'." %
-                    (self.msginfo, prom_out, self.pathname, prom_out, prom_in))
+                if (prom_out in allprocs_prom2abs_list_in or
+                        prom_out in self._var_allprocs_discrete['input']):
+                    raise NameError(
+                        "%s: Attempted to connect from '%s' to '%s', but '%s' is an input. "
+                        "All connections must be from an output to an input." %
+                        (self.msginfo, prom_out, prom_in, prom_out))
+                else:
+                    raise NameError(
+                        "%s: Attempted to connect from '%s' to '%s', but '%s' doesn't exist." %
+                        (self.msginfo, prom_out, prom_in, prom_out))
 
             if (prom_in not in allprocs_prom2abs_list_in and
                     prom_in not in self._var_allprocs_discrete['input']):
-                raise NameError(
-                    "%s: Input '%s' does not exist for connection from '%s' to '%s'." %
-                    (self.msginfo, prom_in, prom_out, prom_in))
+                if (prom_in in allprocs_prom2abs_list_out or
+                        prom_in in self._var_allprocs_discrete['output']):
+                    raise NameError(
+                        "%s: Attempted to connect from '%s' to '%s', but '%s' is an output. "
+                        "All connections must be from an output to an input." %
+                        (self.msginfo, prom_out, prom_in, prom_in))
+                else:
+                    raise NameError(
+                        "%s: Attempted to connect from '%s' to '%s', but '%s' doesn't exist." %
+                        (self.msginfo, prom_out, prom_in, prom_in))
 
             # Throw an exception if output and input are in the same system
             # (not traceable to a connect statement, so provide context)
@@ -916,12 +945,16 @@ class Group(System):
 
         # Recursion
         if recurse:
-            for subsys in self._subgroups_myproc:
-                if subsys.name in new_conns:
-                    subsys._setup_global_connections(recurse=recurse,
-                                                     conns=new_conns[subsys.name])
-                else:
-                    subsys._setup_global_connections(recurse=recurse)
+            distcomps = []
+            for subsys in self._subsystems_myproc:
+                if isinstance(subsys, Group):
+                    if subsys.name in new_conns:
+                        subsys._setup_global_connections(recurse=recurse,
+                                                         conns=new_conns[subsys.name])
+                    else:
+                        subsys._setup_global_connections(recurse=recurse)
+                elif subsys.options['distributed'] and subsys.comm.size > 1:
+                    distcomps.append(subsys)
 
         # Compute global_abs_in2out by first adding this group's contributions,
         # then adding contributions from systems above/below, then allgathering.
@@ -957,9 +990,13 @@ class Group(System):
             for myproc_global_abs_in2out in gathered:
                 global_abs_in2out.update(myproc_global_abs_in2out)
 
+            if recurse:
+                for comp in distcomps:
+                    comp._update_dist_src_indices(global_abs_in2out)
+
     def _setup_connections(self, recurse=True):
         """
-        Compute dict of all implicit and explicit connections owned by this Group.
+        Compute dict of all connections owned by this Group.
 
         Parameters
         ----------
@@ -1179,15 +1216,17 @@ class Group(System):
                             # when running under MPI, there is a value for each proc
                             d_size = out_shape[d] * self.comm.size
                             if src_indices.size > 0:
-                                for i in src_indices[..., d].flat:
-                                    if abs(i) >= d_size:
-                                        msg = ("%s: The source indices do not specify "
-                                               "a valid index for the connection "
-                                               "'%s' to '%s'. Index "
-                                               "'%d' is out of range for source "
-                                               "dimension of size %d.")
-                                        raise ValueError(msg % (self.msginfo, abs_out, abs_in, i,
-                                                                d_size))
+                                arr = src_indices[..., d]
+                                if np.any(arr >= d_size) or np.any(arr <= -d_size):
+                                    for i in arr.flat:
+                                        if abs(i) >= d_size:
+                                            msg = ("%s: The source indices do not specify "
+                                                   "a valid index for the connection "
+                                                   "'%s' to '%s'. Index "
+                                                   "'%d' is out of range for source "
+                                                   "dimension of size %d.")
+                                            raise ValueError(msg % (self.msginfo, abs_out, abs_in,
+                                                                    i, d_size))
 
     def _transfer(self, vec_name, mode, isub=None):
         """
@@ -1348,6 +1387,42 @@ class Group(System):
         if self._conn_discrete_in2out:
             self._vector_class.TRANSFER._setup_discrete_transfers(self, recurse=recurse)
 
+    def promotes(self, subsys_name, any=None, inputs=None, outputs=None):
+        """
+        Promote a variable in the model tree.
+
+        Parameters
+        ----------
+        subsys_name : str
+            The name of the child subsystem whose inputs/outputs are being promoted.
+        any : Sequence of str or tuple
+            A Sequence of variable names (or tuples) to be promoted, regardless
+            of if they are inputs or outputs. This is equivalent to the items
+            passed via the `promotes=` argument to add_subsystem.  If given as a
+            tuple, we use the "promote as" standard of ('real name', 'promoted name')*[]:
+        inputs : Sequence of str or tuple
+            A Sequence of input names (or tuples) to be promoted. Tuples are
+            used for the "promote as" capability.
+        outputs : Sequence of str or tuple
+            A Sequence of output names (or tuples) to be promoted. Tuples are
+            used for the "promote as" capability.
+        """
+        subsys = getattr(self, subsys_name)
+        if any:
+            subsys._var_promotes['any'].extend(any)
+        if inputs:
+            subsys._var_promotes['input'].extend(inputs)
+        if outputs:
+            subsys._var_promotes['output'].extend(outputs)
+
+        list_comp = [i if isinstance(i, tuple) else (i, i) for i in subsys._var_promotes['input']]
+
+        for original, new in list_comp:
+            for original_inside, new_inside in list_comp:
+                if original == original_inside and new != new_inside:
+                    raise RuntimeError("%s: Trying to promote '%s' when it has been aliased to "
+                                       "'%s'." % (self.msginfo, original_inside, new))
+
     def add(self, name, subsys, promotes=None):
         """
         Add a subsystem (deprecated version of <Group.add_subsystem>).
@@ -1416,6 +1491,10 @@ class Group(System):
             enable users to instantiate and add a subsystem at the
             same time, and get the reference back.
         """
+        if self._setup_procs_finished:
+            raise RuntimeError("%s: Cannot call add_subsystem in "
+                               "the configure method" % (self.msginfo))
+
         if inspect.isclass(subsys):
             raise TypeError("%s: Subsystem '%s' should be an instance, but a %s class object was "
                             "found." % (self.msginfo, name, subsys.__name__))
@@ -1717,6 +1796,7 @@ class Group(System):
             jac = self._jacobian
         elif jac is None and self._assembled_jac is not None:
             jac = self._assembled_jac
+
         if jac is not None:
             for vec_name in vec_names:
                 with self._matvec_context(vec_name, scope_out, scope_in, mode) as vecs:
