@@ -434,13 +434,21 @@ class ImplicitComponent(Component):
         """
         Return list of all states at and below this system.
 
+        If final setup has not been performed yet, return relative names for this system only.
+
         Returns
         -------
         list
             List of all states.
         """
-        return [name for name in self._outputs._names] + \
-               [name for name in self._var_allprocs_discrete['output']]
+        if self._outputs is not None:
+            # final setup has been performed, return absolute names
+            return [name for name in self._outputs._names] + \
+                   [name for name in self._var_allprocs_discrete['output']]
+        else:
+            # final setup has not been performed, return relative names for this system only
+            return [name for name in self._var_rel_names['output']] + \
+                   [name for name in self._var_discrete['output']]
 
     def _list_states_allprocs(self):
         """
