@@ -2,7 +2,11 @@
 
 from __future__ import division
 
-import collections
+# note: this is a Python 3.3 change, clean this up for OpenMDAO 3.x
+try:
+    from collections.abc import Iterable
+except ImportError:
+    from collections import Iterable
 
 from six import string_types
 
@@ -57,7 +61,7 @@ class IndepVarComp(ExplicitComponent):
         if isinstance(name, string_types):
             self._indep.append((name, val, kwargs))
         # Mutiple variables are declared during instantiation (deprecated)
-        elif isinstance(name, collections.Iterable):
+        elif isinstance(name, Iterable):
             warn_deprecation('Declaring multiple variables in this way is deprecated. '
                              'In OpenMDAO 2.x or later, multiple variables should be declared '
                              'as separate add_output calls.')
@@ -89,10 +93,16 @@ class IndepVarComp(ExplicitComponent):
                 raise ValueError("IndepVarComp init: '%s' is not supported "
                                  "in IndepVarComp." % illegal)
 
-    def setup(self):
+    def _post_configure(self):
         """
-        Define the independent variables as output variables.
+        Do any remaining setup that had to wait until after final user configuration.
         """
+        # set static mode to False because we are doing things that would normally be done in setup
+        self._static_mode = False
+
+        self._var_rel_names = {'input': [], 'output': []}
+        self._var_rel2meta = {}
+
         for (name, val, kwargs) in self._indep + self._indep_external:
             super(IndepVarComp, self).add_output(name, val, **kwargs)
 
@@ -105,6 +115,10 @@ class IndepVarComp(ExplicitComponent):
                                "instantiation or by calling add_output or add_discrete_output "
                                "afterwards.".format(self.msginfo))
 
+        self._static_mode = True
+
+        super(IndepVarComp, self)._post_configure()
+
     def add_output(self, name, val=1.0, shape=None, units=None, res_units=None, desc='',
                    lower=None, upper=None, ref=1.0, ref0=0.0, res_ref=None, tags=None):
         """
