@@ -18,10 +18,9 @@ import numpy as np
 import scipy.sparse as sparse
 
 from openmdao.core.component import Component
-from openmdao.core.driver import Driver
+from openmdao.core.driver import Driver, record_iteration
 from openmdao.core.explicitcomponent import ExplicitComponent
-from openmdao.core.group import Group
-from openmdao.core.group import System
+from openmdao.core.group import Group, System
 from openmdao.core.indepvarcomp import IndepVarComp
 from openmdao.core.total_jac import _TotalJacInfo
 from openmdao.approximation_schemes.complex_step import ComplexStep
@@ -39,9 +38,11 @@ from openmdao.utils.options_dictionary import OptionsDictionary
 from openmdao.utils.units import get_conversion
 from openmdao.utils import coloring as coloring_mod
 from openmdao.utils.name_maps import abs_key2rel_key
+from openmdao.vectors.vector import INT_DTYPE
 from openmdao.vectors.default_vector import DefaultVector
 from openmdao.utils.logger_utils import get_logger, TestLogger
 import openmdao.utils.coloring as coloring_mod
+from openmdao.utils.hooks import _setup_hooks
 
 try:
     from openmdao.vectors.petsc_vector import PETScVector
@@ -113,8 +114,6 @@ class Problem(object):
         Dictionary with problem recording options.
     _rec_mgr : <RecordingManager>
         Object that manages all recorders added to this problem.
-    _remote_var_set : set
-        Set of variables (absolute names) that require remote data transfer to reach all procs.
     _check : bool
         If True, call check_config at the end of final_setup.
     _recording_iter : _RecIteration
@@ -127,15 +126,11 @@ class Problem(object):
         Force allocation of imaginary part in nonlinear vectors. OpenMDAO can generally
         detect when you need to do this, but in some cases (e.g., complex step is used
         after a reconfiguration) you may need to set this to True.
-    _color_dir_hash : str
-        Hash used to detect collisions of coloring files.
-    __get_remote : bool
-        Flag used to determine when __getitem__ will retrieve remote variables.
+    _name : str
+        Problem name.
     """
 
-    _post_setup_func = None
-
-    def __init__(self, model=None, driver=None, comm=None, root=None, **options):
+    def __init__(self, model=None, driver=None, comm=None, root=None, name=None, **options):
         """
         Initialize attributes.
 
@@ -149,13 +144,14 @@ class Problem(object):
             The global communicator.
         root : <System> or None
             Deprecated kwarg for `model`.
+        name : str
+            Problem name. Can be used to specify a Problem instance when multiple Problems
+            exist.
         **options : named args
             All remaining named args are converted to options.
         """
-        # used by the get_val function when get_remote is True
-        self.__get_remote = False
-
         self.cite = CITATION
+        self._name = name
 
         if comm is None:
             try:
@@ -166,7 +162,7 @@ class Problem(object):
 
         if root is not None:
             if model is not None:
-                raise ValueError("Cannot specify both 'root' and 'model'. "
+                raise ValueError(self.msginfo + ": Cannot specify both 'root' and 'model'. "
                                  "'root' has been deprecated, please use 'model'.")
 
             warn_deprecation("The 'root' argument provides backwards compatibility "
@@ -179,14 +175,16 @@ class Problem(object):
         elif isinstance(model, System):
             self.model = model
         else:
-            raise TypeError("The value provided for 'model' is not a valid System.")
+            raise TypeError(self.msginfo +
+                            ": The value provided for 'model' is not a valid System.")
 
         if driver is None:
             self.driver = Driver()
         elif isinstance(driver, Driver):
             self.driver = driver
         else:
-            raise TypeError("The value provided for 'driver' is not a valid Driver.")
+            raise TypeError(self.msginfo +
+                            ": The value provided for 'driver' is not a valid Driver.")
 
         self.comm = comm
 
@@ -222,11 +220,17 @@ class Problem(object):
         self.recording_options.declare('record_constraints', types=bool, default=True,
                                        desc='Set to True to record constraints at the '
                                             'problem level')
+        self.recording_options.declare('record_responses', types=bool, default=False,
+                                       desc='Set True to record constraints and objectives at the '
+                                            'problem level.')
         self.recording_options.declare('includes', types=list, default=['*'],
-                                       desc='Patterns for variables to include in recording')
+                                       desc='Patterns for variables to include in recording. \
+                                       Uses fnmatch wildcards')
         self.recording_options.declare('excludes', types=list, default=[],
                                        desc='Patterns for vars to exclude in recording '
-                                            '(processed post-includes)')
+                                            '(processed post-includes). Uses fnmatch wildcards')
+
+        _setup_hooks(self)
 
     def _get_var_abs_name(self, name):
         if name in self.model._var_allprocs_abs2meta:
@@ -238,11 +242,29 @@ class Problem(object):
             if len(abs_names) == 1:
                 return abs_names[0]
             else:
-                raise KeyError("Using promoted name `{}' is ambiguous and matches unconnected "
-                               "inputs %s. Use absolute name to disambiguate.".format(name,
+                raise KeyError("{}: Using promoted name `{}' is ambiguous and matches unconnected "
+                               "inputs %s. Use absolute name to disambiguate.".format(self.msginfo,
+                                                                                      name,
                                                                                       abs_names))
 
-        raise KeyError("Variable '{}' not found.".format(name))
+        raise KeyError('{}: Variable "{}" not found.'.format(self.msginfo, name))
+
+    @property
+    def msginfo(self):
+        """
+        Return info to prepend to messages.
+
+        Returns
+        -------
+        str
+            Info to prepend to messages.
+        """
+        if self._name is None:
+            return type(self).__name__
+        return '{} {}'.format(type(self).__name__, self._name)
+
+    def _get_inst_id(self):
+        return self._name
 
     def is_local(self, name):
         """
@@ -259,7 +281,8 @@ class Problem(object):
             True if the named system or variable is local to this process.
         """
         if self._setup_status < 1:
-            raise RuntimeError("is_local('{}') was called before setup() completed.".format(name))
+            raise RuntimeError("{}: is_local('{}') was called before setup() "
+                               "completed.".format(self.msginfo, name))
 
         try:
             abs_name = self._get_var_abs_name(name)
@@ -275,6 +298,55 @@ class Problem(object):
         # variable exists, but may be remote
         return abs_name in self.model._var_abs2meta
 
+    def _get_cached_val(self, name):
+        # We have set and cached already
+        if name in self._initial_condition_cache:
+            return self._initial_condition_cache[name]
+
+        # Vector not setup, so we need to pull values from saved metadata request.
+        else:
+            proms = self.model._var_allprocs_prom2abs_list
+            meta = self.model._var_abs2meta
+            if name in meta:
+                if isinstance(self.model, Group) and name in self.model._conn_abs_in2out:
+                    src_name = self.model._conn_abs_in2out[name]
+                    val = meta[src_name]['value']
+                else:
+                    val = meta[name]['value']
+
+            elif name in proms['output']:
+                abs_name = prom_name2abs_name(self.model, name, 'output')
+                if abs_name in meta:
+                    val = meta[abs_name]['value']
+
+            elif name in proms['input']:
+                abs_name = proms['input'][name][0]
+                conn = self.model._conn_abs_in2out
+                if abs_name in meta:
+                    if isinstance(self.model, Group) and abs_name in conn:
+                        src_name = self.model._conn_abs_in2out[abs_name]
+                        # So, if the inputs and outputs are promoted to the same name, then we
+                        # allow getitem, but if they aren't, then we raise an error due to non
+                        # uniqueness.
+                        if name not in proms['output']:
+                            # This triggers a check for unconnected non-unique inputs, and
+                            # raises the same error as vector access.
+                            abs_name = prom_name2abs_name(self.model, name, 'input')
+                        val = meta[src_name]['value']
+                    else:
+                        # This triggers a check for unconnected non-unique inputs, and
+                        # raises the same error as vector access.
+                        abs_name = prom_name2abs_name(self.model, name, 'input')
+                        val = meta[abs_name]['value']
+            else:
+                raise KeyError('{}: Variable name "{}" not found.'.format(self.msginfo, name))
+
+            if val is not _undefined:
+                # Need to cache the "get" in case the user calls in-place numpy operations.
+                self._initial_condition_cache[name] = val
+
+        return val
+
     def __getitem__(self, name):
         """
         Get an output/input variable.
@@ -289,113 +361,7 @@ class Problem(object):
         float or ndarray or any python object
             the requested output/input variable.
         """
-        # Caching only needed if vectors aren't allocated yet.
-        proms = self.model._var_allprocs_prom2abs_list
-        meta = self.model._var_abs2meta
-
-        val = _undefined
-        abs_name = None
-
-        if self._setup_status == 1:
-
-            # We have set and cached already
-            if name in self._initial_condition_cache:
-                return self._initial_condition_cache[name]
-
-            # Vector not setup, so we need to pull values from saved metadata request.
-            else:
-                if name in meta:
-                    if isinstance(self.model, Group) and name in self.model._conn_abs_in2out:
-                        src_name = self.model._conn_abs_in2out[name]
-                        val = meta[src_name]['value']
-                    else:
-                        val = meta[name]['value']
-
-                elif name in proms['output']:
-                    abs_name = prom_name2abs_name(self.model, name, 'output')
-                    if abs_name in meta:
-                        val = meta[abs_name]['value']
-
-                elif name in proms['input']:
-                    abs_name = proms['input'][name][0]
-                    conn = self.model._conn_abs_in2out
-                    if abs_name in meta:
-                        if isinstance(self.model, Group) and abs_name in conn:
-                            src_name = self.model._conn_abs_in2out[abs_name]
-                            # So, if the inputs and outputs are promoted to the same name, then we
-                            # allow getitem, but if they aren't, then we raise an error due to non
-                            # uniqueness.
-                            if name not in proms['output']:
-                                # This triggers a check for unconnected non-unique inputs, and
-                                # raises the same error as vector access.
-                                abs_name = prom_name2abs_name(self.model, name, 'input')
-                            val = meta[src_name]['value']
-                        else:
-                            # This triggers a check for unconnected non-unique inputs, and
-                            # raises the same error as vector access.
-                            abs_name = prom_name2abs_name(self.model, name, 'input')
-                            val = meta[abs_name]['value']
-
-                if val is not _undefined:
-                    # Need to cache the "get" in case the user calls in-place numpy operations.
-                    self._initial_condition_cache[name] = val
-
-        else:
-            if name in proms['output']:
-                name = proms['output'][name][0]
-            elif name in proms['input']:
-                name = prom_name2abs_name(self.model, name, 'input')
-
-            if name in meta:   # local var
-                if name in self.model._outputs._views:
-                    val = self.model._outputs[name]
-                else:
-                    val = self.model._inputs[name]
-
-            elif name in self.model._discrete_outputs:
-                val = self.model._discrete_outputs[name]
-
-            elif name in self.model._discrete_inputs:
-                val = self.model._discrete_inputs[name]
-
-        if self.model.comm.size > 1:
-            allprocs_meta = self.model._var_allprocs_abs2meta
-            # check for remote var
-            if name in allprocs_meta:
-                abs_name = name
-            elif name in proms['output']:
-                abs_name = proms['output'][name][0]
-            elif name in proms['input']:
-                abs_name = proms['input'][name][0]
-
-            if abs_name in self._remote_var_set:
-                if self.__get_remote:
-                    if abs_name in allprocs_meta and allprocs_meta[abs_name]['distributed']:
-                        raise RuntimeError("Retrieval of the full distributed variable '%s' is not "
-                                           "supported." % abs_name)
-                    loc_val = val
-                    owner = self.model._owning_rank[abs_name]
-                    if owner != self.model.comm.rank:
-                        val = None
-                    else:
-                        owner = self.model._owning_rank[abs_name]
-                        if val is _undefined:
-                            val = None
-                    new_val = self.model.comm.bcast(val, root=owner)
-                    if loc_val is not None and loc_val is not _undefined:
-                        val = loc_val
-                    else:
-                        val = new_val
-                elif val is _undefined:
-                    raise RuntimeError(
-                        "Variable '{}' is not local to rank {}. You can retrieve values from "
-                        "other processes using "
-                        "`problem.get_val(<name>, get_remote=True)`.".format(name, self.comm.rank))
-
-        if val is _undefined:
-            raise KeyError('Variable name "{}" not found.'.format(name))
-
-        return val
+        return self.get_val(name)
 
     def get_val(self, name, units=None, indices=None, get_remote=False):
         """
@@ -408,7 +374,7 @@ class Problem(object):
         name : str
             Promoted or relative variable name in the root system's namespace.
         units : str, optional
-            Units to convert to before upon return.
+            Units to convert to before return.
         indices : int or list of ints or tuple of ints or int ndarray or Iterable or None, optional
             Indices or slice to return.
         get_remote : bool
@@ -418,70 +384,31 @@ class Problem(object):
 
         Returns
         -------
-        float or ndarray
-            The requested output/input variable.
+        object
+            The value of the requested output/input variable.
         """
-        if get_remote:
-            self.__get_remote = True
-            try:
-                val = self[name]
-            finally:
-                self.__get_remote = False
-        else:
-            val = self[name]
+        if self._setup_status == 1:
+            val = self._get_cached_val(name)
+            if indices is not None:
+                val = val[indices]
+            if units is not None:
+                val = self.model.convert2units(name, val, units)
+            return val
 
-        if indices is not None:
-            val = val[indices]
+        val = self.model._get_val(name, units=units, indices=indices, get_remote=get_remote)
 
-        if units is not None:
-            base_units = self._get_units(name)
-
-            if base_units is None:
-                msg = "Can't express variable '{}' with units of 'None' in units of '{}'."
-                raise TypeError(msg.format(name, units))
-
-            try:
-                scale, offset = get_conversion(base_units, units)
-            except TypeError:
-                msg = "Can't express variable '{}' with units of '{}' in units of '{}'."
-                raise TypeError(msg.format(name, base_units, units))
-
-            val = (val + offset) * scale
+        if val is System._undefined:
+            if get_remote:
+                raise KeyError('{}: Variable name "{}" not found.'.format(self.msginfo, name))
+            else:
+                raise RuntimeError(
+                    "{}: Variable '{}' is not local to rank {}. You can retrieve values from "
+                    "other processes using "
+                    "`problem.get_val(<name>, get_remote=True)`.".format(self.msginfo, name,
+                                                                         self.comm.rank))
 
         return val
 
-    def _get_units(self, name):
-        """
-        Get the units for a variable name.
-
-        Parameters
-        ----------
-        name : str
-            Promoted or relative variable name in the root system's namespace.
-
-        Returns
-        -------
-        str
-            Unit string.
-        """
-        meta = self.model._var_allprocs_abs2meta
-        if name in meta:
-            return meta[name]['units']
-
-        proms = self.model._var_allprocs_prom2abs_list
-        if self._setup_status >= 1:
-
-            if name in proms['output']:
-                abs_name = prom_name2abs_name(self.model, name, 'output')
-                return meta[abs_name]['units']
-            elif name in proms['input']:
-                # This triggers a check for unconnected non-unique inputs, and
-                # raises the same error as vector access.
-                abs_name = prom_name2abs_name(self.model, name, 'input')
-                return meta[abs_name]['units']
-
-        raise KeyError('Variable name "{}" not found.'.format(name))
-
     def __setitem__(self, name, value):
         """
         Set an output/input variable.
@@ -520,7 +447,7 @@ class Problem(object):
                     print("Variable '{}' is remote on rank {}.  "
                           "Local assignment ignored.".format(name, self.comm.rank))
                 else:
-                    raise KeyError('Variable name "{}" not found.'.format(name))
+                    raise KeyError('{}: Variable "{}" not found.'.format(self.model.msginfo, name))
 
     def set_val(self, name, value, units=None, indices=None):
         """
@@ -540,17 +467,17 @@ class Problem(object):
             Indices or slice to set to specified value.
         """
         if units is not None:
-            base_units = self._get_units(name)
+            base_units = self.model._get_var_meta(name)['units']
 
             if base_units is None:
-                msg = "Can't set variable '{}' with units of 'None' to value with units of '{}'."
-                raise TypeError(msg.format(name, units))
+                msg = "{}: Can't set variable '{}' with units 'None' to value with units '{}'."
+                raise TypeError(msg.format(self.msginfo, name, units))
 
             try:
                 scale, offset = get_conversion(units, base_units)
             except TypeError:
-                msg = "Can't set variable '{}' with units of '{}' to value with units of '{}'."
-                raise TypeError(msg.format(name, base_units, units))
+                msg = "{}: Can't set variable '{}' with units '{}' to value with units '{}'."
+                raise TypeError(msg.format(self.msginfo, name, base_units, units))
 
             value = (value + offset) * scale
 
@@ -610,11 +537,12 @@ class Problem(object):
             If True and model has been run previously, reset all iteration counters.
         """
         if self._mode is None:
-            raise RuntimeError("The `setup` method must be called before `run_model`.")
+            raise RuntimeError(self.msginfo +
+                               ": The `setup` method must be called before `run_model`.")
 
         if case_prefix:
             if not isinstance(case_prefix, str):
-                raise TypeError("The 'case_prefix' argument should be a string.")
+                raise TypeError(self.msginfo + ": The 'case_prefix' argument should be a string.")
             self._recording_iter.prefix = case_prefix
         else:
             self._recording_iter.prefix = None
@@ -645,11 +573,12 @@ class Problem(object):
             Failure flag; True if failed to converge, False is successful.
         """
         if self._mode is None:
-            raise RuntimeError("The `setup` method must be called before `run_driver`.")
+            raise RuntimeError(self.msginfo +
+                               ": The `setup` method must be called before `run_driver`.")
 
         if case_prefix:
             if not isinstance(case_prefix, str):
-                raise TypeError("The 'case_prefix' argument should be a string.")
+                raise TypeError(self.msginfo + ": The 'case_prefix' argument should be a string.")
             self._recording_iter.prefix = case_prefix
         else:
             self._recording_iter.prefix = None
@@ -686,12 +615,14 @@ class Problem(object):
         """
         if mode == 'fwd':
             if len(wrt) != len(seed):
-                raise RuntimeError("seed and 'wrt' list must be the same length in fwd mode.")
+                raise RuntimeError(self.msginfo +
+                                   ": seed and 'wrt' list must be the same length in fwd mode.")
             lnames, rnames = of, wrt
             lkind, rkind = 'output', 'residual'
         else:  # rev
             if len(of) != len(seed):
-                raise RuntimeError("seed and 'of' list must be the same length in rev mode.")
+                raise RuntimeError(self.msginfo +
+                                   ": seed and 'of' list must be the same length in rev mode.")
             lnames, rnames = wrt, of
             lkind, rkind = 'residual', 'output'
 
@@ -746,7 +677,6 @@ class Problem(object):
         Set up case recording.
         """
         self._filtered_vars_to_record = self.driver._get_vars_to_record(self.recording_options)
-
         self._rec_mgr.startup(self)
 
     def add_recorder(self, recorder):
@@ -781,59 +711,23 @@ class Problem(object):
         case_name : str
             Name used to identify this Problem case.
         """
-        if not self._rec_mgr._recorders:
-            return
+        record_iteration(self, self, case_name)
 
-        # Get the data to record (collective calls that get across all ranks)
-        opts = self.recording_options
-        filt = self._filtered_vars_to_record
+    def _get_recorder_metadata(self, case_name):
+        """
+        Return metadata from the latest iteration for use in the recorder.
 
-        model = self.model
-        driver = self.driver
+        Parameters
+        ----------
+        case_name : str
+            Name of current case.
 
-        if opts['record_desvars']:
-            des_vars = driver.get_design_var_values()
-        else:
-            des_vars = {}
-
-        if opts['record_objectives']:
-            obj_vars = driver.get_objective_values()
-        else:
-            obj_vars = {}
-
-        if opts['record_constraints']:
-            con_vars = driver.get_constraint_values()
-        else:
-            con_vars = {}
-
-        des_vars = {name: des_vars[name] for name in filt['des']}
-        obj_vars = {name: obj_vars[name] for name in filt['obj']}
-        con_vars = {name: con_vars[name] for name in filt['con']}
-
-        names = model._outputs._names
-        views = model._outputs._views
-        sys_vars = {name: views[name] for name in names if name in filt['sys']}
-
-        if MPI:
-            des_vars = driver._gather_vars(model, des_vars)
-            obj_vars = driver._gather_vars(model, obj_vars)
-            con_vars = driver._gather_vars(model, con_vars)
-            sys_vars = driver._gather_vars(model, sys_vars)
-
-        outs = {}
-        if not MPI or model.comm.rank == 0:
-            outs.update(des_vars)
-            outs.update(obj_vars)
-            outs.update(con_vars)
-            outs.update(sys_vars)
-
-        data = {
-            'out': outs,
-        }
-
-        metadata = create_local_meta(case_name)
-
-        self._rec_mgr.record_iteration(self, data, metadata)
+        Returns
+        -------
+        dict
+            Metadata dictionary for the recorder.
+        """
+        return create_local_meta(case_name)
 
     def setup(self, vector_class=None, check=False, logger=None, mode='auto',
               force_alloc_complex=False, distributed_vector_class=PETScVector,
@@ -890,15 +784,16 @@ class Problem(object):
         # PETScVector is required for MPI
         if comm.size > 1:
             if PETScVector is None:
-                raise ValueError("Attempting to run in parallel under MPI but PETScVector could not"
-                                 "be imported.")
+                raise ValueError(self.msginfo +
+                                 ": Attempting to run in parallel under MPI but PETScVector "
+                                 "could not be imported.")
             elif distributed_vector_class is not PETScVector:
-                raise ValueError("The `distributed_vector_class` argument must be `PETScVector` "
-                                 "when running in parallel under MPI but '%s' was specified."
-                                 % distributed_vector_class.__name__)
+                raise ValueError("%s: The `distributed_vector_class` argument must be "
+                                 "`PETScVector` when running in parallel under MPI but '%s' was "
+                                 "specified." % (self.msginfo, distributed_vector_class.__name__))
 
         if mode not in ['fwd', 'rev', 'auto']:
-            msg = "Unsupported mode: '%s'. Use either 'fwd' or 'rev'." % mode
+            msg = "%s: Unsupported mode: '%s'. Use either 'fwd' or 'rev'." % (self.msginfo, mode)
             raise ValueError(msg)
 
         self._mode = self._orig_mode = mode
@@ -913,34 +808,6 @@ class Problem(object):
         model._setup(model_comm, 'full', mode, distributed_vector_class, local_vector_class,
                      derivatives, self.options)
 
-        # get set of all vars that we may need to bcast later
-        self._remote_var_set = remote_var_set = set()
-        if model_comm.size > 1:
-            for type_ in ('input', 'output'):
-                remote_discrete = set()
-                sizes = self.model._var_sizes['nonlinear'][type_]
-                for i, vname in enumerate(self.model._var_allprocs_abs_names[type_]):
-                    if not np.all(sizes[:, i]):
-                        remote_var_set.add(vname)
-
-                if self.model._var_allprocs_discrete[type_]:
-                    local = list(self.model._var_discrete[type_])
-                    byrank = self.comm.gather(local, root=0)
-                    if model_comm.rank == 0:
-                        full = set()
-                        for names in byrank:
-                            full.update(names)
-
-                        for names in byrank:
-                            diff = full.difference(names)
-                            remote_discrete.update(diff)
-
-                        model_comm.bcast(remote_discrete, root=0)
-                    else:
-                        remote_discrete = model_comm.bcast(None, root=0)
-
-                    remote_var_set.update(remote_discrete)
-
         # Cache all args for final setup.
         self._check = check
         self._logger = logger
@@ -1010,6 +877,10 @@ class Problem(object):
             self.set_solver_print(level=items[0], depth=items[1], type_=items[2])
         self._solver_print_cache = []
 
+        if self._setup_status < 2:
+            self._setup_status = 2
+            self._set_initial_conditions()
+
         if self._check:
             if self._check is True:
                 checks = _default_checks
@@ -1021,14 +892,6 @@ class Problem(object):
                 logger = TestLogger()
             self.check_config(logger, checks=checks)
 
-        if self._setup_status < 2:
-            self._setup_status = 2
-            self._set_initial_conditions()
-
-        # check for post-setup hook
-        if Problem._post_setup_func is not None:
-            Problem._post_setup_func(self)
-
     def check_partials(self, out_stream=_DEFAULT_OUT_STREAM, includes=None, excludes=None,
                        compact_print=False, abs_err_tol=1e-6, rel_err_tol=1e-6,
                        method='fd', step=None, form='forward', step_calc='abs',
@@ -1093,7 +956,8 @@ class Problem(object):
         model = self.model
 
         if not model._use_derivatives:
-            raise RuntimeError("Can't check partials.  Derivative support has been turned off.")
+            raise RuntimeError(self.msginfo +
+                               ": Can't check partials.  Derivative support has been turned off.")
 
         # TODO: Once we're tracking iteration counts, run the model if it has not been run before.
 
@@ -1451,12 +1315,14 @@ class Problem(object):
                 forward - fd, adjoint - fd, forward - adjoint.
         """
         if self._setup_status < 2:
-            raise RuntimeError("run_model must be called before total derivatives can be checked.")
+            raise RuntimeError(self.msginfo + ": run_model must be called before total "
+                               "derivatives can be checked.")
 
         model = self.model
 
         if method == 'cs' and not model._outputs._alloc_complex:
-            msg = "\nTo enable complex step, specify 'force_alloc_complex=True' when calling " + \
+            msg = "\n" + self.msginfo + ": To enable complex step, specify "\
+                  "'force_alloc_complex=True' when calling " + \
                   "setup on the problem, e.g. 'problem.setup(force_alloc_complex=True)'"
             raise RuntimeError(msg)
 
@@ -1484,7 +1350,7 @@ class Problem(object):
         old_subjacs = model._subjacs_info.copy()
 
         model.approx_totals(method=method, step=step, form=form,
-                            step_calc=step_calc if method is 'fd' else None)
+                            step_calc=step_calc if method == 'fd' else None)
         total_info = _TotalJacInfo(self, of, wrt, False, return_format='flat_dict', approx=True,
                                    driver_scaling=driver_scaling)
         Jfd = total_info.compute_totals_approx(initialize=True)
@@ -1749,16 +1615,16 @@ class Problem(object):
         if inputs:
             for name in inputs.absolute_names():
                 if name not in self.model._var_abs_names['input']:
-                    raise KeyError("Input variable, '{}', recorded in the case is not "
-                                   "found in the model".format(name))
+                    raise KeyError("{}: Input variable, '{}', recorded in the case is not "
+                                   "found in the model".format(self.msginfo, name))
                 self[name] = inputs[name]
 
         outputs = case.outputs if case.outputs is not None else None
         if outputs:
             for name in outputs.absolute_names():
                 if name not in self.model._var_abs_names['output']:
-                    raise KeyError("Output variable, '{}', recorded in the case is not "
-                                   "found in the model".format(name))
+                    raise KeyError("{}: Output variable, '{}', recorded in the case is not "
+                                   "found in the model".format(self.msginfo, name))
                 self[name] = outputs[name]
 
         return
