@@ -334,7 +334,7 @@ class TestGroupFiniteDifference(unittest.TestCase):
         model.add_subsystem('comp', TestImplCompArrayDenseNoSolve())
         model.connect('p_rhs.rhs', 'comp.rhs')
 
-        model.nonlinear_solver = om.NewtonSolver()
+        model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         model.linear_solver = om.ScipyKrylov()
         model.approx_totals()
 
@@ -620,11 +620,12 @@ class TestGroupFiniteDifference(unittest.TestCase):
 
         prob = om.Problem()
         model = prob.model = om.Group(assembled_jac_type='dense')
-        sub = model.add_subsystem('sub', om.Group(), promotes=['*'])
 
         model.add_subsystem('px', om.IndepVarComp('x', 1.0), promotes=['x'])
         model.add_subsystem('pz', om.IndepVarComp('z', np.array([5.0, 2.0])), promotes=['z'])
 
+        sub = model.add_subsystem('sub', om.Group(), promotes=['*'])
+
         sub.add_subsystem('d1', SellarDis1withDerivatives(), promotes=['x', 'z', 'y1', 'y2'])
         sub.add_subsystem('d2', SellarDis2withDerivatives(), promotes=['z', 'y1', 'y2'])
 
@@ -635,7 +636,7 @@ class TestGroupFiniteDifference(unittest.TestCase):
         model.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
         model.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
-        sub.nonlinear_solver = om.NewtonSolver()
+        sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         sub.linear_solver = om.ScipyKrylov(assemble_jac=True)
 
         model.approx_totals(method='fd', step=1e-5)
@@ -674,7 +675,7 @@ class TestGroupFiniteDifference(unittest.TestCase):
         model.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
         model.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
-        sub.nonlinear_solver = om.NewtonSolver()
+        sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         sub.linear_solver = om.ScipyKrylov(assemble_jac=True)
 
         model.approx_totals(method='fd', step=1e-5)
@@ -812,7 +813,7 @@ class TestGroupFiniteDifference(unittest.TestCase):
         assert_rel_error(self, p['circle.area'], np.pi, 1e-6)
 
 
-@unittest.skipIf(MPI and not PETScVector, "only run under MPI if we have PETSc.")
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
 class TestGroupFiniteDifferenceMPI(unittest.TestCase):
 
     N_PROCS = 2
@@ -831,7 +832,7 @@ class TestGroupFiniteDifferenceMPI(unittest.TestCase):
         assert_rel_error(self, J['sum.y', 'sub.sub2.p2.x'], [[4.0]], 1.0e-6)
 
 
-@unittest.skipIf(MPI and not PETScVector, "only run under MPI if we have PETSc.")
+@unittest.skipUnless(MPI and  PETScVector, "MPI and PETSc are required.")
 class TestGroupCSMPI(unittest.TestCase):
 
     N_PROCS = 2
@@ -849,58 +850,8 @@ class TestGroupCSMPI(unittest.TestCase):
         assert_rel_error(self, J['sum.y', 'sub.sub1.p1.x'], [[2.0]], 1.0e-6)
         assert_rel_error(self, J['sum.y', 'sub.sub2.p2.x'], [[4.0]], 1.0e-6)
 
-    def test_newton_with_direct_solver(self):
-        # Make sure this works under mpi with bug fixed in norm calculation.
 
-        if not PETScVector:
-            raise unittest.SkipTest("PETSc is not installed")
-
-        prob = om.Problem()
-
-        model = prob.model
-        sub = model.add_subsystem('sub', om.ParallelGroup(), promotes=['*'])
-
-        model.add_subsystem('px', om.IndepVarComp('x', 1.0), promotes=['x'])
-        model.add_subsystem('pz', om.IndepVarComp('z', np.array([5.0, 2.0])), promotes=['z'])
-
-        sub.add_subsystem('d1', SellarDis1withDerivatives(), promotes=['x', 'z', 'y1', 'y2'])
-        sub.add_subsystem('d2', SellarDis2withDerivatives(), promotes=['z', 'y1', 'y2'])
-
-        model.add_subsystem('obj_cmp', om.ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
-                                                   z=np.array([0.0, 0.0]), x=0.0),
-                            promotes=['obj', 'x', 'z', 'y1', 'y2'])
-
-        model.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
-        model.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
-
-
-        sub.nonlinear_solver = om.NewtonSolver()
-        sub.linear_solver = om.DirectSolver()
-        sub.nonlinear_solver.options['atol'] = 1e-10
-        sub.nonlinear_solver.options['rtol'] = 1e-10
-
-        model.approx_totals(method='cs')
-
-        prob.setup(check=False, local_vector_class=PETScVector)
-        prob.set_solver_print(level=0)
-        prob.run_model()
-
-        assert_rel_error(self, prob.get_val('y1', get_remote=True), 25.58830273, .00001)
-        assert_rel_error(self, prob.get_val('y2', get_remote=True), 12.05848819, .00001)
-
-        wrt = ['z', 'x']
-        of = ['obj', 'con1', 'con2']
-
-        J = prob.compute_totals(of=of, wrt=wrt, return_format='flat_dict')
-        assert_rel_error(self, J['obj', 'z'][0][0], 9.61001056, 1.0e-6)
-        assert_rel_error(self, J['obj', 'z'][0][1], 1.78448534, 1.0e-6)
-        assert_rel_error(self, J['obj', 'x'][0][0], 2.98061391, 1.0e-6)
-        assert_rel_error(self, J['con1', 'z'][0][0], -9.61002186, 1.0e-6)
-        assert_rel_error(self, J['con1', 'z'][0][1], -0.78449158, 1.0e-6)
-        assert_rel_error(self, J['con1', 'x'][0][0], -0.98061448, 1.0e-6)
-
-
-@unittest.skipIf(MPI and not PETScVector, "only run under MPI if we have PETSc.")
+@unittest.skipUnless(MPI and  PETScVector, "MPI and PETSc are required.")
 class TestGroupFDMPI(unittest.TestCase):
 
     N_PROCS = 2
@@ -1331,7 +1282,7 @@ class TestGroupComplexStep(unittest.TestCase):
         model.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
         model.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
-        sub.nonlinear_solver = om.NewtonSolver()
+        sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         sub.linear_solver = om.DirectSolver(assemble_jac=False)
         sub.nonlinear_solver.options['atol'] = 1e-10
         sub.nonlinear_solver.options['rtol'] = 1e-10
@@ -1382,7 +1333,7 @@ class TestGroupComplexStep(unittest.TestCase):
         model.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
         model.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
-        sub.nonlinear_solver = om.NewtonSolver()
+        sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         sub.linear_solver = om.DirectSolver()
         sub.options['assembled_jac_type'] = 'dense'
 
@@ -1435,7 +1386,7 @@ class TestGroupComplexStep(unittest.TestCase):
         model.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
         model.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
-        sub.nonlinear_solver = om.NewtonSolver()
+        sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         sub.linear_solver = om.DirectSolver()
         sub.options['assembled_jac_type'] = 'csc'
 
@@ -1493,7 +1444,7 @@ class TestGroupComplexStep(unittest.TestCase):
                 sub.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0', con2=0.0, y2=0.0),
                                   promotes=['con2', 'y2'])
 
-                self.nonlinear_solver = om.NewtonSolver()
+                self.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
                 self.linear_solver = om.LinearBlockGS()
                 self.linear_solver.options['maxiter'] = 25
                 self.linear_solver.options['atol'] = 1e-16
@@ -1549,7 +1500,7 @@ class TestGroupComplexStep(unittest.TestCase):
                 sub.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0', con2=0.0, y2=0.0),
                                   promotes=['con2', 'y2'])
 
-                self.nonlinear_solver = om.NewtonSolver()
+                self.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
                 self.linear_solver = om.LinearBlockGS()
                 self.linear_solver.options['maxiter'] = 25
                 self.linear_solver.options['atol'] = 1e-16
@@ -1605,7 +1556,7 @@ class TestGroupComplexStep(unittest.TestCase):
                 sub.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0', con2=0.0, y2=0.0),
                                   promotes=['con2', 'y2'])
 
-                self.nonlinear_solver = om.NewtonSolver()
+                self.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
                 self.linear_solver = om.LinearBlockGS()
                 self.linear_solver.options['maxiter'] = 25
                 self.linear_solver.options['atol'] = 1e-16
@@ -1656,7 +1607,7 @@ class TestGroupComplexStep(unittest.TestCase):
         model.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
         model.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
-        sub.nonlinear_solver = om.NewtonSolver()
+        sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         sub.linear_solver = om.ScipyKrylov()
         sub.nonlinear_solver.options['atol'] = 1e-10
         sub.nonlinear_solver.options['rtol'] = 1e-10
@@ -1702,7 +1653,7 @@ class TestGroupComplexStep(unittest.TestCase):
         model.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
         model.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
-        sub.nonlinear_solver = om.NewtonSolver()
+        sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         sub.linear_solver = om.ScipyKrylov(assemble_jac=True)
         sub.nonlinear_solver.options['atol'] = 1e-20
         sub.nonlinear_solver.options['rtol'] = 1e-20
@@ -1751,7 +1702,7 @@ class TestGroupComplexStep(unittest.TestCase):
         # Finite difference for the Newton linear solve only
         subfd.approx_totals(method='fd')
 
-        sub.nonlinear_solver = om.NewtonSolver()
+        sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         sub.nonlinear_solver.options['maxiter'] = 12
         sub.linear_solver = om.DirectSolver(assemble_jac=False)
         sub.nonlinear_solver.options['atol'] = 1e-20
@@ -1797,7 +1748,7 @@ class TestGroupComplexStep(unittest.TestCase):
         model.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
         model.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
-        prob.model.nonlinear_solver = om.NewtonSolver()
+        prob.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         prob.model.linear_solver = om.DirectSolver(assemble_jac=False)
 
         prob.model.approx_totals(method='cs')
