@@ -4,6 +4,7 @@ import os
 import sys
 import shutil
 import tempfile
+import itertools
 
 import unittest
 import numpy as np
@@ -26,9 +27,15 @@ from openmdao.utils.coloring import Coloring, _compute_coloring, array_viz
 from openmdao.utils.mpi import MPI
 from openmdao.utils.testing_utils import use_tempdirs
 from openmdao.test_suite.tot_jac_builder import TotJacBuilder
+from openmdao.utils.general_utils import run_driver
 
 import openmdao.test_suite
 
+try:
+    from parameterized import parameterized
+except ImportError:
+    from openmdao.utils.assert_utils import SkipParameterized as parameterized
+
 try:
     from openmdao.vectors.petsc_vector import PETScVector
 except ImportError:
@@ -86,7 +93,7 @@ class DynPartialsComp(om.ExplicitComponent):
 
 
 
-def run_opt(driver_class, mode, assemble_type=None, color_info=None, sparsity=None, derivs=True,
+def run_opt(driver_class, mode, assemble_type=None, color_info=None, derivs=True,
             recorder=None, has_lin_constraint=True, has_diag_partials=True, partial_coloring=False,
             **options):
 
@@ -149,6 +156,7 @@ def run_opt(driver_class, mode, assemble_type=None, color_info=None, sparsity=No
         p.driver.declare_coloring(tol=1e-15)
         del options['dynamic_total_coloring']
 
+    p.driver.options['debug_print'] = ['totals']
     p.driver.options.update(options)
 
     p.model.add_design_var('x')
@@ -172,8 +180,6 @@ def run_opt(driver_class, mode, assemble_type=None, color_info=None, sparsity=No
     # # setup coloring
     if color_info is not None:
         p.driver.use_fixed_coloring(color_info)
-    elif sparsity is not None:
-        p.driver.set_total_jac_sparsity(sparsity)
 
     if recorder:
         p.driver.add_recorder(recorder)
@@ -341,6 +347,35 @@ class SimulColoringPyoptSparseTestCase(unittest.TestCase):
         rep = repr(p_color.driver._coloring_info['coloring'])
         self.assertEqual(rep.replace('L', ''), 'Coloring (direction: fwd, ncolors: 5, shape: (22, 21)')
 
+    @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
+    def test_print_options_total_with_coloring_fwd(self):
+        # first, run w/o coloring
+        p = run_opt(pyOptSparseDriver, 'fwd', optimizer='SNOPT', print_results=False)
+        p_color = run_opt(pyOptSparseDriver, 'fwd', optimizer='SNOPT', print_results=False,
+                          dynamic_total_coloring=True)
+
+        failed, output = run_driver(p_color)
+
+        self.assertFalse(failed, "Optimization failed.")
+
+        self.assertTrue('In mode: fwd, Solving variable(s):' in output)
+        self.assertTrue("('indeps.y', [1, 3, 5, 7, 9])" in output)
+        self.assertTrue('Elapsed Time:' in output)
+
+    @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
+    def test_print_options_total_with_coloring_rev(self):
+        # first, run w/o coloring
+        p = run_opt(pyOptSparseDriver, 'rev', optimizer='SNOPT', print_results=False)
+        p_color = run_opt(pyOptSparseDriver, 'rev', optimizer='SNOPT', print_results=False,
+                          dynamic_total_coloring=True)
+
+        failed, output = run_driver(p_color)
+
+        self.assertFalse(failed, "Optimization failed.")
+
+        self.assertTrue('In mode: rev, Solving variable(s):' in output)
+        self.assertTrue("('r_con.g', [0])" in output)
+        self.assertTrue('Elapsed Time:' in output)
 
 @use_tempdirs
 @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
@@ -722,80 +757,15 @@ class SimulColoringRevScipyTestCase(unittest.TestCase):
                          "Derivative support has been turned off but compute_totals was called.")
 
 
-@use_tempdirs
-class SparsityTestCase(unittest.TestCase):
-
-    def setUp(self):
-        self.sparsity = {
-            "circle.area": {
-               "indeps.x": [[], [], [1, 10]],
-               "indeps.y": [[], [], [1, 10]],
-               "indeps.r": [[0], [0], [1, 1]]
-            },
-            "r_con.g": {
-               "indeps.x": [[0, 1, 2, 3, 4, 5, 6, 7, 8, 9], [0, 1, 2, 3, 4, 5, 6, 7, 8, 9], [10, 10]],
-               "indeps.y": [[0, 1, 2, 3, 4, 5, 6, 7, 8, 9], [0, 1, 2, 3, 4, 5, 6, 7, 8, 9], [10, 10]],
-               "indeps.r": [[0, 1, 2, 3, 4, 5, 6, 7, 8, 9], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [10, 1]]
-            },
-            "theta_con.g": {
-               "indeps.x": [[0, 1, 2, 3, 4], [0, 2, 4, 6, 8], [5, 10]],
-               "indeps.y": [[0, 1, 2, 3, 4], [0, 2, 4, 6, 8], [5, 10]],
-               "indeps.r": [[], [], [5, 1]]
-            },
-            "delta_theta_con.g": {
-               "indeps.x": [[0, 0, 1, 1, 2, 2, 3, 3, 4, 4], [0, 1, 2, 3, 4, 5, 6, 7, 8, 9], [5, 10]],
-               "indeps.y": [[0, 0, 1, 1, 2, 2, 3, 3, 4, 4], [0, 1, 2, 3, 4, 5, 6, 7, 8, 9], [5, 10]],
-               "indeps.r": [[], [], [5, 1]]
-            },
-            "l_conx.g": {
-               "indeps.x": [[0], [0], [1, 10]],
-               "indeps.y": [[], [], [1, 10]],
-               "indeps.r": [[], [], [1, 1]]
-            }
-        }
-
-    @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
-    def test_sparsity_snopt(self):
-        # first, run without sparsity
-        p = run_opt(pyOptSparseDriver, 'fwd', optimizer='SNOPT', print_results=False)
-
-        # run with dynamic sparsity
-        p_dynamic = run_opt(pyOptSparseDriver, 'fwd', dynamic_derivs_sparsity=True,
-                            optimizer='SNOPT', print_results=False)
-
-        # run with provided sparsity
-        p_sparsity = run_opt(pyOptSparseDriver, 'fwd', sparsity=self.sparsity,
-                             optimizer='SNOPT', print_results=False)
-
-        assert_almost_equal(p['circle.area'], np.pi, decimal=7)
-        assert_almost_equal(p_dynamic['circle.area'], np.pi, decimal=7)
-        assert_almost_equal(p_sparsity['circle.area'], np.pi, decimal=7)
-
-    def test_sparsity_pyoptsparse_slsqp(self):
+def _test_func_name(func, num, param):
+    args = []
+    for p in param.args:
         try:
-            from pyoptsparse import OPT
-        except ImportError:
-            raise unittest.SkipTest("This test requires pyoptsparse.")
-
-        try:
-            OPT('SLSQP')
+            arg = p.__name__
         except:
-            raise unittest.SkipTest("This test requires pyoptsparse SLSQP.")
-
-        # first, run without sparsity
-        p = run_opt(pyOptSparseDriver, 'fwd', optimizer='SLSQP', print_results=False)
-
-        # run with dynamic sparsity
-        p_dynamic = run_opt(pyOptSparseDriver, 'fwd', dynamic_derivs_sparsity=True,
-                            optimizer='SLSQP', print_results=False)
-
-        # run with provided sparsity
-        p_sparsity = run_opt(pyOptSparseDriver, 'fwd', sparsity=self.sparsity,
-                             optimizer='SLSQP', print_results=False)
-
-        assert_almost_equal(p['circle.area'], np.pi, decimal=7)
-        assert_almost_equal(p_dynamic['circle.area'], np.pi, decimal=7)
-        assert_almost_equal(p_sparsity['circle.area'], np.pi, decimal=7)
+            arg = str(p)
+        args.append(arg)
+    return func.__name__ + '_'.join(args)
 
 
 class BidirectionalTestCase(unittest.TestCase):
@@ -821,46 +791,41 @@ class BidirectionalTestCase(unittest.TestCase):
                              "%d" % (n, tot_colors, n))
 
     def test_arrowhead(self):
-        for n in [55, 50, 5]:
+        for n in [5, 50, 55]:
             builder = TotJacBuilder(n, n)
-            builder.add_row(0)
-            builder.add_col(0)
-            builder.add_block_diag([(1,1)] * (n-1), 1, 1)
+            builder.add_row(n-1)
+            builder.add_col(n-1)
+            builder.add_block_diag([(1,1)] * (n-1), 0, 0)
             builder.color('auto')
             tot_size, tot_colors, fwd_solves, rev_solves, pct = builder.coloring._solves_info()
             self.assertEqual(tot_colors, 3)
 
-    @unittest.skipIf(LooseVersion(scipy.__version__) < LooseVersion("0.19.1"), "scipy version too old")
-    def test_can_715(self):
-        # this test is just to show the superiority of bicoloring vs. single coloring in
-        # either direction.  Bicoloring gives only 21 colors in this case vs. 105 for either
-        # fwd or rev.
+    @parameterized.expand(itertools.product(
+        [('n4c6-b15', 3), ('can_715', 21), ('lp_finnis', 14), ('ash608', 6), ('ash331', 6),
+         ('D_6', 28), ('Harvard500', 26), ('illc1033', 5)],
+        ), name_func=_test_func_name
+    )
+    @unittest.skipIf(load_npz is None, "scipy version too old")
+    def test_bidir_coloring(self, tup):
+        matname, expected_colors = tup
         matdir = os.path.join(os.path.dirname(openmdao.test_suite.__file__), 'matrices')
 
-        # uses matrix can_715 from the sparse matrix collection website
-        mat = load_npz(os.path.join(matdir, 'can_715.npz')).toarray()
+        # uses matrices from the sparse matrix collection website (sparse.tamu.edu)
+        matfile = os.path.join(matdir, matname + '.npz')
+        if not os.path.exists(matfile):
+            raise unittest.SkipTest("Matrix test file were not included.")
+
+        mat = load_npz(matfile).toarray()
         mat = np.asarray(mat, dtype=bool)
         coloring = _compute_coloring(mat, 'auto')
+        mat = None
 
         tot_size, tot_colors, fwd_solves, rev_solves, pct = coloring._solves_info()
 
-        self.assertEqual(tot_colors, 21)
-
-        # verify that unidirectional colorings are much worse (105 vs 21 for bidirectional)
-        coloring = _compute_coloring(mat, 'fwd')
-
-        tot_size, tot_colors, fwd_solves, rev_solves, pct = coloring._solves_info()
-
-        self.assertEqual(tot_colors, 105)
-
-        coloring = _compute_coloring(mat, 'rev')
-
-        tot_size, tot_colors, fwd_solves, rev_solves, pct = coloring._solves_info()
-
-        self.assertEqual(tot_colors, 105)
+        self.assertEqual(tot_colors, expected_colors)
 
 
-def _get_mat(rows, cols):
+def _get_random_mat(rows, cols):
     if MPI:
         if MPI.COMM_WORLD.rank == 0:
             mat = np.random.random(rows * cols).reshape((rows, cols)) - 0.5
@@ -873,13 +838,12 @@ def _get_mat(rows, cols):
 
 
 @use_tempdirs
-@unittest.skipUnless(MPI is not None and PETScVector is not None and OPTIMIZER is not None, "PETSc and pyOptSparse required.")
+@unittest.skipUnless(OPTIMIZER is not None, "pyOptSparse required.")
 class MatMultMultipointTestCase(unittest.TestCase):
-    N_PROCS = 4
 
     def test_multipoint_with_coloring(self):
         size = 10
-        num_pts = self.N_PROCS
+        num_pts = 4
 
         np.random.seed(11)
 
@@ -891,7 +855,7 @@ class MatMultMultipointTestCase(unittest.TestCase):
             p.driver.opt_settings['Major iterations limit'] = 100
             p.driver.opt_settings['Major feasibility tolerance'] = 1.0E-6
             p.driver.opt_settings['Major optimality tolerance'] = 1.0E-6
-            p.driver.opt_settings['iSumm'] = 6
+            # p.driver.opt_settings['iSumm'] = 6
 
         model = p.model
         for i in range(num_pts):
@@ -900,13 +864,13 @@ class MatMultMultipointTestCase(unittest.TestCase):
 
         par1 = model.add_subsystem('par1', om.ParallelGroup())
         for i in range(num_pts):
-            mat = _get_mat(5, size)
+            mat = _get_random_mat(5, size)
             par1.add_subsystem('comp%d' % i, om.ExecComp('y=A.dot(x)', A=mat, x=np.ones(size), y=np.ones(5)))
             model.connect('indep%d.x' % i, 'par1.comp%d.x' % i)
 
         par2 = model.add_subsystem('par2', om.ParallelGroup())
         for i in range(num_pts):
-            mat = _get_mat(size, 5)
+            mat = _get_random_mat(size, 5)
             par2.add_subsystem('comp%d' % i, om.ExecComp('y=A.dot(x)', A=mat, x=np.ones(5), y=np.ones(size)))
             model.connect('par1.comp%d.y' % i, 'par2.comp%d.x' % i)
             par2.add_constraint('comp%d.y' % i, lower=-1.)
@@ -921,25 +885,93 @@ class MatMultMultipointTestCase(unittest.TestCase):
 
         model.add_objective('obj.y')
 
-        try:
-            p.setup()
+        p.setup()
 
-            p.run_driver()
+        p.run_driver()
 
-            J = p.compute_totals()
+        J = p.compute_totals()
 
-            for i in range(num_pts):
-                vname = 'par2.comp%d.A' % i
-                if vname in model._var_abs_names['input']:
-                    norm = np.linalg.norm(J['par2.comp%d.y'%i,'indep%d.x'%i] -
-                                          getattr(par2, 'comp%d'%i)._inputs['A'].dot(getattr(par1, 'comp%d'%i)._inputs['A']))
-                    self.assertLess(norm, 1.e-7)
-                elif vname not in model._var_allprocs_abs_names['input']:
-                    self.fail("Can't find variable par2.comp%d.A" % i)
+        for i in range(num_pts):
+            cname = 'par2.comp%d' % i
+            vname = cname + '.A'
+            if vname in model._var_abs_names['input']:
+                A1 = p.get_val('par1.comp%d.A'%i)
+                A2 = p.get_val('par2.comp%d.A'%i)
+                norm = np.linalg.norm(J['par2.comp%d.y'%i,'indep%d.x'%i] - A2.dot(A1))
+                self.assertLess(norm, 1.e-7)
 
-            print("final obj:", p['obj.y'])
-        except Exception as err:
-            print(str(err))
+        print("final obj:", p['obj.y'])
+
+
+# use_tempdirs is inherited
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
+class MatMultMultipointMPI2TestCase(MatMultMultipointTestCase):
+    N_PROCS = 2
+
+
+# use_tempdirs is inherited
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
+class MatMultMultipointMPI4TestCase(MatMultMultipointTestCase):
+    N_PROCS = 4
+
+
+class SimulColoringVarOutputTestClass(unittest.TestCase):
+    def test_multi_variable_coloring_debug_print_totals(self):
+        size = 10
+        num_pts = 4
+
+        np.random.seed(11)
+
+        p = om.Problem()
+        p.driver = om.ScipyOptimizeDriver()
+        p.driver.options['optimizer'] = 'SLSQP'
+        p.driver.declare_coloring()
+        p.driver.options['debug_print'] = ['totals']
+        if OPTIMIZER == 'SNOPT':
+            p.driver.opt_settings['Major iterations limit'] = 100
+            p.driver.opt_settings['Major feasibility tolerance'] = 1.0E-6
+            p.driver.opt_settings['Major optimality tolerance'] = 1.0E-6
+        #     p.driver.opt_settings['iSumm'] = 6
+
+        model = p.model
+        for i in range(num_pts):
+            model.add_subsystem('indep%d' % i, om.IndepVarComp('x', val=np.ones(size)))
+            model.add_design_var('indep%d.x' % i)
+
+        par1 = model.add_subsystem('par1', om.ParallelGroup())
+        for i in range(num_pts):
+            mat = _get_random_mat(5, size)
+            par1.add_subsystem('comp%d' % i, om.ExecComp('y=A.dot(x)', A=mat, x=np.ones(size), y=np.ones(5)))
+            model.connect('indep%d.x' % i, 'par1.comp%d.x' % i)
+
+        par2 = model.add_subsystem('par2', om.ParallelGroup())
+        for i in range(num_pts):
+            mat = _get_random_mat(size, 5)
+            par2.add_subsystem('comp%d' % i, om.ExecComp('y=A.dot(x)', A=mat, x=np.ones(5), y=np.ones(size)))
+            model.connect('par1.comp%d.y' % i, 'par2.comp%d.x' % i)
+            par2.add_constraint('comp%d.y' % i, lower=-1.)
+
+            model.add_subsystem('normcomp%d' % i, om.ExecComp("y=sum(x*x)", x=np.ones(size)))
+            model.connect('par2.comp%d.y' % i, 'normcomp%d.x' % i)
+
+        model.add_subsystem('obj', om.ExecComp("y=" + '+'.join(['x%d' % i for i in range(num_pts)])))
+
+        for i in range(num_pts):
+            model.connect('normcomp%d.y' % i, 'obj.x%d' % i)
+
+        model.add_objective('obj.y')
+
+        p.setup(check=False)
+
+        failed, output = run_driver(p)
+
+        self.assertFalse(failed, "Optimization failed.")
+
+        self.assertTrue('In mode: fwd, Solving variable(s):' in output)
+        self.assertTrue("('indep0.x', [7])" in output)
+        self.assertTrue("('indep1.x', [7])" in output)
+        self.assertTrue("('indep2.x', [7])" in output)
+        self.assertTrue("('indep3.x', [7])" in output)
 
 
 class DumbComp(om.ExplicitComponent):
@@ -1049,7 +1081,6 @@ class SimulColoringConfigCheckTestCase(unittest.TestCase):
                               sizes=[3, 4, 5], color='partial', fixed=False)
         p.run_driver()
 
-        print('++++++++++++')
         p = self._build_model(ofnames=['w', 'x', 'y', 'z'], wrtnames=['a', 'b', 'c', 'd'],
                                 sizes=[3, 4, 5, 6], color='partial', fixed=True)
 
@@ -1132,4 +1163,3 @@ class SimulColoringConfigCheckTestCase(unittest.TestCase):
 
 if __name__ == '__main__':
     unittest.main()
-
