@@ -16,6 +16,7 @@ from openmdao.test_suite.components.sellar import StateConnection, \
      SellarDis1withDerivatives, SellarDis2withDerivatives
 from openmdao.utils.assert_utils import assert_rel_error
 from openmdao.utils.general_utils import remove_whitespace
+from openmdao.utils.logger_utils import TestLogger
 
 
 class ModCompEx(om.ExplicitComponent):
@@ -237,6 +238,39 @@ class DiscreteTestCase(unittest.TestCase):
         model.connect('indep.x', ['expl.x', 'impl.x'])
 
         prob.setup()
+
+        #
+        # list vars before model has been run (relative names)
+        #
+        expl_inputs = prob.model.expl.list_inputs(out_stream=None)
+        expected = {
+            'a': {'value': [10.]},
+            'x': {'value': 10}
+        }
+        self.assertEqual(dict(expl_inputs), expected)
+
+        impl_inputs = prob.model.impl.list_inputs(out_stream=None)
+        expected = {
+            'x': {'value': 10}
+        }
+        self.assertEqual(dict(impl_inputs), expected)
+
+        expl_outputs = prob.model.expl.list_outputs(out_stream=None)
+        expected = {
+            'b': {'value': [0.]},
+            'y': {'value': 0}
+        }
+        self.assertEqual(dict(expl_outputs), expected)
+
+        impl_outputs = prob.model.impl.list_outputs(out_stream=None)
+        expected = {
+            'y': {'value': 0}
+        }
+        self.assertEqual(dict(impl_outputs), expected)
+
+        #
+        # run model
+        #
         prob.run_model()
 
         #
@@ -259,11 +293,11 @@ class DiscreteTestCase(unittest.TestCase):
         text = stream.getvalue()
 
         self.assertEqual(1, text.count("3 Input(s) in 'model'"))
-        self.assertEqual(1, text.count('top'))
-        self.assertEqual(1, text.count('  expl'))
-        self.assertEqual(1, text.count('    a'))
-        self.assertEqual(1, text.count('  impl'))
-        self.assertEqual(2, text.count('    x'))      # both implicit & explicit
+        self.assertEqual(1, text.count('\nmodel'))
+        self.assertEqual(1, text.count('\n  expl'))
+        self.assertEqual(1, text.count('\n    a'))
+        self.assertEqual(1, text.count('\n  impl'))
+        self.assertEqual(2, text.count('\n    x'))      # both implicit & explicit
 
         #
         # list outputs, not hierarchical
@@ -286,13 +320,13 @@ class DiscreteTestCase(unittest.TestCase):
         prob.model.list_outputs(values=True, residuals=True, hierarchical=True, out_stream=stream)
         text = stream.getvalue()
 
-        self.assertEqual(text.count('top'), 2)        # both implicit & explicit
-        self.assertEqual(text.count('  indep'), 1)
-        self.assertEqual(text.count('    x'), 1)
-        self.assertEqual(text.count('  expl'), 1)
-        self.assertEqual(text.count('    b'), 1)
-        self.assertEqual(text.count('  impl'), 1)
-        self.assertEqual(text.count('    y'), 2)      # both implicit & explicit
+        self.assertEqual(text.count('\nmodel'), 2)      # both implicit & explicit
+        self.assertEqual(text.count('\n  indep'), 1)
+        self.assertEqual(text.count('\n    x'), 1)
+        self.assertEqual(text.count('\n  expl'), 1)
+        self.assertEqual(text.count('\n    b'), 1)
+        self.assertEqual(text.count('\n  impl'), 1)
+        self.assertEqual(text.count('\n    y'), 2)      # both implicit & explicit
 
     def test_list_inputs_outputs_promoted(self):
         model = om.Group()
@@ -348,7 +382,7 @@ class DiscreteTestCase(unittest.TestCase):
             "",
             "varname  value  prom_name",
             "-------  -----  ---------",
-            "top",
+            "model",
             "  indep",
             "    x    11     x",
             "  expl",
@@ -361,7 +395,7 @@ class DiscreteTestCase(unittest.TestCase):
             "",
             "varname  value  prom_name",
             "-------  -----  ---------",
-            "top",
+            "model",
             "  impl",
             "    y    2      impl.y",
         ]
@@ -524,6 +558,27 @@ class DiscreteTestCase(unittest.TestCase):
 
         np.testing.assert_almost_equal(J, np.array([[3.]]))
 
+    def test_discrete_unconn_inputs(self):
+        prob = om.Problem()
+        model = prob.model
+
+        c1 = model.add_subsystem('c1', PathCompEx(), promotes=['x'])
+        c2 = model.add_subsystem('c2', PathCompEx(), promotes=['x'])
+
+        testlogger = TestLogger()
+
+        prob.setup(check=['unconnected_inputs'], logger=testlogger)
+        prob.run_model()
+
+        expected_warning_1 = (
+            "The following inputs are not connected:\n"
+            "   x  (p):\n"
+            "      c1.x  c1\n"
+            "      c2.x  c2\n"
+        )
+
+        self.assertTrue(testlogger.contains('warning', expected_warning_1))
+
     def test_discrete_deriv_implicit(self):
         prob = om.Problem()
         model = prob.model
@@ -585,6 +640,36 @@ class DiscreteTestCase(unittest.TestCase):
         self.assertEqual(str(ctx.exception),
                          "Total derivative with respect to 'indep.x' depends upon discrete output variables ['G.G1.C1.y'].")
 
+    def test_connection_to_output(self):
+        prob = om.Problem()
+        model = prob.model
+
+        model.add_subsystem('C1', ModCompEx(modval=2))
+        model.add_subsystem('C2', ModCompEx(modval=2))
+
+        model.connect('C1.y', 'C2.y')
+
+        with self.assertRaises(Exception) as cm:
+            prob.setup()
+
+        msg = "Group (<model>): Attempted to connect from 'C1.y' to 'C2.y', but 'C2.y' is an output. All connections must be from an output to an input."
+        self.assertEqual(str(cm.exception), msg)
+
+    def test_connection_from_input(self):
+        prob = om.Problem()
+        model = prob.model
+
+        model.add_subsystem('C1', ModCompEx(modval=2))
+        model.add_subsystem('C2', ModCompEx(modval=2))
+
+        model.connect('C1.x', 'C2.x')
+
+        with self.assertRaises(Exception) as cm:
+            prob.setup()
+
+        msg = "Group (<model>): Attempted to connect from 'C1.x' to 'C2.x', but 'C1.x' is an input. All connections must be from an output to an input."
+        self.assertEqual(str(cm.exception), msg)
+
 
 class SolverDiscreteTestCase(unittest.TestCase):
     def _setup_model(self, solver_class):
@@ -832,7 +917,7 @@ class DiscreteFeatureTestCase(unittest.TestCase):
 
         model.add_subsystem('comp', ImpWithInitial())
 
-        model.nonlinear_solver = om.NewtonSolver()
+        model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         model.linear_solver = om.ScipyKrylov()
 
         prob.setup()
