@@ -155,7 +155,8 @@ class ImplicitCompTestCase(unittest.TestCase):
         assert_rel_error(self, total_derivs['comp3.x', 'comp1.c'], [[-0.5]])
 
     def test_list_inputs_before_run(self):
-        msg = "Group (<model>): Unable to list inputs until model has been run."
+        # cannot list_inputs on a Group before running
+        msg = "Group (<model>): Unable to list inputs on a Group until model has been run."
         try:
             self.prob.model.list_inputs()
         except Exception as err:
@@ -163,8 +164,41 @@ class ImplicitCompTestCase(unittest.TestCase):
         else:
             self.fail("Exception expected")
 
+        # list_inputs on a component before running is okay
+        c2_inputs = self.prob.model.comp2.list_inputs(desc=True, out_stream=None)
+        expected = {
+            'a': {'value': [1.], 'desc': ''},
+            'b': {'value': [1.], 'desc': ''},
+            'c': {'value': [1.], 'desc': ''}
+        }
+        self.assertEqual(dict(c2_inputs), expected)
+
+        # listing component inputs based on tags should work
+        c2_inputs = self.prob.model.comp2.list_inputs(tags='tag_a', out_stream=None)
+        self.assertEqual(dict(c2_inputs), {'a': {'value': [1.]}})
+
+        # includes and excludes based on relative names should work
+        c2_inputs = self.prob.model.comp2.list_inputs(includes='a', out_stream=None)
+        self.assertEqual(dict(c2_inputs), {'a': {'value': [1.]}})
+
+        c2_inputs = self.prob.model.comp2.list_inputs(excludes='c', out_stream=None)
+        expected = {
+            'a': {'value': [1.]},
+            'b': {'value': [1.]},
+        }
+        self.assertEqual(dict(c2_inputs), expected)
+
+        # specifying prom_name should not cause an error
+        c2_inputs = self.prob.model.comp2.list_inputs(prom_name=True, out_stream=None)
+        self.assertEqual(dict(c2_inputs), {
+            'a': {'value': [1.], 'prom_name': 'a'},
+            'b': {'value': [1.], 'prom_name': 'b'},
+            'c': {'value': [1.], 'prom_name': 'c'}
+        })
+
     def test_list_outputs_before_run(self):
-        msg = "Group (<model>): Unable to list outputs until model has been run."
+        # cannot list_outputs on a Group before running
+        msg = "Group (<model>): Unable to list outputs on a Group until model has been run."
         try:
             self.prob.model.list_outputs()
         except Exception as err:
@@ -172,18 +206,46 @@ class ImplicitCompTestCase(unittest.TestCase):
         else:
             self.fail("Exception expected")
 
+        # list_outputs on a component before running is okay
+        c2_outputs = self.prob.model.comp2.list_outputs(out_stream=None)
+        expected = {
+            'x': {'value': [0.]}
+        }
+        self.assertEqual(dict(c2_outputs), expected)
+
+        # listing component outputs based on tags should work
+        c2_outputs = self.prob.model.comp2.list_outputs(tags='tag_x', out_stream=None)
+        self.assertEqual(dict(c2_outputs), expected)
+
+        # includes and excludes based on relative names should work
+        c2_outputs = self.prob.model.comp2.list_outputs(includes='x', out_stream=None)
+        self.assertEqual(dict(c2_outputs), expected)
+
+        c2_outputs = self.prob.model.comp2.list_outputs(excludes='x', out_stream=None)
+        self.assertEqual(dict(c2_outputs), {})
+
+        # specifying residuals_tol should not cause an error
+        c2_outputs = self.prob.model.comp2.list_outputs(residuals_tol=.01, out_stream=None)
+        self.assertEqual(dict(c2_outputs), expected)
+
+        # specifying prom_name should not cause an error
+        c2_outputs = self.prob.model.comp2.list_outputs(prom_name=True, out_stream=None)
+        self.assertEqual(dict(c2_outputs), {
+            'x': {'value': 0., 'prom_name': 'x'}
+        })
+
     def test_list_inputs(self):
         self.prob.run_model()
 
         stream = cStringIO()
-        inputs = self.prob.model.list_inputs(hierarchical=False, out_stream=stream)
+        inputs = self.prob.model.list_inputs(hierarchical=False, desc=True, out_stream=stream)
         self.assertEqual(sorted(inputs), [
-            ('comp2.a', {'value': [1.]}),
-            ('comp2.b', {'value': [-4.]}),
-            ('comp2.c', {'value': [3.]}),
-            ('comp3.a', {'value': [1.]}),
-            ('comp3.b', {'value': [-4.]}),
-            ('comp3.c', {'value': [3.]})
+            ('comp2.a', {'value':  [1.], 'desc': ''}),
+            ('comp2.b', {'value': [-4.], 'desc': ''}),
+            ('comp2.c', {'value':  [3.], 'desc': ''}),
+            ('comp3.a', {'value':  [1.], 'desc': ''}),
+            ('comp3.b', {'value': [-4.], 'desc': ''}),
+            ('comp3.c', {'value':  [3.], 'desc': ''})
         ])
         text = stream.getvalue()
         self.assertEqual(text.count('comp2.'), 3)
@@ -425,7 +487,7 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
         model.connect('p.c', 'fn.c')
         model.connect('comp.x', 'fn.x')
 
-        model.nonlinear_solver = om.NewtonSolver()
+        model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         model.nonlinear_solver.options['rtol'] = 1e-12
         model.nonlinear_solver.options['atol'] = 1e-12
         model.nonlinear_solver.options['maxiter'] = 15
@@ -470,7 +532,7 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
         group.connect('px.x', 'comp1.x')
         group.connect('comp1.y', 'comp2.x')
 
-        group.nonlinear_solver = om.NewtonSolver()
+        group.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         group.nonlinear_solver.options['maxiter'] = 1
 
         prob = om.Problem(model=group)
@@ -513,7 +575,7 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
 
         group.add_subsystem('sub', sub)
 
-        group.nonlinear_solver = om.NewtonSolver()
+        group.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         group.nonlinear_solver.options['maxiter'] = 1
 
         prob = om.Problem(model=group)
@@ -556,7 +618,7 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
 
         group.add_subsystem('sub', sub)
 
-        sub.nonlinear_solver = om.NewtonSolver()
+        sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         sub.nonlinear_solver.options['maxiter'] = 1
 
         prob = om.Problem(model=group)
@@ -611,7 +673,7 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
 
         model.add_subsystem('comp', ImpWithInitial())
 
-        model.nonlinear_solver = om.NewtonSolver()
+        model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         model.linear_solver = om.ScipyKrylov()
 
         prob.setup()
@@ -638,7 +700,7 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
         group.connect('px.x', 'comp1.x')
         group.connect('comp1.y', 'comp2.x')
 
-        group.nonlinear_solver = om.NewtonSolver()
+        group.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         group.nonlinear_solver.options['maxiter'] = 1
 
         prob = om.Problem(model=group)
@@ -670,7 +732,7 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
         group.connect('px.x', 'comp1.x')
         group.connect('comp1.y', 'comp2.x')
 
-        group.nonlinear_solver = om.NewtonSolver()
+        group.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         group.nonlinear_solver.options['maxiter'] = 1
 
         prob = om.Problem(model=group)
@@ -702,7 +764,7 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
         group.connect('px.x', 'comp1.x')
         group.connect('comp1.y', 'comp2.x')
 
-        group.nonlinear_solver = om.NewtonSolver()
+        group.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         group.nonlinear_solver.options['maxiter'] = 1
 
         prob = om.Problem(model=group)
@@ -1229,10 +1291,10 @@ class ListFeatureTestCase(unittest.TestCase):
             ('sub.comp3.c', {}),
         ])
         self.assertEqual(1, text.count("6 Input(s) in 'model'"))
-        self.assertEqual(1, text.count("top"))
-        self.assertEqual(1, text.count("  sub"))
-        self.assertEqual(1, text.count("    comp2"))
-        self.assertEqual(2, text.count("      a"))
+        self.assertEqual(1, text.count("\nmodel"))
+        self.assertEqual(1, text.count("\n  sub"))
+        self.assertEqual(1, text.count("\n    comp2"))
+        self.assertEqual(2, text.count("\n      a"))
         num_non_empty_lines = sum([1 for s in text.splitlines() if s.strip()])
         self.assertEqual(num_non_empty_lines, 14)
 
@@ -1265,7 +1327,7 @@ class ListFeatureTestCase(unittest.TestCase):
         model.connect('d1.y1', 'd2.y1')
         model.connect('d2.y2', 'd1.y2')
 
-        model.nonlinear_solver = om.NewtonSolver()
+        model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         model.nonlinear_solver.options['maxiter'] = 5
         model.linear_solver = om.ScipyKrylov()
         model.linear_solver.precon = om.LinearBlockGS()
