@@ -128,6 +128,7 @@ class SerialSimpleFDTestCase(TestCase):
         assert_rel_error(self, J['C1.y']['P1.x'], np.eye(size)*mult, 1e-6)
 
 
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
 class ParallelSimpleFDTestCase2(TestCase):
 
     N_PROCS = 2
@@ -153,6 +154,7 @@ class ParallelSimpleFDTestCase2(TestCase):
         assert_rel_error(self, J['C1.y']['P1.x'], np.eye(size)*mult, 1e-6)
 
 
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
 class ParallelFDTestCase5(TestCase):
 
     N_PROCS = 5
@@ -201,6 +203,7 @@ class SerialDiamondFDTestCase(TestCase):
             self.assertEquals(str(err), "Value (0) of option 'num_par_fd' is less than minimum allowed value of 1.")
 
 
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
 class ParallelDiamondFDTestCase(TestCase):
 
     N_PROCS = 4
@@ -265,13 +268,13 @@ def _test_func_name(func, num, param):
     return func.__name__ + '_' + '_'.join(args)
 
 
-@unittest.skipUnless(PETScVector, "PETSc is required.")
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
 class MatMultTestCase(unittest.TestCase):
     N_PROCS = 4
 
-    @parameterized.expand(itertools.product([20, 21, 22], [2, 3, 4], ['fd', 'cs'], [om.LinearRunOnce, om.DirectSolver]),
+    @parameterized.expand(itertools.product([20, 21, 22], [2, 3, 4], ['fd', 'cs']),
                           name_func=_test_func_name)
-    def test_par_fd(self, size, num_par_fd, method, solver):
+    def test_par_fd(self, size, num_par_fd, method):
         if MPI:
             if MPI.COMM_WORLD.rank == 0:
                 mat = np.random.random(5 * size).reshape((5, size)) - 0.5
@@ -284,7 +287,6 @@ class MatMultTestCase(unittest.TestCase):
         p = om.Problem()
 
         model = p.model
-        model.linear_solver = solver()
 
         model.add_subsystem('indep', om.IndepVarComp('x', val=np.ones(mat.shape[1])))
         comp = model.add_subsystem('comp', MatMultComp(mat, approx_method=method, num_par_fd=num_par_fd))
@@ -314,7 +316,7 @@ class MatMultTestCase(unittest.TestCase):
         self.assertLess(norm, 1.e-7)
 
 
-@unittest.skipUnless(PETScVector, "PETSc is required.")
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
 class MatMultParallelTestCase(unittest.TestCase):
     N_PROCS = 8
 
@@ -468,7 +470,7 @@ class ParFDWarningsTestCase(unittest.TestCase):
             _setup_problem(self.mat, partial_method='fd', partial_num_par_fd = 3)
 
 
-@unittest.skipUnless(PETScVector, "PETSc is required.")
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
 class ParFDErrorsMPITestCase(unittest.TestCase):
     N_PROCS = 3
 
@@ -489,7 +491,7 @@ class ParFDErrorsMPITestCase(unittest.TestCase):
         self.assertEqual(str(ctx.exception), "MatMultComp (comp): num_par_fd is > 1 but no FD is active.")
 
 
-@unittest.skipUnless(PETScVector, "PETSc is required.")
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
 class ParFDFeatureTestCase(unittest.TestCase):
     N_PROCS = 3
 
