@@ -12,6 +12,7 @@ import openmdao.api as om
 from openmdao.core.system import get_relevant_vars
 from openmdao.core.driver import Driver
 from openmdao.utils.assert_utils import assert_rel_error, assert_warning
+import openmdao.utils.hooks as hooks
 from openmdao.test_suite.components.paraboloid import Paraboloid
 from openmdao.test_suite.components.sellar import SellarDerivatives
 
@@ -55,7 +56,6 @@ class SellarOneComp(om.ImplicitComponent):
         self.declare_partials('R_y1', ['R_y1', 'x', 'z', 'y1', 'y2'])
         self.declare_partials('R_y2', ['R_y2','z', 'y1', 'y2'])
 
-
     def apply_nonlinear(self, inputs, outputs, residuals):
 
         z0 = inputs['z'][0]
@@ -105,7 +105,6 @@ class SellarOneComp(om.ImplicitComponent):
         J['R_y2','z'] = [1, 1]
         J['R_y2','y1'] = 0.5*outputs['y1']**-0.5
 
-
     def solve_nonlinear(self, inputs, outputs):
         z0 = inputs['z'][0]
         z1 = inputs['z'][1]
@@ -232,6 +231,43 @@ class TestProblem(unittest.TestCase):
 
         prob.compute_totals(of=['comp.f_xy'], wrt=['p1.x', 'p2.y'])
 
+    def test_single_string_wrt_of(self):
+
+        prob = om.Problem()
+        model = prob.model
+
+        model.add_subsystem('p1', om.IndepVarComp('x', 3.0))
+        model.add_subsystem('p2', om.IndepVarComp('y', -4.0))
+        model.add_subsystem('comp', Paraboloid())
+
+        model.connect('p1.x', 'comp.x')
+        model.connect('p2.y', 'comp.y')
+
+        prob.setup()
+        prob.run_model()
+
+        totals = prob.compute_totals(of='comp.f_xy', wrt='p1.x')
+        assert_rel_error(self, totals[('comp.f_xy', 'p1.x')][0][0], -4.0)
+
+    def test_two_var_single_string_error(self):
+
+        prob = om.Problem()
+        model = prob.model
+
+        model.add_subsystem('p1', om.IndepVarComp('x', 3.0))
+        model.add_subsystem('p2', om.IndepVarComp('y', -4.0))
+        model.add_subsystem('comp', Paraboloid())
+
+        model.connect('p1.x', 'comp.x')
+        model.connect('p2.y', 'comp.y')
+
+        prob.setup()
+        prob.run_model()
+
+        with self.assertRaises(KeyError) as cm:
+            totals = prob.compute_totals(of='comp.f_xy', wrt="p1.x, p2.y")
+        self.assertEqual(str(cm.exception), "'p1.x, p2.y'")
+
     def test_compute_totals_cleanup(self):
         p = om.Problem()
         model = p.model
@@ -1006,7 +1042,7 @@ class TestProblem(unittest.TestCase):
         with assertRaisesRegex(self, TypeError, msg):
             prob.get_val('comp.x', 'degK')
 
-        msg = "Can't set variable 'comp.x' with units of 'cm' to value with units of 'degK'."
+        msg = "Can't set variable 'comp.x' with units 'cm' to value with units 'degK'."
         with assertRaisesRegex(self, TypeError, msg):
             prob.set_val('comp.x', 55.0, 'degK')
 
@@ -1014,7 +1050,7 @@ class TestProblem(unittest.TestCase):
         with assertRaisesRegex(self, TypeError, msg):
             prob.get_val('no_unit.x', 'degK')
 
-        msg = "Can't set variable 'no_unit.x' with units of 'None' to value with units of 'degK'."
+        msg = "Can't set variable 'no_unit.x' with units 'None' to value with units 'degK'."
         with assertRaisesRegex(self, TypeError, msg):
             prob.set_val('no_unit.x', 55.0, 'degK')
 
@@ -1134,12 +1170,12 @@ class TestProblem(unittest.TestCase):
     def test_setup_bad_mode(self):
         # Test error message when passing bad mode to setup.
 
-        prob = om.Problem()
+        prob = om.Problem(name='foo')
 
         try:
             prob.setup(mode='junk')
         except ValueError as err:
-            msg = "Unsupported mode: 'junk'. Use either 'fwd' or 'rev'."
+            msg = "Problem foo: Unsupported mode: 'junk'. Use either 'fwd' or 'rev'."
             self.assertEqual(str(err), msg)
         else:
             self.fail('Expecting ValueError')
@@ -1194,7 +1230,7 @@ class TestProblem(unittest.TestCase):
         try:
             prob.run_model()
         except RuntimeError as err:
-            msg = "The `setup` method must be called before `run_model`."
+            msg = "Problem: The `setup` method must be called before `run_model`."
             self.assertEqual(str(err), msg)
         else:
             self.fail('Expecting RuntimeError')
@@ -1202,7 +1238,7 @@ class TestProblem(unittest.TestCase):
         try:
             prob.run_driver()
         except RuntimeError as err:
-            msg = "The `setup` method must be called before `run_driver`."
+            msg = "Problem: The `setup` method must be called before `run_driver`."
             self.assertEqual(str(err), msg)
         else:
             self.fail('Expecting RuntimeError')
@@ -1210,7 +1246,7 @@ class TestProblem(unittest.TestCase):
     def test_run_with_invalid_prefix(self):
         # Test error message when running with invalid prefix.
 
-        msg = "The 'case_prefix' argument should be a string."
+        msg = "Problem: The 'case_prefix' argument should be a string."
 
         prob = om.Problem()
 
@@ -1249,7 +1285,7 @@ class TestProblem(unittest.TestCase):
             prob = om.Problem(root=om.Group(), model=om.Group())
 
         self.assertEqual(str(cm.exception),
-                         "Cannot specify both 'root' and 'model'. "
+                         "Problem: Cannot specify both 'root' and 'model'. "
                          "'root' has been deprecated, please use 'model'.")
 
         msg = "The 'root' argument provides backwards " \
@@ -1284,14 +1320,14 @@ class TestProblem(unittest.TestCase):
             prob = om.Problem(om.ScipyOptimizeDriver())
 
         self.assertEqual(str(cm.exception),
-                         "The value provided for 'model' is not a valid System.")
+                         "Problem: The value provided for 'model' is not a valid System.")
 
         # invalid driver
         with self.assertRaises(TypeError) as cm:
             prob = om.Problem(driver=SellarDerivatives())
 
         self.assertEqual(str(cm.exception),
-                         "The value provided for 'driver' is not a valid Driver.")
+                         "Problem: The value provided for 'driver' is not a valid Driver.")
 
     def test_relevance(self):
         p = om.Problem()
@@ -1386,7 +1422,7 @@ class TestProblem(unittest.TestCase):
         p_opt.model = SellarOneComp(solve_y1=SOLVE_Y1, solve_y2=SOLVE_Y2)
 
         if SOLVE_Y1 or SOLVE_Y2:
-            newton = p_opt.model.nonlinear_solver = om.NewtonSolver()
+            newton = p_opt.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
             newton.options['iprint'] = 0
 
         # NOTE: need to have this direct solver attached to the sellar comp until I define a solve_linear for it
@@ -1457,7 +1493,7 @@ class TestProblem(unittest.TestCase):
 
             def configure(self):
                 # This will solve it.
-                self.sub.nonlinear_solver = om.NewtonSolver()
+                self.sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
                 self.sub.linear_solver = om.ScipyKrylov()
 
         top = om.Problem(model=Super())
@@ -1507,12 +1543,203 @@ class TestProblem(unittest.TestCase):
         top.setup()
 
         # These solvers override the ones set in the setup method of the 'sub' groups
-        top.model.sub.nonlinear_solver = om.NewtonSolver()
+        top.model.sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.sub.linear_solver = om.ScipyKrylov()
 
         self.assertTrue(isinstance(top.model.sub.nonlinear_solver, om.NewtonSolver))
         self.assertTrue(isinstance(top.model.sub.linear_solver, om.ScipyKrylov))
 
+    def test_configure_add_indep_var(self):
+        # add outputs to an IndepVarComp in Group configure
+
+        class Model(om.Group):
+            def initialize(self):
+                self.options.declare('where_to_add', values=('setup', 'configure'))
+
+            def setup(self):
+                comp1 = self.add_subsystem('comp1', om.IndepVarComp())
+                comp2 = self.add_subsystem('comp2', om.IndepVarComp())
+
+                comp1.add_output('foo', val=1.0)
+                comp2.add_output('foo', val=1.0)
+
+                self.add_subsystem('comp3', om.ExecComp('y=a+b'))
+
+                if self.options['where_to_add'] == 'setup':
+                    comp1.add_output('a', val=2.0)
+                    comp2.add_output('b', val=3.0)
+
+                    self.connect('comp1.a', 'comp3.a')
+                    self.connect('comp2.b', 'comp3.b')
+
+            def configure(self):
+                if self.options['where_to_add'] == 'configure':
+                    self.comp1.add_output('a', val=2.0)
+                    self.comp2.add_output('b', val=3.0)
+
+                    self.connect('comp1.a', 'comp3.a')
+                    self.connect('comp2.b', 'comp3.b')
+
+        for where in ('setup', 'configure'):
+            p = om.Problem(Model(where_to_add=where))
+            p.setup()
+            p.run_model()
+
+            inputs = p.model.list_inputs(out_stream=None)
+            self.assertEqual(sorted(inputs), [
+                ('comp3.a', {'value': [2.]}),
+                ('comp3.b', {'value': [3.]})
+            ], "Inputs don't match when added in %s." % where)
+
+            outputs = p.model.list_outputs(out_stream=None)
+            self.assertEqual(sorted(outputs), [
+                ('comp1.a',   {'value': [2.]}),
+                ('comp1.foo', {'value': [1.]}),
+                ('comp2.b',   {'value': [3.]}),
+                ('comp2.foo', {'value': [1.]}),
+                ('comp3.y',   {'value': [5.]})
+            ], "Outputs don't match when added in %s." % where)
+
+    def test_configure_add_input_output(self):
+        # add inputs and outputs to an ExplicitComponent in Group configure
+
+        class MyComp(om.ExplicitComponent):
+            def setup(self):
+                self.add_input('a', val=0.)
+                self.add_output('a2', val=0.)
+
+            def compute(self, inputs, outputs):
+                outputs['a2'] = inputs['a'] * 2.
+                if 'b' in inputs:
+                    outputs['b2'] = inputs['b'] * 2.
+
+        class Model(om.Group):
+            def initialize(self):
+                self.options.declare('add_b2', default=False)
+
+            def setup(self):
+                self.add_subsystem('indep', om.IndepVarComp(), promotes=['*'])
+                self.add_subsystem('mcomp', MyComp(), promotes=['*'])
+
+                self.add_subsystem('sub', om.Group(), promotes_inputs=['*'])
+                self.sub.add_subsystem('mcomp', MyComp(), promotes=['*'])
+
+                self.indep.add_output('a', val=2.0)
+
+            def configure(self):
+                if self.options['add_b2']:
+                    self.indep.add_output('b', val=3.0)
+
+                    self.mcomp.add_input('b', val=0.)
+                    self.mcomp.add_output('b2', val=0.)
+
+                    self.sub.mcomp.add_input('b', val=0.)
+                    self.sub.mcomp.add_output('b2', val=0.)
+
+        # add inputs/outputs in setup only
+        p = om.Problem(Model(add_b2=False))
+        p.setup()
+        p.run_model()
+
+        inputs = p.model.list_inputs(out_stream=None)
+        self.assertEqual(sorted(inputs), [
+            ('mcomp.a',     {'value': [2.]}),
+            ('sub.mcomp.a', {'value': [2.]}),
+        ])
+
+        outputs = p.model.list_outputs(out_stream=None)
+        self.assertEqual(sorted(outputs), [
+            ('indep.a',      {'value': [2.]}),
+            ('mcomp.a2',     {'value': [4.]}),
+            ('sub.mcomp.a2', {'value': [4.]}),
+        ])
+
+        # add inputs/outputs in configure
+        p = om.Problem(Model(add_b2=True))
+        p.setup()
+        p.run_model()
+
+        inputs = p.model.list_inputs(out_stream=None)
+        self.assertEqual(sorted(inputs), [
+            ('mcomp.a',     {'value': [2.]}),
+            ('mcomp.b',     {'value': [3.]}),
+            ('sub.mcomp.a', {'value': [2.]}),
+            ('sub.mcomp.b', {'value': [3.]}),
+        ])
+
+        outputs= p.model.list_outputs(out_stream=None)
+        self.assertEqual(sorted(outputs), [
+            ('indep.a',      {'value': [2.]}),
+            ('indep.b',      {'value': [3.]}),
+            ('mcomp.a2',     {'value': [4.]}),
+            ('mcomp.b2',     {'value': [6.]}),
+            ('sub.mcomp.a2', {'value': [4.]}),
+            ('sub.mcomp.b2', {'value': [6.]}),
+        ])
+
+    def test_feature_configure_add_input_output(self):
+        """
+        A simple example to compute the resultant force on an aircraft using data
+        from an external source. Demonstrates adding I/O in the 'configure' method.
+        """
+        import numpy as np
+        import openmdao.api as om
+
+        class FlightDataComp(om.ExplicitComponent):
+            """
+            Simulate data generated by an external source/code
+            """
+            def setup(self):
+                # number of points may not be known a priori
+                n = 3
+
+                # The vector represents forces at n time points (rows) in 2 dimensional plane (cols)
+                self.add_output(name='thrust', shape=(n, 2), units='kN')
+                self.add_output(name='drag', shape=(n, 2), units='kN')
+                self.add_output(name='lift', shape=(n, 2), units='kN')
+                self.add_output(name='weight', shape=(n, 2), units='kN')
+
+            def compute(self, inputs, outputs):
+                outputs['thrust'][:, 0] = [500, 600, 700]
+                outputs['drag'][:, 0]  = [400, 400, 400]
+                outputs['weight'][:, 1] = [1000, 1001, 1002]
+                outputs['lift'][:, 1]  = [1000, 1000, 1000]
+
+
+        class ForceModel(om.Group):
+            def setup(self):
+                self.add_subsystem('flightdatacomp', FlightDataComp(),
+                                   promotes_outputs=['thrust', 'drag', 'lift', 'weight'])
+
+                self.add_subsystem('totalforcecomp', om.AddSubtractComp())
+
+            def configure(self):
+                # Some models that require self-interrogation need to be able to add
+                # I/O in components from the configure method of their containing groups.
+                # In this case, we can only determine the 'vec_size' for totalforcecomp
+                # after flightdatacomp has been setup.
+
+                flight_data = dict(self.flightdatacomp.list_outputs(shape=True, out_stream=None))
+                data_shape = flight_data['thrust']['shape']
+
+                self.totalforcecomp.add_equation('total_force',
+                                                 input_names=['thrust', 'drag', 'lift', 'weight'],
+                                                 vec_size=data_shape[0], length=data_shape[1],
+                                                 scaling_factors=[1, -1, 1, -1], units='kN')
+
+                p.model.connect('thrust', 'totalforcecomp.thrust')
+                p.model.connect('drag', 'totalforcecomp.drag')
+                p.model.connect('lift', 'totalforcecomp.lift')
+                p.model.connect('weight', 'totalforcecomp.weight')
+
+
+        p = om.Problem(model=ForceModel())
+        p.setup()
+        p.run_model()
+
+        assert_rel_error(self, p.get_val('totalforcecomp.total_force', units='kN'),
+                         np.array([[100, 200, 300], [0, -1, -2]]).T)
+
     def test_feature_system_configure(self):
         import openmdao.api as om
 
@@ -1546,7 +1773,7 @@ class TestProblem(unittest.TestCase):
 
             def configure(self):
                 # This will solve it.
-                self.sub.nonlinear_solver = om.NewtonSolver()
+                self.sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
                 self.sub.linear_solver = om.ScipyKrylov()
 
         top = om.Problem(model=Super())
@@ -1596,21 +1823,22 @@ class TestProblem(unittest.TestCase):
         top.setup()
 
         # This will solve it.
-        top.model.sub.nonlinear_solver = om.NewtonSolver()
+        top.model.sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.sub.linear_solver = om.ScipyKrylov()
 
         self.assertTrue(isinstance(top.model.sub.nonlinear_solver, om.NewtonSolver))
         self.assertTrue(isinstance(top.model.sub.linear_solver, om.ScipyKrylov))
 
-    def test_post_setup_hook(self):
+    def test_post_final_setup_hook(self):
         def hook_func(prob):
             prob['p2.y'] = 5.0
 
-        prob = om.Problem()
-        model = prob.model
-        om.Problem._post_setup_func = hook_func
-
+        hooks.use_hooks = True
+        hooks._register_hook('final_setup', class_name='Problem', post=hook_func)
         try:
+            prob = om.Problem()
+            model = prob.model
+
             model.add_subsystem('p1', om.IndepVarComp('x', 3.0))
             model.add_subsystem('p2', om.IndepVarComp('y', -4.0))
             model.add_subsystem('comp', om.ExecComp("f_xy=2.0*x+3.0*y"))
@@ -1624,7 +1852,8 @@ class TestProblem(unittest.TestCase):
             assert_rel_error(self, prob['p2.y'], 5.0)
             assert_rel_error(self, prob['comp.f_xy'], 21.0)
         finally:
-            om.Problem._post_setup_func = None
+            hooks._unregister_hook('final_setup', class_name='Problem')
+            hooks.use_hooks = False
 
     def test_list_problem_vars(self):
         model = SellarDerivatives()
@@ -1800,7 +2029,5 @@ class NestedProblemTestCase(unittest.TestCase):
         p.run_model()
 
 
-
-
 if __name__ == "__main__":
     unittest.main()
