@@ -285,7 +285,7 @@ class TestScaling(unittest.TestCase):
                                                              use_scal=use_scal))
             prob.model.connect('row1.y', 'row2.x')
             prob.model.connect('row2.y', 'row1.x')
-            prob.model.nonlinear_solver = om.NewtonSolver(maxiter=2, atol=1e-5, rtol=0)
+            prob.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, maxiter=2, atol=1e-5, rtol=0)
             prob.model.nonlinear_solver.linear_solver = om.ScipyKrylov(maxiter=1)
 
             prob.set_solver_print(level=0)
@@ -854,7 +854,7 @@ class TestScaling(unittest.TestCase):
 
         model.connect('p1.x', 'comp.x')
 
-        model.nonlinear_solver = om.NewtonSolver()
+        model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         model.linear_solver = om.DirectSolver()
 
         prob.setup()
@@ -872,7 +872,7 @@ class TestScaling(unittest.TestCase):
 
         model.connect('p1.x', 'comp.x')
 
-        model.nonlinear_solver = om.NewtonSolver()
+        model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         model.linear_solver = om.DirectSolver(assemble_jac=True)
 
         prob.setup()
@@ -1063,7 +1063,7 @@ class MyDriver(Driver):
         self.con_meta = deepcopy(self._cons)
 
         # Run model
-        model = self._problem.model
+        model = self._problem().model
         model.run_solve_nonlinear()
 
         # Con vals and derivs
@@ -1198,7 +1198,7 @@ class TestScalingOverhaul(unittest.TestCase):
         model.connect('p.x1_u', 'comp.x2_u')
 
         model.linear_solver = om.DirectSolver()
-        model.nonlinear_solver = om.NewtonSolver()
+        model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         model.nonlinear_solver.options['atol'] = 1e-12
         model.nonlinear_solver.options['rtol'] = 1e-12
 
