@@ -7,7 +7,6 @@ import os
 from itertools import product, chain
 
 import numpy as np
-import cProfile
 from contextlib import contextmanager
 from six import iteritems, iterkeys, itervalues
 from collections import Counter
@@ -24,73 +23,6 @@ from openmdao.utils.name_maps import abs_key2rel_key, rel_key2abs_key
 # an object used to detect when a named value isn't found
 _notfound = object()
 
-def dump_dist_idxs(problem, vec_name='nonlinear', stream=sys.stdout):  # pragma: no cover
-    """Print out the distributed idxs for each variable in input and output vecs.
-
-    Output looks like this:
-
-    C3.y     24
-    C2.y     21
-    sub.C3.y 18
-    C1.y     17     18 C3.x
-    P.x      14     15 C2.x
-    C3.y     12     12 sub.C3.x
-    C3.y     11     11 C1.x
-    C2.y      8      8 C3.x
-    sub.C2.y  5      5 C2.x
-    C1.y      3      2 sub.C2.x
-    P.x       0      0 C1.x
-
-    Parameters
-    ----------
-    problem : <Problem>
-        The problem object that contains the model.
-    vec_name : str
-        Name of vector to dump (when there are multiple vectors due to parallel derivs)
-    stream : File-like
-        Where dump output will go.
-    """
-    def _get_data(g, type_):
-
-        sizes = g._var_sizes[vec_name]
-        vnames = g._var_allprocs_abs_names
-        abs2meta = g._var_allprocs_abs2meta
-
-        idx = 0
-        data = []
-        nwid = 0
-        iwid = 0
-        total = 0
-        for rank in range(g.comm.size):
-            for ivar, vname in enumerate(vnames[type_]):
-                sz = sizes[type_][rank, ivar]
-                if sz > 0:
-                    data.append((vname, str(total)))
-                nwid = max(nwid, len(vname))
-                iwid = max(iwid, len(data[-1][1]))
-                total += sz
-
-        return data, nwid, iwid
-
-    def _dump(g, stream):
-
-        pdata, pnwid, piwid = _get_data(g, 'input')
-        udata, unwid, uiwid = _get_data(g, 'output')
-
-        data = []
-        for u, p in zip_longest(udata, pdata, fillvalue=('', '')):
-            data.append((u[0], u[1], p[1], p[0]))
-
-        template = "{0:<{wid0}} {1:>{wid1}}     {2:>{wid2}} {3:<{wid3}}\n"
-        for d in data[::-1]:
-            stream.write(template.format(d[0], d[1], d[2], d[3],
-                                         wid0=unwid, wid1=uiwid,
-                                         wid2=piwid, wid3=pnwid))
-        stream.write("\n\n")
-
-    if not MPI or MPI.COMM_WORLD.rank == 0:
-        _dump(problem.model, stream)
-
 
 class _NoColor(object):
     """
@@ -122,7 +54,7 @@ def _get_color_printer(stream=sys.stdout, colors=True, rank=0):
         def color_print(s, **kwargs):
             pass
     else:
-        def color_print(s, fore='', color='', end=''):
+        def color_print(s, color='', end=''):
             """
             """
             print(color + s, file=stream, end='')
@@ -133,7 +65,7 @@ def _get_color_printer(stream=sys.stdout, colors=True, rank=0):
 
 
 def tree(top, show_solvers=True, show_jacs=True, show_colors=True, show_approx=True,
-         filter=None, max_depth=0, rank=0, stream=sys.stdout):
+         filter=None, show_sizes=False, max_depth=0, rank=0, stream=sys.stdout):
     """
     Dump the model tree structure to the given stream.
 
@@ -156,6 +88,8 @@ def tree(top, show_solvers=True, show_jacs=True, show_colors=True, show_approx=T
         A function taking a System arg and returning None or an iter of (name, value) tuples.
         If None is returned, that system will not be displayed.  Otherwise, the system will
         be displayed along with any name, value pairs returned from the filter.
+    show_sizes : bool
+        If True, show input and output sizes for each System.
     max_depth : int
         Maximum depth for display.
     rank : int
@@ -200,10 +134,13 @@ def tree(top, show_solvers=True, show_jacs=True, show_colors=True, show_approx=T
                 colr = Fore.CYAN + Style.BRIGHT
             cprint("%s " % type(s).__name__, color=colr)
             cprint("%s" % s.name)
+            if s.options['distributed']:
+                cprint(" (distributed)", color=Fore.MAGENTA)
 
         # FIXME: these sizes could be wrong under MPI
-        cprint(" (%d / %d)" % (s._inputs._data.size, s._outputs._data.size),
-               color=Fore.RED + Style.BRIGHT)
+        if show_sizes:
+            cprint(" (%d / %d)" % (s._inputs._data.size, s._outputs._data.size),
+                color=Fore.RED + Style.BRIGHT)
 
         if show_solvers:
             lnsolver = type(s.linear_solver).__name__
@@ -431,6 +368,7 @@ def profiling(outname='prof.out'):
     outname : str
         Name of the output file containing profiling stats.
     """
+    import cProfile
     prof = cProfile.Profile()
     prof.enable()
 
@@ -516,6 +454,7 @@ def trace_mpi(fname='mpi_trace', skip=(), flush=True):
 
     def _mpi_trace_callback(frame, event, arg):
         pname = None
+        commsize = ''
         if event == 'call':
             if 'openmdao' in frame.f_code.co_filename:
                 if frame.f_code.co_name in skip:
@@ -525,10 +464,14 @@ def trace_mpi(fname='mpi_trace', skip=(), flush=True):
                         pname = frame.f_locals['self'].msginfo
                     except:
                         pass
+                    try:
+                        commsize = frame.f_locals['self'].comm.size
+                    except:
+                        pass
                 if pname is not None:
                     if not stack or pname != stack[-1][0]:
                         stack.append([pname, 1])
-                        print('   ' * len(stack), pname, file=outfile, flush=flush)
+                        print('   ' * len(stack), commsize, pname, file=outfile, flush=flush)
                     else:
                         stack[-1][1] += 1
                 print('   ' * len(stack), '-->', frame.f_code.co_name, "%s:%d" %
@@ -543,6 +486,10 @@ def trace_mpi(fname='mpi_trace', skip=(), flush=True):
                         pname = frame.f_locals['self'].msginfo
                     except:
                         pass
+                    try:
+                        commsize = frame.f_locals['self'].comm.size
+                    except:
+                        pass
                 print('   ' * len(stack), '<--', frame.f_code.co_name, "%s:%d" %
                       (frame.f_code.co_filename, frame.f_code.co_firstlineno),
                       file=outfile, flush=flush)
@@ -551,7 +498,8 @@ def trace_mpi(fname='mpi_trace', skip=(), flush=True):
                     if stack[-1][1] < 1:
                         stack.pop()
                         if stack:
-                            print('   ' * len(stack), stack[-1][0], file=outfile, flush=flush)
+                            print('   ' * len(stack), commsize, stack[-1][0], file=outfile,
+                                  flush=flush)
         else:
             _print_c_func(frame, arg, _c_map[event])
 
