@@ -236,8 +236,6 @@ def _process_1_profile(fname):
     tree_nodes = {}
     tree_parts = []
 
-    empty = [None, 0., 0]
-
     for funcpath, count, t in _iter_raw_prof_file(fname):
         parts = funcpath.split('|')
 
@@ -351,7 +349,7 @@ def _iprof_totals_setup_parser(parser):
                         help='Raw profile data files or a python file.')
 
 
-def _iprof_totals_exec(options):
+def _iprof_totals_exec(options, user_args):
     """
     Called from the command line (openmdao prof_totals command) to create a file containing total
     elapsed times and number of calls for all profiled functions.
@@ -370,7 +368,7 @@ def _iprof_totals_exec(options):
         if len(options.file) > 1:
             print("iprofview can only process a single python file.", file=sys.stderr)
             sys.exit(-1)
-        _iprof_py_file(options)
+        _iprof_py_file(options, user_args)
         if MPI:
             options.file = ['iprof.%d' % i for i in range(MPI.COMM_WORLD.size)]
         else:
@@ -398,7 +396,7 @@ def _iprof_totals_exec(options):
             out_stream.close()
 
 
-def _iprof_py_file(options):
+def _iprof_py_file(options, user_args):
     """
     Run instance-based profiling on the given python script.
 
@@ -406,6 +404,8 @@ def _iprof_py_file(options):
     ----------
     options : argparse Namespace
         Command line options.
+    user_args : list of str
+        Command line options after '--' (if any).  Passed to user script.
     """
     if not func_group:
         _setup_func_group()
@@ -413,6 +413,9 @@ def _iprof_py_file(options):
     progname = options.file[0]
     sys.path.insert(0, os.path.dirname(progname))
 
+    # update sys.argv in case python script takes cmd line args
+    sys.argv[:] = [progname] + user_args
+
     with open(progname, 'rb') as fp:
         code = compile(fp.read(), progname, 'exec')
 
