@@ -9,6 +9,7 @@ from __future__ import print_function
 
 from collections import OrderedDict
 import json
+import signal
 import sys
 import traceback
 
@@ -23,6 +24,7 @@ from openmdao.core.analysis_error import AnalysisError
 from openmdao.core.driver import Driver, RecordingDebugging
 import openmdao.utils.coloring as coloring_mod
 from openmdao.utils.general_utils import warn_deprecation, simple_warning
+from openmdao.utils.class_util import weak_method_wrapper
 from openmdao.utils.mpi import FakeComm
 
 
@@ -58,6 +60,17 @@ CITATIONS = """@article{Hwang_maud_2018
 """
 
 
+class UserRequestedException(Exception):
+    """
+    User Requested Exception.
+
+    This exception indicates that the user has requested that SNOPT/pyoptsparse ceases
+    model execution and reports to SNOPT that execution should be terminated.
+    """
+
+    pass
+
+
 class pyOptSparseDriver(Driver):
     """
     Driver wrapper for pyoptsparse.
@@ -91,10 +104,18 @@ class pyOptSparseDriver(Driver):
         Dictionary for setting optimizer-specific options.
     pyopt_solution : Solution
         Pyopt_sparse solution object.
+    _in_user_function :bool
+        This is set to True at the start of a pyoptsparse callback to _objfunc and _gradfunc, and
+        restored to False at the finish of each callback.
     _indep_list : list
         List of design variables.
     _quantities : list
         Contains the objectives plus nonlinear constraints.
+    _signal_cache : <Function>
+        Cached function pointer that was assigned as handler for signal defined in option
+        user_teriminate_signal.
+    _user_termination_flag : bool
+        This is set to True when the user sends a signal to terminate the job.
     """
 
     def __init__(self, **kwargs):
@@ -137,6 +158,9 @@ class pyOptSparseDriver(Driver):
         self._indep_list = []
         self._quantities = []
         self.fail = False
+        self._signal_cache = None
+        self._user_termination_flag = False
+        self._in_user_function = False
 
         self.cite = CITATIONS
 
@@ -153,14 +177,12 @@ class pyOptSparseDriver(Driver):
         self.options.declare('gradient method', default='openmdao',
                              values={'openmdao', 'pyopt_fd', 'snopt_fd'},
                              desc='Finite difference implementation to use')
-        self.options.declare('dynamic_derivs_sparsity', default=False, types=bool,
-                             desc='Compute derivative sparsity dynamically if True.')
         self.options.declare('dynamic_simul_derivs', default=False, types=bool,
                              desc='Compute simultaneous derivative coloring dynamically '
                              'if True (deprecated)')
-        self.options.declare('dynamic_derivs_repeats', default=3, types=int,
-                             desc='Number of compute_totals calls during dynamic computation of '
-                                  'simultaneous derivative coloring or derivatives sparsity')
+        self.options.declare('user_teriminate_signal', default=signal.SIGUSR1, allow_none=True,
+                             desc='OS signal that triggers a clean user-terimnation. Only SNOPT'
+                             'supports this option.')
 
     def _setup_driver(self, problem):
         """
@@ -196,7 +218,7 @@ class pyOptSparseDriver(Driver):
         boolean
             Failure flag; True if failed to converge, False is successful.
         """
-        problem = self._problem
+        problem = self._problem()
         model = problem.model
         relevant = model._relevant
         self.pyopt_solution = None
@@ -237,8 +259,6 @@ class pyOptSparseDriver(Driver):
                 coloring = self._coloring_info['coloring']
 
                 self._setup_tot_jac_sparsity()
-            elif self.options['dynamic_derivs_sparsity']:
-                coloring_mod.dynamic_derivs_sparsity(self)
 
             if coloring is not None:
                 # if the improvement wasn't large enough, don't use coloring
@@ -251,7 +271,8 @@ class pyOptSparseDriver(Driver):
                                                                    info['min_improve_pct']))
 
         comm = None if isinstance(problem.comm, FakeComm) else problem.comm
-        opt_prob = Optimization(self.options['title'], self._objfunc, comm=comm)
+        opt_prob = Optimization(self.options['title'], weak_method_wrapper(self, '_objfunc'),
+                                comm=comm)
 
         # Add all design variables
         param_meta = self._designvars
@@ -385,8 +406,8 @@ class pyOptSparseDriver(Driver):
         else:
 
             # Use OpenMDAO's differentiator for the gradient
-            sol = opt(opt_prob, sens=self._gradfunc, storeHistory=self.hist_file,
-                      hotStart=self.hotstart_file)
+            sol = opt(opt_prob, sens=weak_method_wrapper(self, '_gradfunc'),
+                      storeHistory=self.hist_file, hotStart=self.hotstart_file)
 
         # Print results
         if self.options['print_results']:
@@ -406,6 +427,7 @@ class pyOptSparseDriver(Driver):
 
         # Save the most recent solution.
         self.pyopt_solution = sol
+
         try:
             exit_status = sol.optInform['value']
             self.fail = False
@@ -418,6 +440,12 @@ class pyOptSparseDriver(Driver):
             # optimizers other than pySNOPT may not populate this dict
             pass
 
+        # revert signal handler to cached version
+        sigusr = self.options['user_teriminate_signal']
+        if sigusr is not None:
+            signal.signal(sigusr, self._signal_cache)
+            self._signal_cache = None   # to prevent memory leak test from failing
+
         return self.fail
 
     def _objfunc(self, dv_dict):
@@ -441,9 +469,16 @@ class pyOptSparseDriver(Driver):
             0 for successful function evaluation
             1 for unsuccessful function evaluation
         """
-        model = self._problem.model
+        model = self._problem().model
         fail = 0
 
+        # Note: we place our handler as late as possible so that codes that run in the
+        # workflow can place their own handlers.
+        sigusr = self.options['user_teriminate_signal']
+        if sigusr is not None and self._signal_cache is None:
+            self._signal_cache = signal.getsignal(sigusr)
+            signal.signal(sigusr, self._signal_handler)
+
         try:
             for name in self._indep_list:
                 self.set_design_var(name, dv_dict[name])
@@ -451,10 +486,17 @@ class pyOptSparseDriver(Driver):
             # print("Setting DV")
             # print(dv_dict)
 
+            # Check if we caught a termination signal while SNOPT was running.
+            if self._user_termination_flag:
+                func_dict = self.get_objective_values()
+                func_dict.update(self.get_constraint_values(lintype='nonlinear'))
+                return func_dict, 2
+
             # Execute the model
             with RecordingDebugging(self._get_name(), self.iter_count, self) as rec:
                 self.iter_count += 1
                 try:
+                    self._in_user_function = True
                     model.run_solve_nonlinear()
 
                 # Let the optimizer try to handle the error
@@ -462,6 +504,11 @@ class pyOptSparseDriver(Driver):
                     model._clear_iprint()
                     fail = 1
 
+                # User requested termination
+                except UserRequestedException:
+                    model._clear_iprint()
+                    fail = 2
+
                 func_dict = self.get_objective_values()
                 func_dict.update(self.get_constraint_values(lintype='nonlinear'))
 
@@ -483,6 +530,7 @@ class pyOptSparseDriver(Driver):
         # print("Functions calculated")
         # print(dv_dict)
 
+        self._in_user_function = False
         return func_dict, fail
 
     def _gradfunc(self, dv_dict, func_dict):
@@ -508,12 +556,17 @@ class pyOptSparseDriver(Driver):
             0 for successful function evaluation
             1 for unsuccessful function evaluation
         """
-        prob = self._problem
+        prob = self._problem()
         fail = 0
 
         try:
 
+            # Check if we caught a termination signal while SNOPT was running.
+            if self._user_termination_flag:
+                return {}, 2
+
             try:
+                self._in_user_function = True
                 sens_dict = self._compute_totals(of=self._quantities,
                                                  wrt=self._indep_list,
                                                  return_format='dict')
@@ -522,16 +575,11 @@ class pyOptSparseDriver(Driver):
                 prob.model._clear_iprint()
                 fail = 1
 
-                # We need to cobble together a sens_dict of the correct size.
-                # Best we can do is return zeros.
+            # User requested termination
+            except UserRequestedException:
+                prob.model._clear_iprint()
+                fail = 2
 
-                sens_dict = OrderedDict()
-                for okey, oval in iteritems(func_dict):
-                    sens_dict[okey] = OrderedDict()
-                    osize = len(oval)
-                    for ikey, ival in iteritems(dv_dict):
-                        isize = len(ival)
-                        sens_dict[okey][ikey] = np.zeros((osize, isize))
             else:
                 # if we don't convert to 'coo' here, pyoptsparse will do a
                 # conversion of our dense array into a fully dense 'coo', which is bad.
@@ -551,6 +599,18 @@ class pyOptSparseDriver(Driver):
                             newdv[ikey] = sens_dict[okey][ikey]
                 sens_dict = new_sens
 
+            if fail > 0:
+                # We need to cobble together a sens_dict of the correct size.
+                # Best we can do is return zeros.
+
+                sens_dict = OrderedDict()
+                for okey, oval in iteritems(func_dict):
+                    sens_dict[okey] = OrderedDict()
+                    osize = len(oval)
+                    for ikey, ival in iteritems(dv_dict):
+                        isize = len(ival)
+                        sens_dict[okey][ikey] = np.zeros((osize, isize))
+
         except Exception as msg:
             tb = traceback.format_exc()
 
@@ -563,6 +623,7 @@ class pyOptSparseDriver(Driver):
         # print("Derivatives calculated")
         # print(dv_dict)
         # print(sens_dict)
+        self._in_user_function = False
         return sens_dict, fail
 
     def _get_name(self):
@@ -634,3 +695,13 @@ class pyOptSparseDriver(Driver):
                     'coo': [rows, cols, np.zeros(rows.size)],
                     'shape': shape,
                 }
+
+    def _signal_handler(self, signum, frame):
+        # Subsystems (particularly external codes) may declare their own signal handling, so
+        # execute the cached handler first.
+        if self._signal_cache is not signal.Handlers.SIG_DFL:
+            self._signal_cache(signum, frame)
+
+        self._user_termination_flag = True
+        if self._in_user_function:
+            raise UserRequestedException('User requested termination.')
