@@ -9,10 +9,17 @@ import openmdao.api as om
 from openmdao.drivers.genetic_algorithm_driver import GeneticAlgorithm
 from openmdao.test_suite.components.branin import Branin, BraninDiscrete
 from openmdao.test_suite.components.paraboloid import Paraboloid
+from openmdao.test_suite.components.sellar_feature import SellarMDA
 from openmdao.test_suite.components.three_bar_truss import ThreeBarTruss
 from openmdao.utils.assert_utils import assert_rel_error
 from openmdao.utils.mpi import MPI
 
+try:
+    from openmdao.vectors.petsc_vector import PETScVector
+except ImportError:
+    PETScVector = None
+
+extra_prints = False  # enable printing results
 
 class TestSimpleGA(unittest.TestCase):
 
@@ -65,11 +72,15 @@ class TestSimpleGA(unittest.TestCase):
         prob.setup()
         prob.run_driver()
 
+        if extra_prints:
+            print('obj.f', prob['obj.f'])
+            print('px.x', prob['px.x'])
+
         # TODO: Satadru listed this solution, but I get a way better one.
         # Solution: xopt = [0.2857, -0.8571], fopt = 23.2933
-        assert_rel_error(self, prob['obj.f'], 12.37341703, 1e-4)
+        assert_rel_error(self, prob['obj.f'], 12.37306086, 1e-4)
         assert_rel_error(self, prob['px.x'][0], 0.2, 1e-4)
-        assert_rel_error(self, prob['px.x'][1], -0.88654333, 1e-4)
+        assert_rel_error(self, prob['px.x'][1], -0.88653391, 1e-4)
 
     def test_mixed_integer_branin(self):
         prob = om.Problem()
@@ -94,8 +105,11 @@ class TestSimpleGA(unittest.TestCase):
         prob.setup()
         prob.run_driver()
 
+        if extra_prints:
+            print('comp.f', prob['comp.f'])
+
         # Optimal solution
-        assert_rel_error(self, prob['comp.f'], 0.49398, 1e-4)
+        assert_rel_error(self, prob['comp.f'], 0.49399549, 1e-4)
         self.assertTrue(int(prob['p2.xI']) in [3, -3])
 
     def test_mixed_integer_branin_discrete(self):
@@ -124,8 +138,12 @@ class TestSimpleGA(unittest.TestCase):
         prob.setup()
         prob.run_driver()
 
+        if extra_prints:
+            print('comp.f', prob['comp.f'])
+            print('p.xI', prob['p.xI'])
+
         # Optimal solution
-        assert_rel_error(self, prob['comp.f'], 0.49398, 1e-4)
+        assert_rel_error(self, prob['comp.f'], 0.49399549, 1e-4)
         self.assertTrue(prob['p.xI'] in [3, -3])
         self.assertTrue(isinstance(prob['p.xI'], int))
 
@@ -181,6 +199,11 @@ class TestSimpleGA(unittest.TestCase):
         prob['area3'] = 0.0005
         prob.run_driver()
 
+        if extra_prints:
+            print('mass', prob['mass'])
+            print('mat1', prob['mat1'])
+            print('mat2', prob['mat2'])
+
         # Note, GA doesn't do so well with the continuous vars, naturally, so we reduce the space
         # as much as we can. Objective is still rather random, but it is close. GA does a great job
         # of picking the correct values for the integer desvars though.
@@ -244,6 +267,11 @@ class TestSimpleGA(unittest.TestCase):
         prob['area3'] = 0.0005
         prob.run_driver()
 
+        if extra_prints:
+            print('mass', prob['mass'])
+            print('mat1', prob['mat1'])
+            print('mat2', prob['mat2'])
+
         # Note, GA doesn't do so well with the continuous vars, naturally, so we reduce the space
         # as much as we can. Objective is still rather random, but it is close. GA does a great job
         # of picking the correct values for the integer desvars though.
@@ -280,21 +308,17 @@ class TestSimpleGA(unittest.TestCase):
 
     def test_encode_and_decode(self):
         ga = GeneticAlgorithm(None)
-        gen = np.array([[0, 0, 1, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1,
-                         1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 0,
-                         1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0,
-                         0, 1, 0],
-                        [1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1,
-                         1, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 0, 0,
-                         1, 0, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 1,
-                         0, 1, 0]])
+        gen = np.array([[0, 0, 1, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1,
+                         0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 1, 1,
+                         1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0],
+                        [1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1,
+                         0, 1, 0, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0,
+                         1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 1, 0, 1, 0]])
         vlb = np.array([-170.0, -170.0, -170.0, -170.0, -170.0, -170.0])
         vub = np.array([255.0, 255.0, 255.0, 170.0, 170.0, 170.0])
         bits = np.array([9, 9, 9, 9, 9, 9])
-        x = np.array([[-69.36399217, 22.12328767, -7.81800391, -66.86888454,
-                       116.77103718, 76.18395303],
-                      [248.34637965, 191.79060665, -31.93737769, 97.47553816,
-                       118.76712329, 92.15264188]])
+        x = np.array([[-69.36399217, 22.12328767, -7.81800391, -66.86888454, 116.77103718, 76.18395303],
+                      [248.34637965, 191.79060665, -31.93737769, 97.47553816, 118.76712329, 92.15264188]])
 
         ga.npop = 2
         ga.lchrom = int(np.sum(bits))
@@ -302,6 +326,39 @@ class TestSimpleGA(unittest.TestCase):
         np.testing.assert_array_almost_equal(gen[0], ga.encode(x[0], vlb, vub, bits))
         np.testing.assert_array_almost_equal(gen[1], ga.encode(x[1], vlb, vub, bits))
 
+        dec = ga.decode(gen, vlb, vub, bits)
+        enc0 = ga.encode(dec[0], vlb, vub, bits)
+        enc1 = ga.encode(dec[1], vlb, vub, bits)
+        np.testing.assert_array_almost_equal(gen[0], enc0)  # decode followed by encode gives original array
+        np.testing.assert_array_almost_equal(gen[1], enc1)
+
+    def test_encode_and_decode_gray_code(self):
+        ga = GeneticAlgorithm(None)
+        gen = np.array([[0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0,
+                         1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0,
+                         0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1],
+                        [1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0,
+                         1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 0, 1, 1,
+                         1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 1]])
+        vlb = np.array([-170.0, -170.0, -170.0, -170.0, -170.0, -170.0])
+        vub = np.array([255.0, 255.0, 255.0, 170.0, 170.0, 170.0])
+        bits = np.array([9, 9, 9, 9, 9, 9])
+        x = np.array([[-69.36399217, 22.12328767, -7.81800391, -66.86888454, 116.77103718, 76.18395303],
+                      [248.34637965, 191.79060665, -31.93737769, 97.47553816, 118.76712329, 92.15264188]])
+
+        ga.npop = 2
+        ga.lchrom = int(np.sum(bits))
+        ga.gray_code = True
+        np.testing.assert_array_almost_equal(x, ga.decode(gen, vlb, vub, bits))
+        np.testing.assert_array_almost_equal(gen[0], ga.encode(x[0], vlb, vub, bits))
+        np.testing.assert_array_almost_equal(gen[1], ga.encode(x[1], vlb, vub, bits))
+
+        dec = ga.decode(gen, vlb, vub, bits)
+        enc0 = ga.encode(dec[0], vlb, vub, bits)
+        enc1 = ga.encode(dec[1], vlb, vub, bits)
+        np.testing.assert_array_almost_equal(gen[0], enc0)  # decode followed by encode gives original array
+        np.testing.assert_array_almost_equal(gen[1], enc1)
+
     def test_vector_desvars_multiobj(self):
         prob = om.Problem()
 
@@ -326,6 +383,10 @@ class TestSimpleGA(unittest.TestCase):
         prob.setup()
         prob.run_driver()
 
+        if extra_prints:
+            print('indeps.x', prob['indeps.x'])
+            print('indeps.y', prob['indeps.y'])
+
         np.testing.assert_array_almost_equal(prob['indeps.x'], -5)
         np.testing.assert_array_almost_equal(prob['indeps.y'], [3, 1])
 
@@ -352,6 +413,31 @@ class TestSimpleGA(unittest.TestCase):
 
         self.assertEqual(exception.args[0], msg)
 
+    def test_vectorized_constraints(self):
+        prob = om.Problem()
+        model = prob.model
+
+        dim = 2
+        model.add_subsystem('x', om.IndepVarComp('x', np.ones(dim)), promotes=['*'])
+        model.add_subsystem('f_x', om.ExecComp('f_x = sum(x * x)', x=np.ones(dim), f_x=1.0), promotes=['*'])
+        model.add_subsystem('g_x', om.ExecComp('g_x = 1 - x', x=np.ones(dim), g_x=np.zeros(dim)), promotes=['*'])
+
+        prob.driver = om.SimpleGADriver()
+
+        prob.model.add_design_var('x', lower=-10, upper=10)
+        prob.model.add_objective('f_x')
+        prob.model.add_constraint('g_x', upper=np.zeros(dim))
+
+        prob.setup()
+        prob.run_driver()
+
+        if extra_prints:
+            print('x', prob['x'])
+
+        # Check that the constraint is satisfied (x >= 1)
+        for i in range(dim):
+            self.assertLessEqual(1.0, prob["x"][i])
+
 
 class TestDriverOptionsSimpleGA(unittest.TestCase):
 
@@ -367,16 +453,16 @@ class TestDriverOptionsSimpleGA(unittest.TestCase):
         indeps.add_output('x', 1.)
         model.add_subsystem('model', om.ExecComp('y=x**2'), promotes=['*'])
         driver = prob.driver = om.SimpleGADriver()
-        driver.options['Pm'] = 0.1
-        driver.options['Pc'] = 0.01
+        driver.options['Pm'] = 0.123
+        driver.options['Pc'] = 0.0123
         driver.options['max_gen'] = 5
         driver.options['bits'] = {'x': 8}
         prob.model.add_design_var('x', lower=-10., upper=10.)
         prob.model.add_objective('y')
         prob.setup()
         prob.run_driver()
-        self.assertEqual(driver.options['Pm'], 0.1)
-        self.assertEqual(driver.options['Pc'], 0.01)
+        self.assertEqual(driver.options['Pm'], 0.123)
+        self.assertEqual(driver.options['Pc'], 0.0123)
 
 
 class TestMultiObjectiveSimpleGA(unittest.TestCase):
@@ -442,9 +528,11 @@ class TestMultiObjectiveSimpleGA(unittest.TestCase):
         l1 = prob['length']
         w1 = prob['width']
         h1 = prob['height']
-        print('Box dims: ', l1, w1, h1)
-        print('Front and top area: ', front, top)
-        print('Volume: ', prob['volume'])  # should be around 1
+
+        if extra_prints:
+            print('Box dims: ', l1, w1, h1)
+            print('Front and top area: ', front, top)
+            print('Volume: ', prob['volume'])  # should be around 1
 
         # run #2
         # weights changed
@@ -481,9 +569,12 @@ class TestMultiObjectiveSimpleGA(unittest.TestCase):
         l2 = prob2['length']
         w2 = prob2['width']
         h2 = prob2['height']
-        print('Box dims: ', l2, w2, h2)
-        print('Front and top area: ', front2, top2)
-        print('Volume: ', prob['volume'])  # should be around 1
+
+        if extra_prints:
+            print('Box dims: ', l2, w2, h2)
+            print('Front and top area: ', front2, top2)
+            print('Volume: ', prob['volume'])  # should be around 1
+
         self.assertGreater(w1, w2)  # front area does not depend on width
         self.assertGreater(h2, h1)  # top area does not depend on height
 
@@ -536,10 +627,13 @@ class TestConstrainedSimpleGA(unittest.TestCase):
 
         prob.setup()
         prob.run_driver()
-        print('radius', prob['radius'])  # exact solution is (5/pi)^(1/3) ~= 1.167
-        print('height', prob['height'])  # exact solution is 2*radius
-        print('Area', prob['Area'])
-        print('Volume', prob['Volume'])  # should be around 10
+
+        if extra_prints:
+            print('radius', prob['radius'])  # exact solution is (5/pi)^(1/3) ~= 1.167
+            print('height', prob['height'])  # exact solution is 2*radius
+            print('Area', prob['Area'])
+            print('Volume', prob['Volume'])  # should be around 10
+
         self.assertTrue(driver.supports["equality_constraints"], True)
         self.assertTrue(driver.supports["inequality_constraints"], True)
         # check that it is not going to the unconstrained optimum
@@ -588,10 +682,13 @@ class TestConstrainedSimpleGA(unittest.TestCase):
 
         prob.setup()
         prob.run_driver()
-        print('radius', prob['radius'])  # exact solution is (5/pi)^(1/3) ~= 1.167
-        print('height', prob['height'])  # exact solution is 2*radius
-        print('Area', prob['Area'])
-        print('Volume', prob['Volume'])  # should be around 10
+
+        if extra_prints:
+            print('radius', prob['radius'])  # exact solution is (5/pi)^(1/3) ~= 1.167
+            print('height', prob['height'])  # exact solution is 2*radius
+            print('Area', prob['Area'])
+            print('Volume', prob['Volume'])  # should be around 10
+
         self.assertTrue(driver.supports["equality_constraints"], True)
         self.assertTrue(driver.supports["inequality_constraints"], True)
         # it is going to the unconstrained optimum
@@ -639,17 +736,20 @@ class TestConstrainedSimpleGA(unittest.TestCase):
 
         prob.setup()
         prob.run_driver()
-        print('radius', prob['radius'])  # exact solution is (5/pi)^(1/3) ~= 1.167
-        print('height', prob['height'])  # exact solution is 2*radius
-        print('Area', prob['Area'])
-        print('Volume', prob['Volume'])  # should be around 10
+
+        if extra_prints:
+            print('radius', prob['radius'])  # exact solution is (5/pi)^(1/3) ~= 1.167
+            print('height', prob['height'])  # exact solution is 2*radius
+            print('Area', prob['Area'])
+            print('Volume', prob['Volume'])  # should be around 10
+
         self.assertTrue(driver.supports["equality_constraints"], True)
         self.assertTrue(driver.supports["inequality_constraints"], True)
         self.assertAlmostEqual(prob['radius'], 0.5, 1)  # it is going to the unconstrained optimum
         self.assertAlmostEqual(prob['height'], 0.5, 1)  # it is going to the unconstrained optimum
 
 
-@unittest.skipUnless(om.PETScVector, "PETSc is required.")
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
 class MPITestSimpleGA(unittest.TestCase):
 
     N_PROCS = 2
@@ -684,8 +784,12 @@ class MPITestSimpleGA(unittest.TestCase):
         prob.setup()
         prob.run_driver()
 
+        if extra_prints:
+            print('comp.f', prob['comp.f'])
+            print('p2.xI', prob['p2.xI'])
+
         # Optimal solution
-        assert_rel_error(self, prob['comp.f'], 0.49398, 1e-4)
+        assert_rel_error(self, prob['comp.f'], 0.49399549, 1e-4)
         self.assertTrue(int(prob['p2.xI']) in [3, -3])
 
     def test_two_branin_parallel_model(self):
@@ -724,6 +828,10 @@ class MPITestSimpleGA(unittest.TestCase):
         prob.setup()
         prob.run_driver()
 
+        if extra_prints:
+            print('comp.f', prob['comp.f'])
+            print('p2.xI', prob['p2.xI'])
+
         # Optimal solution
         assert_rel_error(self, prob['comp.f'], 0.98799098, 1e-4)
         self.assertTrue(int(prob['p2.xI']) in [3, -3])
@@ -784,6 +892,11 @@ class MPITestSimpleGA(unittest.TestCase):
         prob['area3'] = 0.0005
         prob.run_driver()
 
+        if extra_prints:
+            print('mass', prob['mass'])
+            print('mat1', prob['mat1'])
+            print('mat2', prob['mat2'])
+
         # Note, GA doesn't do so well with the continuous vars, naturally, so we reduce the space
         # as much as we can. Objective is still rather random, but it is close. GA does a great job
         # of picking the correct values for the integer desvars though.
@@ -792,8 +905,120 @@ class MPITestSimpleGA(unittest.TestCase):
         assert_rel_error(self, prob['mat2'], 3, 1e-5)
         # Material 3 can be anything
 
+    def test_mpi_bug_solver(self):
+        # This test verifies that mpi doesn't hang due to collective calls in the solver.
 
-@unittest.skipUnless(om.PETScVector, "PETSc is required.")
+        prob = om.Problem()
+        prob.model = SellarMDA()
+
+        prob.model.add_design_var('x', lower=0, upper=10)
+        prob.model.add_design_var('z', lower=0, upper=10)
+        prob.model.add_objective('obj')
+
+        prob.driver = om.SimpleGADriver(run_parallel=True)
+
+        # Set these low because we don't need to run long.
+        prob.driver.options['max_gen'] = 2
+        prob.driver.options['pop_size'] = 5
+
+        prob.setup()
+        prob.set_solver_print(level=0)
+
+        prob.run_driver()
+
+
+class D1(om.ExplicitComponent):
+    def initialize(self):
+        self.options['distributed'] = True
+
+    def setup(self):
+        comm = self.comm
+        rank = comm.rank
+
+        if rank == 1:
+            start = 1
+            end = 2
+        else:
+            start = 0
+            end = 1
+
+        self.add_input('y2', np.ones((1, ), float),
+                       src_indices=np.arange(start, end, dtype=int))
+        self.add_input('x', np.ones((1, ), float))
+
+        self.add_output('y1', np.ones((1, ), float))
+
+        self.declare_partials('y1', ['y2', 'x'])
+
+    def compute(self, inputs, outputs):
+        y2 = inputs['y2']
+        x = inputs['x']
+
+        if self.comm.rank == 1:
+            outputs['y1'] = 18.0 - 0.2*y2 + 2*x
+        else:
+            outputs['y1'] = 28.0 - 0.2*y2 + x
+
+    def compute_partials(self, inputs, partials, discrete_inputs=None):
+        y2 = inputs['y2']
+        x = inputs['x']
+
+        partials['y1', 'y2'] = -0.2
+        if self.comm.rank == 1:
+            partials['y1', 'x'] = 2.0
+        else:
+            partials['y1', 'x'] = 1.0
+
+
+class D2(om.ExplicitComponent):
+    def initialize(self):
+        self.options['distributed'] = True
+
+    def setup(self):
+        comm = self.comm
+        rank = comm.rank
+
+        if rank == 1:
+            start = 1
+            end = 2
+        else:
+            start = 0
+            end = 1
+
+        self.add_input('y1', np.ones((1, ), float),
+                       src_indices=np.arange(start, end, dtype=int))
+
+        self.add_output('y2', np.ones((1, ), float))
+
+        self.declare_partials('y2', ['y1'])
+
+    def compute(self, inputs, outputs):
+        y1 = inputs['y1']
+
+        if self.comm.rank == 1:
+            outputs['y2'] = y2 = y1**.5 - 3
+        else:
+            outputs['y2'] = y1**.5 + 7
+
+    def compute_partials(self, inputs, partials, discrete_inputs=None):
+        y1 = inputs['y1']
+
+        partials['y2', 'y1'] = 0.5 / y1**.5
+
+
+class Summer(om.ExplicitComponent):
+    def setup(self):
+        self.add_input('y1', val=np.zeros((2, )))
+        self.add_input('y2', val=np.zeros((2, )))
+        self.add_output('obj', 0.0, shape=1)
+
+        self.declare_partials('obj', 'y1', rows=np.array([0, 0]), cols=np.array([0, 1]), val=np.ones((2, )))
+
+    def compute(self, inputs, outputs):
+        outputs['obj'] = np.sum(inputs['y1']) + np.sum(inputs['y2'])
+
+
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
 class MPITestSimpleGA4Procs(unittest.TestCase):
 
     N_PROCS = 4
@@ -838,6 +1063,10 @@ class MPITestSimpleGA4Procs(unittest.TestCase):
         prob.setup()
         prob.run_driver()
 
+        if extra_prints:
+            print('comp.f', prob['comp.f'])
+            print('p2.xI', prob['p2.xI'])
+
         # Optimal solution
         assert_rel_error(self, prob['comp.f'], 0.98799098, 1e-4)
         self.assertTrue(int(prob['p2.xI']) in [3, -3])
@@ -892,6 +1121,34 @@ class MPITestSimpleGA4Procs(unittest.TestCase):
         # No meaningful result from a short run; just make sure we don't hang.
         prob.run_driver()
 
+    def test_proc_per_model(self):
+        # Test that we can run a GA on a distributed component without lockups.
+        prob = om.Problem()
+        model = prob.model
+
+        model.add_subsystem('p', om.IndepVarComp('x', 3.0), promotes=['x'])
+
+        model.add_subsystem('d1', D1(), promotes=['*'])
+        model.add_subsystem('d2', D2(), promotes=['*'])
+
+        model.add_subsystem('obj_comp', Summer(), promotes=['*'])
+        model.nonlinear_solver = om.NewtonSolver()
+        model.linear_solver = om.LinearBlockGS()
+
+        model.add_design_var('x', lower=-0.5, upper=0.5)
+        model.add_objective('obj')
+
+        driver = prob.driver = om.SimpleGADriver()
+        prob.driver.options['pop_size'] = 4
+        prob.driver.options['max_gen'] = 3
+        prob.driver.options['run_parallel'] = True
+        prob.driver.options['procs_per_model'] = 2
+
+        prob.setup()
+        prob.set_solver_print(level=0)
+
+        prob.run_driver()
+
 
 class TestFeatureSimpleGA(unittest.TestCase):
 
@@ -926,11 +1183,6 @@ class TestFeatureSimpleGA(unittest.TestCase):
         prob.setup()
         prob.run_driver()
 
-        # Optimal solution
-        print('comp.f', prob['comp.f'])
-        print('p2.xI', prob['p2.xI'])
-        print('p1.xC', prob['p1.xC'])
-
     def test_basic_with_assert(self):
         import openmdao.api as om
         from openmdao.test_suite.components.branin import Branin
@@ -985,11 +1237,6 @@ class TestFeatureSimpleGA(unittest.TestCase):
         prob.setup()
         prob.run_driver()
 
-        # Optimal solution
-        print('comp.f', prob['comp.f'])
-        print('p2.xI', prob['p2.xI'])
-        print('p1.xC', prob['p1.xC'])
-
     def test_option_pop_size(self):
         import openmdao.api as om
         from openmdao.test_suite.components.branin import Branin
@@ -1015,11 +1262,6 @@ class TestFeatureSimpleGA(unittest.TestCase):
         prob.setup()
         prob.run_driver()
 
-        # Optimal solution
-        print('comp.f', prob['comp.f'])
-        print('p2.xI', prob['p2.xI'])
-        print('p1.xC', prob['p1.xC'])
-
     def test_constrained_with_penalty(self):
         import openmdao.api as om
 
@@ -1070,8 +1312,7 @@ class TestFeatureSimpleGA(unittest.TestCase):
         self.assertGreater(prob['height'], 1.)
 
 
-@unittest.skipUnless(om.PETScVector, "PETSc is required.")
-@unittest.skipUnless(MPI, "MPI is required.")
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
 class MPIFeatureTests(unittest.TestCase):
     N_PROCS = 2
 
@@ -1109,13 +1350,13 @@ class MPIFeatureTests(unittest.TestCase):
         prob.run_driver()
 
         # Optimal solution
-        print('comp.f', prob['comp.f'])
-        print('p2.xI', prob['p2.xI'])
-        print('p1.xC', prob['p1.xC'])
+        if extra_prints:
+            print('comp.f', prob['comp.f'])
+            print('p2.xI', prob['p2.xI'])
+            print('p1.xC', prob['p1.xC'])
 
 
-@unittest.skipUnless(om.PETScVector, "PETSc is required.")
-@unittest.skipUnless(MPI, "MPI is required.")
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
 class MPIFeatureTests4(unittest.TestCase):
     N_PROCS = 4
 
@@ -1166,9 +1407,10 @@ class MPIFeatureTests4(unittest.TestCase):
         prob.run_driver()
 
         # Optimal solution
-        print('comp.f', prob['comp.f'])
-        print('p2.xI', prob['p2.xI'])
-        print('p1.xC', prob['p1.xC'])
+        if extra_prints:
+            print('comp.f', prob['comp.f'])
+            print('p2.xI', prob['p2.xI'])
+            print('p1.xC', prob['p1.xC'])
 
 
 if __name__ == "__main__":
