@@ -412,7 +412,8 @@ def assertViewerDataRecorded(test, expected):
         test.assertEqual(expected['tree_length'], len(tr))
 
         test.assertEqual({'name', 'type', 'subsystem_type', 'children', 'linear_solver',
-                          'nonlinear_solver', 'is_parallel', 'component_type', 'class'},
+                          'nonlinear_solver', 'is_parallel', 'component_type', 'class',
+                          'expressions'},
                          set(tr.keys()))
         test.assertEqual(expected['tree_children_length'],
                          len(model_viewer_data['tree']['children']))
