@@ -115,7 +115,7 @@ class DistribStateImplicit(om.ImplicitComponent):
             d_residuals.set_vec(d_outputs)
 
 
-@unittest.skipUnless(PETScVector, "PETSc is required.")
+@unittest.skipUnless(PETScVector is not None, "PETSc is required.")
 class TestUserDefinedSolver(unittest.TestCase):
 
     def test_method(self):
