@@ -25,7 +25,7 @@ class TestArmejoGoldsteinBounds(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStates())
         top.model.connect('px.x', 'comp.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -167,7 +167,7 @@ class TestAnalysisErrorExplicit(unittest.TestCase):
         top.model.connect('px.x', 'comp.x')
         top.model.connect('comp.z', 'par.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 1
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -278,7 +278,7 @@ class TestAnalysisErrorImplicit(unittest.TestCase):
         top.model.nonlinear_solver.options['solve_subsystems'] = True
         top.model.linear_solver = om.ScipyKrylov()
 
-        sub.nonlinear_solver = om.NewtonSolver()
+        sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         sub.nonlinear_solver.options['maxiter'] = 2
         sub.linear_solver = om.ScipyKrylov()
 
@@ -327,7 +327,7 @@ class TestAnalysisErrorImplicit(unittest.TestCase):
         top.model.nonlinear_solver.options['solve_subsystems'] = True
         top.model.linear_solver = om.ScipyKrylov()
 
-        sub.nonlinear_solver = om.NewtonSolver()
+        sub.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         sub.nonlinear_solver.options['maxiter'] = 2
         sub.linear_solver = om.ScipyKrylov()
 
@@ -368,7 +368,7 @@ class TestBoundsEnforceLSArrayBounds(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
         top.model.connect('px.x', 'comp.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -503,7 +503,7 @@ class TestBoundsEnforceLSArrayBounds(unittest.TestCase):
         top.model.connect('px.x', 'comp.x')
         top.model.connect('comp.z', 'par.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 3
 
         top.model.nonlinear_solver.linesearch = om.BoundsEnforceLS(bound_enforcement='vector')
@@ -580,7 +580,7 @@ class DoubleSellarMod(om.Group):
         self.connect('g2.y2', 'g1.x')
 
         # Converge the outer loop with Gauss Seidel, with a looser tolerance.
-        self.nonlinear_solver = om.NewtonSolver()
+        self.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         self.linear_solver = om.DirectSolver()
 
 
@@ -592,7 +592,7 @@ class TestArmijoGoldsteinLSArrayBounds(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
         top.model.connect('px.x', 'comp.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -681,12 +681,12 @@ class TestArmijoGoldsteinLSArrayBounds(unittest.TestCase):
         model = prob.model = DoubleSellarMod()
 
         g1 = model.g1
-        g1.nonlinear_solver = om.NewtonSolver()
+        g1.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         g1.nonlinear_solver.options['rtol'] = 1.0e-5
         g1.linear_solver = om.DirectSolver()
 
         g2 = model.g2
-        g2.nonlinear_solver = om.NewtonSolver()
+        g2.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         g2.nonlinear_solver.options['rtol'] = 1.0e-5
         g2.linear_solver = om.DirectSolver()
 
@@ -788,7 +788,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
         top.model.connect('px.x', 'comp.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -816,7 +816,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
         top.model.connect('px.x', 'comp.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -844,7 +844,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
         top.model.connect('px.x', 'comp.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -872,7 +872,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
         top.model.connect('px.x', 'comp.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -900,7 +900,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
         top.model.connect('px.x', 'comp.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -929,7 +929,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
         top.model.connect('px.x', 'comp.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -955,7 +955,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
         top.model.connect('px.x', 'comp.x')
 
-        newt = top.model.nonlinear_solver = om.NewtonSolver()
+        newt = top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 2
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -985,7 +985,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
         top.model.connect('px.x', 'comp.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -1013,7 +1013,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
         top.model.connect('px.x', 'comp.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -1042,7 +1042,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
         top.model.connect('px.x', 'comp.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -1068,7 +1068,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
         top.model.connect('px.x', 'comp.x')
 
-        newt = top.model.nonlinear_solver = om.NewtonSolver()
+        newt = top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 2
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -1098,7 +1098,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
         top.model.connect('px.x', 'comp.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
