@@ -1,6 +1,8 @@
 """Test the Newton nonlinear solver. """
 
 import unittest
+import warnings
+
 
 import numpy as np
 
@@ -12,14 +14,20 @@ from openmdao.test_suite.components.implicit_newton_linesearch import ImplCompTw
 from openmdao.test_suite.components.sellar import SellarDerivativesGrouped, \
      SellarNoDerivatives, SellarDerivatives, SellarStateConnection, StateConnection, \
      SellarDis1withDerivatives, SellarDis2withDerivatives
-from openmdao.utils.assert_utils import assert_rel_error, assert_warning
+from openmdao.utils.assert_utils import assert_rel_error, assert_warning, assert_no_warning
+from openmdao.utils.mpi import MPI
+
+try:
+    from openmdao.vectors.petsc_vector import PETScVector
+except ImportError:
+    PETScVector = None
 
 
 class TestNewton(unittest.TestCase):
 
     def test_specify_newton_linear_solver_in_system(self):
 
-        my_newton = om.NewtonSolver()
+        my_newton = om.NewtonSolver(solve_subsystems=False)
         my_newton.linear_solver = om.DirectSolver()
 
         prob = om.Problem(model=SellarDerivatives(nonlinear_solver=my_newton))
@@ -40,7 +48,7 @@ class TestNewton(unittest.TestCase):
         import openmdao.api as om
         from openmdao.test_suite.components.sellar import SellarDerivatives
 
-        prob = om.Problem(model=SellarDerivatives(nonlinear_solver=om.NewtonSolver()))
+        prob = om.Problem(model=SellarDerivatives(nonlinear_solver=om.NewtonSolver(solve_subsystems=False)))
 
         prob.setup()
         prob.run_model()
@@ -51,7 +59,7 @@ class TestNewton(unittest.TestCase):
     def test_sellar_grouped(self):
         # Tests basic Newton solution on Sellar in a subgroup
 
-        prob = om.Problem(model=SellarDerivativesGrouped(nonlinear_solver=om.NewtonSolver()))
+        prob = om.Problem(model=SellarDerivativesGrouped(nonlinear_solver=om.NewtonSolver(solve_subsystems=False)))
 
         prob.setup()
         prob.set_solver_print(level=0)
@@ -66,7 +74,7 @@ class TestNewton(unittest.TestCase):
     def test_sellar(self):
         # Just tests Newton on Sellar with FD derivs.
 
-        prob = om.Problem(model=SellarNoDerivatives(nonlinear_solver=om.NewtonSolver()))
+        prob = om.Problem(model=SellarNoDerivatives(nonlinear_solver=om.NewtonSolver(solve_subsystems=False)))
 
         prob.setup()
         prob.run_model()
@@ -83,7 +91,7 @@ class TestNewton(unittest.TestCase):
         top.model.add_subsystem('comp', ImplCompTwoStates())
         top.model.connect('px.x', 'comp.x')
 
-        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
@@ -121,7 +129,7 @@ class TestNewton(unittest.TestCase):
         # on the head component behind the cycle break.
 
         prob = om.Problem()
-        prob.model = SellarDerivatives(nonlinear_solver=om.NewtonSolver(),
+        prob.model = SellarDerivatives(nonlinear_solver=om.NewtonSolver(solve_subsystems=False),
                                        linear_solver=om.LinearBlockGS())
 
         prob.setup()
@@ -144,7 +152,7 @@ class TestNewton(unittest.TestCase):
 
     def test_sellar_derivs_with_Lin_GS(self):
 
-        prob = om.Problem(model=SellarDerivatives(nonlinear_solver=om.NewtonSolver()))
+        prob = om.Problem(model=SellarDerivatives(nonlinear_solver=om.NewtonSolver(solve_subsystems=False)))
 
         prob.setup()
         prob.set_solver_print(level=0)
@@ -159,7 +167,7 @@ class TestNewton(unittest.TestCase):
     def test_sellar_state_connection(self):
         # Sellar model closes loop with state connection instead of a cycle.
 
-        prob = om.Problem(model=SellarStateConnection(nonlinear_solver=om.NewtonSolver()))
+        prob = om.Problem(model=SellarStateConnection(nonlinear_solver=om.NewtonSolver(solve_subsystems=False)))
 
         prob.set_solver_print(level=0)
         prob.setup()
@@ -174,7 +182,7 @@ class TestNewton(unittest.TestCase):
     def test_sellar_state_connection_fd_system(self):
         # Sellar model closes loop with state connection instead of a cycle.
         # This test is just fd.
-        prob = om.Problem(model=SellarStateConnection(nonlinear_solver=om.NewtonSolver()))
+        prob = om.Problem(model=SellarStateConnection(nonlinear_solver=om.NewtonSolver(solve_subsystems=False)))
 
         prob.model.approx_totals(method='fd')
 
@@ -219,7 +227,7 @@ class TestNewton(unittest.TestCase):
         model.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2'])
         model.connect('d2.y2', 'con_cmp2.y2')
 
-        model.nonlinear_solver = om.NewtonSolver()
+        model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
 
         # Use bad settings for this one so that problem doesn't converge.
         # That way, we test that we are really using Newton's Lin Solver
@@ -273,7 +281,7 @@ class TestNewton(unittest.TestCase):
         model.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2'])
         model.connect('d2.y2', 'con_cmp2.y2')
 
-        model.nonlinear_solver = om.NewtonSolver()
+        model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
 
         # Use bad settings for this one so that problem doesn't converge.
         # That way, we test that we are really using Newton's Lin Solver
@@ -300,13 +308,13 @@ class TestNewton(unittest.TestCase):
         model = prob.model
 
         g1 = model.g1
-        g1.nonlinear_solver = om.NewtonSolver()
+        g1.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         g1.nonlinear_solver.options['rtol'] = 1.0e-5
         g1.linear_solver = om.DirectSolver(assemble_jac=True)
         g1.options['assembled_jac_type'] = 'dense'
 
         g2 = model.g2
-        g2.nonlinear_solver = om.NewtonSolver()
+        g2.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         g2.nonlinear_solver.options['rtol'] = 1.0e-5
         g2.linear_solver = om.DirectSolver(assemble_jac=True)
         g2.options['assembled_jac_type'] = 'dense'
@@ -330,12 +338,12 @@ class TestNewton(unittest.TestCase):
         model = prob.model
 
         g1 = model.g1
-        g1.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g1.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g1.options['assembled_jac_type'] = 'dense'
         g1.linear_solver = om.DirectSolver(assemble_jac=True)
 
         g2 = model.g2
-        g2.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g2.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g2.linear_solver = om.DirectSolver(assemble_jac=True)
         g2.options['assembled_jac_type'] = 'dense'
 
@@ -355,11 +363,11 @@ class TestNewton(unittest.TestCase):
         model = prob.model
 
         g1 = model.g1
-        g1.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g1.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g1.linear_solver = om.DirectSolver()
 
         g2 = model.g2
-        g2.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g2.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g2.linear_solver = om.DirectSolver()
 
         model.nonlinear_solver = om.NewtonSolver(solve_subsystems=True)
@@ -379,11 +387,11 @@ class TestNewton(unittest.TestCase):
         model = prob.model
 
         g1 = model.g1
-        g1.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g1.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g1.linear_solver = om.DirectSolver()
 
         g2 = model.g2
-        g2.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g2.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g2.linear_solver = om.DirectSolver()
 
         model.nonlinear_solver = om.NewtonSolver(solve_subsystems=True)
@@ -403,11 +411,11 @@ class TestNewton(unittest.TestCase):
         model = prob.model
 
         g1 = model.g1
-        g1.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g1.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g1.linear_solver = om.DirectSolver()
 
         g2 = model.g2
-        g2.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g2.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g2.linear_solver = om.DirectSolver()
 
         model.nonlinear_solver = om.NewtonSolver(solve_subsystems=True)
@@ -427,11 +435,11 @@ class TestNewton(unittest.TestCase):
         model = prob.model
 
         g1 = model.g1
-        g1.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g1.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g1.linear_solver = om.DirectSolver()
 
         g2 = model.g2
-        g2.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g2.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g2.linear_solver = om.DirectSolver()
 
         model.nonlinear_solver = om.NewtonSolver(solve_subsystems=True)
@@ -451,11 +459,11 @@ class TestNewton(unittest.TestCase):
         model = prob.model
 
         g1 = model.g1
-        g1.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g1.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g1.linear_solver = om.DirectSolver()
 
         g2 = model.g2
-        g2.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g2.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g2.linear_solver = om.DirectSolver()
 
         model.nonlinear_solver = om.NewtonSolver(solve_subsystems=True)
@@ -474,11 +482,11 @@ class TestNewton(unittest.TestCase):
         model = prob.model
 
         g1 = model.g1
-        g1.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g1.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g1.linear_solver = om.DirectSolver()
 
         g2 = model.g2
-        g2.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g2.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g2.linear_solver = om.DirectSolver()
 
         model.nonlinear_solver = om.NewtonSolver(solve_subsystems=True)
@@ -498,11 +506,11 @@ class TestNewton(unittest.TestCase):
         model = prob.model
 
         g1 = model.g1
-        g1.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g1.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g1.linear_solver = om.DirectSolver()
 
         g2 = model.g2
-        g2.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g2.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g2.linear_solver = om.DirectSolver()
 
         model.nonlinear_solver = om.NewtonSolver(solve_subsystems=True)
@@ -522,11 +530,11 @@ class TestNewton(unittest.TestCase):
         model = prob.model
 
         g1 = model.g1
-        g1.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g1.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g1.linear_solver = om.DirectSolver()
 
         g2 = model.g2
-        g2.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g2.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g2.linear_solver = om.DirectSolver()
 
         model.nonlinear_solver = om.NewtonSolver(solve_subsystems=True)
@@ -546,15 +554,15 @@ class TestNewton(unittest.TestCase):
         model = prob.model
 
         g1 = model.g1
-        g1.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g1.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g1.linear_solver = om.DirectSolver(assemble_jac=True)
         model.options['assembled_jac_type'] = 'dense'
 
         g2 = model.g2
-        g2.nonlinear_solver = om.NewtonSolver(rtol=1.0e-5)
+        g2.nonlinear_solver = om.NewtonSolver(solve_subsystems=False, rtol=1.0e-5)
         g2.linear_solver = om.DirectSolver()
 
-        model.nonlinear_solver = om.NewtonSolver()
+        model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         model.linear_solver = om.ScipyKrylov()
 
         prob.setup()
@@ -574,6 +582,7 @@ class TestNewton(unittest.TestCase):
 
             def __init__(self, **kwargs):
                 super(CountNewton, self).__init__(**kwargs)
+                self.options['solve_subsystems'] = True
                 self.total_count = 0
 
             def _single_iteration(self):
@@ -725,7 +734,7 @@ class TestNewton(unittest.TestCase):
         root.add_subsystem('comp', ImpComp())
         root.connect('p1.a', 'comp.a')
 
-        root.nonlinear_solver = om.NewtonSolver()
+        root.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         root.nonlinear_solver.options['maxiter'] = 1
         prob.set_solver_print(level=0)
 
@@ -736,7 +745,7 @@ class TestNewton(unittest.TestCase):
         # Raise AnalysisError when it fails to converge
 
         prob = om.Problem()
-        nlsolver = om.NewtonSolver()
+        nlsolver = om.NewtonSolver(solve_subsystems=False)
         prob.model = SellarDerivatives(nonlinear_solver=nlsolver,
                                        linear_solver=om.LinearBlockGS())
 
@@ -764,7 +773,7 @@ class TestNewton(unittest.TestCase):
         # Raise AnalysisError when it fails to converge
 
         prob = om.Problem()
-        nlsolver = om.NewtonSolver()
+        nlsolver = om.NewtonSolver(solve_subsystems=False)
         prob.model = SellarDerivatives(nonlinear_solver=nlsolver,
                                        linear_solver=om.LinearBlockGS())
 
@@ -780,10 +789,60 @@ class TestNewton(unittest.TestCase):
         msg = "Solver 'NL: Newton' on system '' failed to converge in 1 iterations."
         self.assertEqual(str(context.exception), msg)
 
+    def test_reraise_child_analysiserror(self):
+        # Raise AnalysisError when it fails to converge
+
+        prob = om.Problem(model=DoubleSellar())
+        model = prob.model
+
+        g1 = model.g1
+        g1.nonlinear_solver = om.NewtonSolver()
+        g1.nonlinear_solver.options['maxiter'] = 1
+        g1.nonlinear_solver.options['err_on_non_converge'] = True
+        g1.nonlinear_solver.options['solve_subsystems'] = True
+        g1.linear_solver = om.DirectSolver(assemble_jac=True)
+
+        g2 = model.g2
+        g2.nonlinear_solver = om.NewtonSolver()
+        g2.nonlinear_solver.options['maxiter'] = 1
+        g2.nonlinear_solver.options['err_on_non_converge'] = True
+        g2.nonlinear_solver.options['solve_subsystems'] = True
+        g2.linear_solver = om.DirectSolver(assemble_jac=True)
+
+        model.nonlinear_solver = om.NewtonSolver()
+        model.linear_solver = om.ScipyKrylov(assemble_jac=True)
+        model.nonlinear_solver.options['solve_subsystems'] = True
+        model.nonlinear_solver.options['err_on_non_converge'] = True
+        model.nonlinear_solver.options['reraise_child_analysiserror'] = True
+
+        prob.setup()
+
+        with self.assertRaises(om.AnalysisError) as context:
+            prob.run_model()
+
+        msg = "Solver 'NL: Newton' on system 'g1' failed to converge in 1 iterations."
+        self.assertEqual(str(context.exception), msg)
+
+    def test_reraise_child_analysiserror_deprecation_warning(self):
+
+        prob = om.Problem()
+        model = prob.model
+
+        model.nonlinear_solver = om.NewtonSolver()
+        model.linear_solver = om.ScipyKrylov(assemble_jac=True)
+        model.nonlinear_solver.options['solve_subsystems'] = True
+        model.nonlinear_solver.options['err_on_non_converge'] = True
+
+        prob.setup()
+
+        msg = "Deprecation warning: In V 3.x, reraise_child_analysiserror will default to False."
+        with assert_warning(DeprecationWarning, msg):
+            prob.run_model()
+
     def test_err_message_inf_nan(self):
 
         prob = om.Problem()
-        nlsolver = om.NewtonSolver()
+        nlsolver = om.NewtonSolver(solve_subsystems=False)
         prob.model = SellarDerivatives(nonlinear_solver=nlsolver,
                                        linear_solver=om.LinearBlockGS())
 
@@ -834,7 +893,7 @@ class TestNewton(unittest.TestCase):
         model.add_design_var('p1.x', 3.0)
         model.add_objective('ecomp.y')
 
-        model.nonlinear_solver = om.NewtonSolver()
+        model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         model.linear_solver = om.ScipyKrylov()
 
         prob.setup()
@@ -844,6 +903,37 @@ class TestNewton(unittest.TestCase):
         J = prob.compute_totals()
         assert_rel_error(self, J['ecomp.y', 'p1.x'][0][0], -0.703467422498, 1e-6)
 
+    def test_linsearch_3_deprecation(self):
+        prob = om.Problem(model=SellarDerivatives(nonlinear_solver=om.NewtonSolver(solve_subsystems=False)))
+
+        prob.setup()
+
+        msg = 'Deprecation warning: In V 3.0, the default Newton solver setup will change ' + \
+              'to use the BoundsEnforceLS line search.'
+
+        with assert_warning(DeprecationWarning, msg):
+            prob.final_setup()
+
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
+class MPITestCase(unittest.TestCase):
+    N_PROCS = 4
+
+    def test_comm_warning(self):
+
+        rank = MPI.COMM_WORLD.rank if MPI is not None else 0
+
+        prob = om.Problem(model=SellarDerivatives(nonlinear_solver=om.NewtonSolver(solve_subsystems=False)))
+
+        prob.setup()
+
+        msg = 'Deprecation warning: In V 3.0, the default Newton solver setup will change ' + \
+              'to use the BoundsEnforceLS line search.'
+        if rank == 0:
+            with assert_warning(DeprecationWarning, msg):
+                prob.final_setup()
+        else:
+            with assert_no_warning(DeprecationWarning, msg):
+                prob.final_setup()
 
 class TestNewtonFeatures(unittest.TestCase):
 
@@ -871,7 +961,7 @@ class TestNewtonFeatures(unittest.TestCase):
 
         model.linear_solver = om.DirectSolver()
 
-        model.nonlinear_solver = om.NewtonSolver()
+        model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
 
         prob.setup()
 
@@ -904,7 +994,7 @@ class TestNewtonFeatures(unittest.TestCase):
 
         model.linear_solver = om.DirectSolver()
 
-        nlgbs = model.nonlinear_solver = om.NewtonSolver()
+        nlgbs = model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         nlgbs.options['maxiter'] = 2
 
         prob.setup()
@@ -938,7 +1028,7 @@ class TestNewtonFeatures(unittest.TestCase):
 
         model.linear_solver = om.DirectSolver()
 
-        nlgbs = model.nonlinear_solver = om.NewtonSolver()
+        nlgbs = model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         nlgbs.options['rtol'] = 1e-3
 
         prob.setup()
@@ -972,7 +1062,7 @@ class TestNewtonFeatures(unittest.TestCase):
 
         model.linear_solver = om.DirectSolver()
 
-        nlgbs = model.nonlinear_solver = om.NewtonSolver()
+        nlgbs = model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         nlgbs.options['atol'] = 1e-4
 
         prob.setup()
@@ -1007,7 +1097,7 @@ class TestNewtonFeatures(unittest.TestCase):
 
         model.linear_solver = om.LinearBlockGS()
 
-        nlgbs = model.nonlinear_solver = om.NewtonSolver()
+        nlgbs = model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
 
         nlgbs.linear_solver = om.DirectSolver()
 
@@ -1038,12 +1128,12 @@ class TestNewtonFeatures(unittest.TestCase):
         model.linear_solver = om.DirectSolver()
 
         g1 = model.g1
-        g1.nonlinear_solver = om.NewtonSolver()
+        g1.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         g1.nonlinear_solver.options['rtol'] = 1.0e-5
         g1.linear_solver = om.DirectSolver()
 
         g2 = model.g2
-        g2.nonlinear_solver = om.NewtonSolver()
+        g2.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         g2.nonlinear_solver.options['rtol'] = 1.0e-5
         g2.linear_solver = om.DirectSolver()
 
@@ -1080,7 +1170,7 @@ class TestNewtonFeatures(unittest.TestCase):
 
         model.linear_solver = om.DirectSolver()
 
-        nlgbs = model.nonlinear_solver = om.NewtonSolver()
+        nlgbs = model.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         nlgbs.options['maxiter'] = 1
         nlgbs.options['err_on_non_converge'] = True
 
@@ -1099,12 +1189,12 @@ class TestNewtonFeatures(unittest.TestCase):
         model = prob.model
 
         g1 = model.g1
-        g1.nonlinear_solver = om.NewtonSolver()
+        g1.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         g1.nonlinear_solver.options['rtol'] = 1.0e-5
         g1.linear_solver = om.DirectSolver()
 
         g2 = model.g2
-        g2.nonlinear_solver = om.NewtonSolver()
+        g2.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
         g2.nonlinear_solver.options['rtol'] = 1.0e-5
         g2.linear_solver = om.DirectSolver()
 
