@@ -153,12 +153,11 @@ class TestNLBlockJacobi(unittest.TestCase):
         assert_rel_error(self, prob['y2'], 12.05848819, .00001)
 
 
-@unittest.skipUnless(PETScVector, "PETSc is required.")
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
 class TestNonlinearBlockJacobiMPI(unittest.TestCase):
 
     N_PROCS = 2
 
-    @unittest.skipUnless(MPI, "MPI is not active.")
     def test_reraise_analylsis_error(self):
         prob = om.Problem()
         model = prob.model
