@@ -383,12 +383,12 @@ def _flatten_src_indices(src_indices, shape_in, shape_out, size_out):
         return convert_neg(src_indices.flatten(), size_out)
 
     entries = [list(range(x)) for x in shape_in]
-    cols = np.vstack(src_indices[i] for i in product(*entries))
+    cols = np.vstack([src_indices[i] for i in product(*entries)])
     dimidxs = [convert_neg(cols[:, i], shape_out[i]) for i in range(cols.shape[1])]
     return np.ravel_multi_index(dimidxs, shape_out)
 
 
-def sizes2offsets(size_array, dtype=int):
+def sizes2offsets(size_array):
     """
     For a given array of sizes, return an array of offsets.
 
@@ -399,15 +399,13 @@ def sizes2offsets(size_array, dtype=int):
     ----------
     size_array : ndarray
         Array of sizes.
-    dtype : type
-        numpy dtype of size_array.
 
     Returns
     -------
     ndarray
         Array of offsets.
     """
-    offsets = np.zeros(size_array.size, dtype=dtype)
+    offsets = np.zeros(size_array.size, dtype=size_array.dtype)
     offsets[1:] = np.cumsum(size_array.flat)[:-1]
     return offsets.reshape(size_array.shape)
 
@@ -435,7 +433,7 @@ def abs_complex(x):
 
 def dv_abs_complex(x, x_deriv):
     """
-    Apply the complex-step derivative of the absolute value function.
+    Compute the complex-step derivative of the absolute value function and its derivative.
 
     Parameters
     ----------
@@ -446,6 +444,8 @@ def dv_abs_complex(x, x_deriv):
 
     Returns
     -------
+    ndarray
+        Absolute value applied to x.
     ndarray
         Absolute value applied to x_deriv.
     """
@@ -454,8 +454,9 @@ def dv_abs_complex(x, x_deriv):
     # Special case when x is (1, ) and x_deriv is (1, n).
     if len(x_deriv.shape) == 1:
         if idx_neg[0].size != 0:
-            return -x_deriv
+            return -x, -x_deriv
 
+    x[idx_neg] = -x[idx_neg]
     x_deriv[idx_neg] = -x_deriv[idx_neg]
 
-    return x_deriv
+    return x, x_deriv
