@@ -6,37 +6,47 @@ from __future__ import print_function
 import sys
 import os
 import argparse
+try:
+    import pkg_resources
+except ImportError:
+    pkg_resources = None
+
 from itertools import chain
 from six import iteritems
 
-from openmdao.core.problem import Problem
+import openmdao.utils.hooks as hooks
 from openmdao.visualization.n2_viewer.n2_viewer import n2
 from openmdao.visualization.connection_viewer.viewconns import view_connections
-from openmdao.components.meta_model_unstructured_comp import MetaModelUnStructuredComp
-from openmdao.components.meta_model_structured_comp import MetaModelStructuredComp
+from openmdao.visualization.xdsm_viewer.xdsm_writer import write_xdsm, \
+    _DEFAULT_BOX_STACKING, _DEFAULT_BOX_WIDTH, _MAX_BOX_LINES, _DEFAULT_OUTPUT_SIDE, _CHAR_SUBS
 try:
     import bokeh
     from openmdao.visualization.meta_model_viewer.meta_model_visualization import view_metamodel
 except ImportError:
     bokeh = None
-from openmdao.devtools.debug import config_summary, tree, dump_dist_idxs
+from openmdao.components.meta_model_unstructured_comp import MetaModelUnStructuredComp
+from openmdao.components.meta_model_structured_comp import MetaModelStructuredComp
+from openmdao.devtools.debug import config_summary, tree
 from openmdao.devtools.itrace import _itrace_exec, _itrace_setup_parser
 from openmdao.devtools.iprofile_app.iprofile_app import _iprof_exec, _iprof_setup_parser
 from openmdao.devtools.iprofile import _iprof_totals_exec, _iprof_totals_setup_parser
 from openmdao.devtools.iprof_mem import _mem_prof_exec, _mem_prof_setup_parser, \
     _mempost_exec, _mempost_setup_parser
 from openmdao.devtools.iprof_utils import _Options
-from openmdao.visualization.xdsm_viewer.xdsm_writer import write_xdsm, \
-    _DEFAULT_BOX_STACKING, _DEFAULT_BOX_WIDTH, _MAX_BOX_LINES, _DEFAULT_OUTPUT_SIDE, _CHAR_SUBS
 from openmdao.error_checking.check_config import _check_config_cmd, _check_config_setup_parser
 from openmdao.utils.mpi import MPI
 from openmdao.utils.find_cite import print_citations
 from openmdao.utils.code_utils import _calltree_setup_parser, _calltree_exec
 from openmdao.utils.coloring import _total_coloring_setup_parser, _total_coloring_cmd, \
-    _sparsity_setup_parser, _sparsity_cmd, _partial_coloring_setup_parser, _partial_coloring_cmd, \
+    _partial_coloring_setup_parser, _partial_coloring_cmd, \
     _view_coloring_setup_parser, _view_coloring_exec
 from openmdao.utils.scaffold import _scaffold_setup_parser, _scaffold_exec
 from openmdao.utils.general_utils import warn_deprecation
+from openmdao.utils.file_utils import _load_and_exec
+from openmdao.utils.entry_points import _list_installed_setup_parser, _list_installed_cmd, \
+    split_ep, _compute_entry_points_setup_parser, _compute_entry_points_exec, \
+        _find_plugins_setup_parser, _find_plugins_exec
+from openmdao.core.component import Component
 
 
 def _n2_setup_parser(parser):
@@ -62,7 +72,7 @@ def _n2_setup_parser(parser):
                         help="use declare partial info for internal connectivity.")
 
 
-def _n2_cmd(options):
+def _n2_cmd(options, user_args):
     """
     Process command line args and call n2 on the specified file.
 
@@ -70,6 +80,8 @@ def _n2_cmd(options):
     ----------
     options : argparse Namespace
         Command line options.
+    user_args : list of str
+        Command line options after '--' (if any).  Passed to user script.
     """
     filename = options.file[0]
 
@@ -81,8 +93,9 @@ def _n2_cmd(options):
                use_declare_partial_info=options.use_declare_partial_info)
             exit()  # could make this command line selectable later
 
-        options.func = lambda options: _viewmod
-        _post_setup_exec(options)
+        hooks._register_hook('final_setup', 'Problem', post=_viewmod)
+
+        _load_and_exec(filename, user_args)
     else:
         # assume the file is a recording, run standalone
         n2(filename, outfile=options.outfile, title=options.title,
@@ -90,9 +103,9 @@ def _n2_cmd(options):
            use_declare_partial_info=options.use_declare_partial_info)
 
 
-def _view_model_cmd(options):
+def _view_model_cmd(options, user_args):
     warn_deprecation("The 'view_model' command has been deprecated. Use 'n2' instead.")
-    _n2_cmd(options)
+    _n2_cmd(options, user_args)
 
 
 def _xdsm_setup_parser(parser):
@@ -149,10 +162,12 @@ def _xdsm_setup_parser(parser):
                         help='If True, show legend.')
     parser.add_argument('--class_names', action='store_true', dest='class_names',
                         help='If true, appends class name of the groups/components to the '
-                             'component blocks of the diagram..')
+                             'component blocks of the diagram.')
+    parser.add_argument('--equations', action='store_true', dest='equations',
+                        help='If true, for ExecComps their equations are shown in the diagram.')
 
 
-def _xdsm_cmd(options):
+def _xdsm_cmd(options, user_args):
     """
     Process command line args and call xdsm on the specified file.
 
@@ -160,6 +175,8 @@ def _xdsm_cmd(options):
     ----------
     options : argparse Namespace
         Command line options.
+    user_args : list of str
+        Command line options after '--' (if any).  Passed to user script.
     """
     filename = options.file[0]
 
@@ -182,12 +199,13 @@ def _xdsm_cmd(options):
                        output_side=options.output_side,
                        legend=options.legend,
                        class_names=options.class_names,
+                       equations=options.equations,
                        **kwargs)
             exit()
 
-        options.func = lambda options: _xdsm
+        hooks._register_hook('setup', 'Problem', post=_xdsm)
 
-        _post_setup_exec(options)
+        _load_and_exec(filename, user_args)
     else:
         # assume the file is a recording, run standalone
         write_xdsm(filename, filename=options.outfile, model_path=options.model_path,
@@ -197,6 +215,9 @@ def _xdsm_cmd(options):
                    include_solver=options.include_solver, subs=_CHAR_SUBS,
                    show_browser=not options.no_browser, show_parallel=not options.no_parallel,
                    add_process_conns=not options.no_process_conns, output_side=options.output_side,
+                   legend=options.legend,
+                   class_names=options.class_names,
+                   equations=options.equations,
                    **kwargs)
 
 
@@ -216,9 +237,12 @@ def _view_connections_setup_parser(parser):
                         help='title of web page.')
     parser.add_argument('--no_browser', action='store_true', dest='no_browser',
                         help="don't display in a browser.")
+    parser.add_argument('-v', '--show_values', action='store_true', dest='show_values',
+                        help="Display values.")
+    parser.add_argument('-p', '--problem', action='store', dest='problem', help='Problem name')
 
 
-def _view_connections_cmd(options):
+def _view_connections_cmd(options, user_args):
     """
     Return the post_setup hook function for 'openmdao view_connections'.
 
@@ -226,11 +250,8 @@ def _view_connections_cmd(options):
     ----------
     options : argparse Namespace
         Command line options.
-
-    Returns
-    -------
-    function
-        The post-setup hook function.
+    user_args : list of str
+        Args to be passed to the user script.
     """
     def _viewconns(prob):
         if options.title:
@@ -238,9 +259,17 @@ def _view_connections_cmd(options):
         else:
             title = "Connections for %s" % os.path.basename(options.file[0])
         view_connections(prob, outfile=options.outfile, show_browser=not options.no_browser,
-                         title=title)
+                         show_values=options.show_values, title=title)
         exit()
-    return _viewconns
+
+    # register the hook
+    if options.show_values:
+        funcname = 'final_setup'
+    else:
+        funcname = 'setup'
+    hooks._register_hook(funcname, class_name='Problem', inst_id=options.problem, post=_viewconns)
+
+    _load_and_exec(options.file[0], user_args)
 
 
 def _meta_model_parser(parser):
@@ -260,9 +289,11 @@ def _meta_model_parser(parser):
                         help='Number of points to create contour grid')
     parser.add_argument('-p', '--port_number', default=5007, action='store', dest='port_number',
                         help='Port number to open viewer')
+    parser.add_argument('--no_browser', action='store_false', dest='browser',
+                        help='Bokeh server will start server without browser')
 
 
-def _meta_model_cmd(options):
+def _meta_model_cmd(options, user_args):
     """
     Return the post_setup hook function for 'openmdao meta_model'.
 
@@ -270,11 +301,8 @@ def _meta_model_cmd(options):
     ----------
     options : argparse Namespace
         Command line options.
-
-    Returns
-    -------
-    function
-        The post-setup hook function.
+    user_args : list of str
+        Args to be passed to the user script.
     """
     def _view_metamodel(prob):
         if bokeh is None:
@@ -282,18 +310,19 @@ def _meta_model_cmd(options):
                   "    pip install bokeh")
             exit()
 
-        Problem._post_setup_func = None
+        hooks._unregister_hook('final_setup', 'Problem')
 
         mm_types = (MetaModelStructuredComp, MetaModelUnStructuredComp)
 
         pathname = options.pathname
         port_number = options.port_number
         resolution = options.resolution
+        browser = options.browser
 
         if pathname:
             comp = prob.model._get_subsystem(pathname)
             if comp and isinstance(comp, mm_types):
-                view_metamodel(comp, resolution, port_number)
+                view_metamodel(comp, resolution, port_number, browser)
                 exit()
         else:
             comp = None
@@ -309,7 +338,7 @@ def _meta_model_cmd(options):
 
         elif mm_count == 1 and not pathname:
             comp = metamodels[mm_names[0]]
-            view_metamodel(comp, resolution, port_number)
+            view_metamodel(comp, resolution, port_number, browser)
 
         else:
             try_str = "Try one of the following: {}.".format(mm_names)
@@ -322,7 +351,9 @@ def _meta_model_cmd(options):
                 print("\n'{}' is not a Metamodel.\n {}".format(pathname, try_str))
         exit()
 
-    return _view_metamodel
+    hooks._register_hook('final_setup', 'Problem', post=_view_metamodel)
+
+    _load_and_exec(options.file[0], user_args)
 
 
 def _config_summary_setup_parser(parser):
@@ -337,7 +368,7 @@ def _config_summary_setup_parser(parser):
     parser.add_argument('file', nargs=1, help='Python file containing the model.')
 
 
-def _config_summary_cmd(options):
+def _config_summary_cmd(options, user_args):
     """
     Return the post_setup hook function for 'openmdao summary'.
 
@@ -345,16 +376,16 @@ def _config_summary_cmd(options):
     ----------
     options : argparse Namespace
         Command line options.
-
-    Returns
-    -------
-    function
-        The post-setup hook function.
+    user_args : list of str
+        Args to be passed to the user script.
     """
     def summary(prob):
         config_summary(prob)
-        exit()
-    return summary
+        sys.exit(0)
+
+    hooks._register_hook('final_setup', 'Problem', post=summary)
+
+    _load_and_exec(options.file[0], user_args)
 
 
 def _tree_setup_parser(parser):
@@ -377,9 +408,15 @@ def _tree_setup_parser(parser):
     parser.add_argument('-a', '--attr', action='append', default=[], dest='attrs',
                         help='Add an attribute to search for in tree systems.')
     parser.add_argument('-v', '--var', action='append', default=[], dest='vecvars',
-                        help='Add a variable to search for in vectors of tree systems.')
+                        help='Add a variable to search for in vectors of tree components. '
+                             'Use component relative names.')
     parser.add_argument('-r', '--rank', action='store', type=int, dest='rank',
                         default=0, help="Display the tree on this rank (if MPI is active).")
+    parser.add_argument('-p', '--problem', action='store', dest='problem', help='Problem name')
+    parser.add_argument('-s', '--sizes', action='store_true', dest='show_sizes',
+                        help="Display input and output sizes.")
+    parser.add_argument('--approx', action='store_true', dest='show_approx',
+                        help="Show which components compute approximations.")
 
 
 def _get_tree_filter(attrs, vecvars):
@@ -391,7 +428,8 @@ def _get_tree_filter(attrs, vecvars):
     attrs : list of str
         Names of attributes (may contain dots).
     vecvars : list of str
-        Names of variables contained in the input or output vectors.
+        Names of variables contained in the input or output vectors.  Use component relative
+        names.
 
     Returns
     -------
@@ -410,18 +448,20 @@ def _get_tree_filter(attrs, vecvars):
             except AttributeError:
                 pass
 
-        for var in vecvars:
-            if var in system._outputs:
-                found.append((var, system._outputs[var]))
-            elif var in system._inputs:
-                found.append((var, system._inputs[var]))
+        if isinstance(system, Component):
+            for var in vecvars:
+                if var in system._var_rel2meta:
+                    if var in system._outputs:
+                        found.append((var, system._outputs[var]))
+                    elif var in system._inputs:
+                        found.append((var, system._inputs[var]))
 
         return found
 
     return _finder
 
 
-def _tree_cmd(options):
+def _tree_cmd(options, user_args):
     """
     Return the post_setup hook function for 'openmdao tree'.
 
@@ -429,11 +469,8 @@ def _tree_cmd(options):
     ----------
     options : argparse Namespace
         Command line options.
-
-    Returns
-    -------
-    function
-        The post-setup hook function.
+    user_args : list of str
+        Args to be passed to the user script.
     """
     if options.outfile is None:
         out = sys.stdout
@@ -446,51 +483,19 @@ def _tree_cmd(options):
         filt = None
 
     def _tree(prob):
-        tree(prob, show_colors=options.show_colors,
-             filter=filt, max_depth=options.depth, rank=options.rank, stream=out)
+        tree(prob, show_colors=options.show_colors, show_sizes=options.show_sizes,
+             show_approx=options.show_approx, filter=filt, max_depth=options.depth,
+             rank=options.rank, stream=out)
         exit()
-    return _tree
 
-
-def _dump_dist_idxs_setup_parser(parser):
-    """
-    Set up the openmdao subparser for the 'openmdao dump_idxs' command.
-
-    Parameters
-    ----------
-    parser : argparse subparser
-        The parser we're adding options to.
-    """
-    parser.add_argument('file', nargs=1, help='Python file containing the model.')
-    parser.add_argument('-o', default=None, action='store', dest='outfile',
-                        help='Name of output file.  By default, output goes to stdout.')
-    parser.add_argument('-v', '--vecname', action='store', default='nonlinear', dest='vecname',
-                        help='Name of vectors to show indices for.  Default is "nonlinear".')
-
-
-def _dump_dist_idxs_cmd(options):
-    """
-    Return the post_setup hook function for 'openmdao dump_idxs'.
-
-    Parameters
-    ----------
-    options : argparse Namespace
-        Command line options.
-
-    Returns
-    -------
-    function
-        The post-setup hook function.
-    """
-    if options.outfile is None:
-        out = sys.stdout
+    # register the hook
+    if options.vecvars or options.show_sizes or options.show_approx:
+        funcname = 'final_setup'
     else:
-        out = open(options.outfile, 'w')
+        funcname = 'setup'
+    hooks._register_hook(funcname, class_name='Problem', inst_id=options.problem, post=_tree)
 
-    def _dumpdist(prob):
-        dump_dist_idxs(prob, vec_name=options.vecname, stream=out)
-        exit()
-    return _dumpdist
+    _load_and_exec(options.file[0], user_args)
 
 
 def _cite_setup_parser(parser):
@@ -509,19 +514,21 @@ def _cite_setup_parser(parser):
                         help='Find citation for this class.')
 
 
-def _cite_cmd(options):
+def _cite_cmd(options, user_args):
     """
-    Return the post setup hook function for `openmdao cite`.
+    Run the `openmdao cite` command.
 
     Parameters
     ----------
     options : argparse Namespace
         Command line options.
+    user_args : list of str
+        Args to be passed to the user script.
 
     Returns
     -------
     function
-        The post-setup hook function.
+        The hook function.
     """
     if options.outfile is None:
         out = sys.stdout
@@ -536,116 +543,131 @@ def _cite_cmd(options):
             print_citations(prob, classes=options.classes, out_stream=out)
         exit()
 
-    return _cite
+    hooks._register_hook('setup', 'Problem', post=_cite)
+
+    _load_and_exec(options.file[0], user_args)
 
 
-def _post_setup_exec(options):
-    """
-    Use this as executor for commands that run as Problem post-setup commands.
-
-    Parameters
-    ----------
-    options : argparse Namespace
-        Command line options.
-    """
-    progname = options.file[0]
-
-    sys.path.insert(0, os.path.dirname(progname))
-
-    with open(progname, 'rb') as fp:
-        code = compile(fp.read(), progname, 'exec')
-
-    globals_dict = {
-        '__file__': progname,
-        '__name__': '__main__',
-        '__package__': None,
-        '__cached__': None,
-    }
-
-    if options.func is not None:
-        Problem._post_setup_func = options.func(options)
-
-    exec(code, globals_dict)
-
-
-# NOTE: any post_setup functions must handle their own exit behavior. If you want them
-# to exit after running, exit() must be called from within your function.  This also gives
-# you the option of controlling the exit behavior via a command line argument.
-
-# All post-setup functions go here.
 # this dict should contain names mapped to tuples of the form:
-#   (setup_parser_func, func, description)
-_post_setup_map = {
-    'view_connections': (_view_connections_setup_parser, _view_connections_cmd,
-                         'Connection viewer showing values and source/target units.'),
-    'summary': (_config_summary_setup_parser, _config_summary_cmd,
-                'Print a short top-level summary of the problem.'),
-    'tree': (_tree_setup_parser, _tree_cmd, 'Print the system tree.'),
-    'dump_idxs': (_dump_dist_idxs_setup_parser, _dump_dist_idxs_cmd,
-                  'Show distributed index information.'),
-    'total_coloring': (_total_coloring_setup_parser, _total_coloring_cmd,
-                       'Compute a coloring for the total jacobian.'),
-    'partial_coloring': (_partial_coloring_setup_parser, _partial_coloring_cmd,
-                         'Compute coloring(s) for specified partial jacobians.'),
-    'total_sparsity': (_sparsity_setup_parser, _sparsity_cmd,
-                       'Compute the sparsity pattern of the total jacobian.'),
-    'cite': (_cite_setup_parser, _cite_cmd,
-             'Print citations referenced by problem'),
-    'check': (_check_config_setup_parser, _check_config_cmd,
-              'Perform a number of configuration checks on the problem.'),
-    'view_mm': (_meta_model_parser, _meta_model_cmd, "Meta Model Viewer.")
-}
-
-
-# Other non-post-setup functions go here
-_non_post_setup_map = {
-    'n2': (_n2_setup_parser, _n2_cmd, 'Display an interactive N2 diagram of the problem.'),
-    'view_model': (_n2_setup_parser, _view_model_cmd,
-                   'Display an interactive N2 diagram of the problem. '
-                   '(Deprecated, please use n2 instead.)'),
-    'trace': (_itrace_setup_parser, _itrace_exec, 'Dump trace output.'),
+#   (setup_parser_func, executor, description)
+_command_map = {
     'call_tree': (_calltree_setup_parser, _calltree_exec,
                   "Display the call tree for the specified class method and all 'self' class "
                   "methods it calls."),
+    'check': (_check_config_setup_parser, _check_config_cmd,
+              'Perform a number of configuration checks on the problem.'),
+    'cite': (_cite_setup_parser, _cite_cmd, 'Print citations referenced by the problem.'),
+    'compute_entry_points': (_compute_entry_points_setup_parser, _compute_entry_points_exec,
+                             'Compute entry point declarations to add to the setup.py file.'),
+    'find_plugins': (_find_plugins_setup_parser, _find_plugins_exec,
+                     'Find openmdao plugins on github.'),
     'iprof': (_iprof_setup_parser, _iprof_exec,
-              'Profiling of calls to particular object instances.'),
+              'Profile calls to particular object instances.'),
     'iprof_totals': (_iprof_totals_setup_parser, _iprof_totals_exec,
-                     'Total timings of calls to particular object instances.'),
-    'mem': (_mem_prof_setup_parser, _mem_prof_exec, 'Memory profiler.'),
-    'mempost': (_mempost_setup_parser, _mempost_exec, 'Post-processor for memory profile output.'),
-    'view_coloring': (_view_coloring_setup_parser, _view_coloring_exec,
-                      'Colored jacobian viewer.'),
-    'xdsm': (_xdsm_setup_parser, _xdsm_cmd, 'XDSM viewer.'),
+                     'Generate total timings of calls to particular object instances.'),
+    'list_installed': (_list_installed_setup_parser, _list_installed_cmd,
+                       'List installed types recognized by OpenMDAO.'),
+    'mem': (_mem_prof_setup_parser, _mem_prof_exec,
+            'Profile memory used by OpenMDAO related functions.'),
+    'mempost': (_mempost_setup_parser, _mempost_exec, 'Post-process memory profile output.'),
+    'n2': (_n2_setup_parser, _n2_cmd, 'Display an interactive N2 diagram of the problem.'),
+    'partial_coloring': (_partial_coloring_setup_parser, _partial_coloring_cmd,
+                         'Compute coloring(s) for specified partial jacobians.'),
     'scaffold': (_scaffold_setup_parser, _scaffold_exec,
                  'Generate a simple scaffold for a component.'),
+    'summary': (_config_summary_setup_parser, _config_summary_cmd,
+                'Print a short top-level summary of the problem.'),
+    'total_coloring': (_total_coloring_setup_parser, _total_coloring_cmd,
+                       'Compute a coloring for the total jacobian.'),
+    'trace': (_itrace_setup_parser, _itrace_exec, 'Dump trace output.'),
+    'tree': (_tree_setup_parser, _tree_cmd, 'Print the system tree.'),
+    'view_coloring': (_view_coloring_setup_parser, _view_coloring_exec, 'View a colored jacobian.'),
+    'view_connections': (_view_connections_setup_parser, _view_connections_cmd,
+                         'View connections showing values and source/target units.'),
+    'view_mm': (_meta_model_parser, _meta_model_cmd, "View a metamodel."),
+    'view_model': (_n2_setup_parser, _view_model_cmd,
+                   'Display an interactive N2 diagram of the problem. '
+                   '(Deprecated, please use n2 instead.)'),
+    'xdsm': (_xdsm_setup_parser, _xdsm_cmd, 'Generate an XDSM diagram of a model.'),
 }
 
 
 def openmdao_cmd():
     """
-    Wrap a number of Problem viewing/debugging command line functions.
+    Run an 'openmdao' sub-command or list help info for 'openmdao' command or sub-commands.
     """
-    parser = argparse.ArgumentParser(description='OpenMDAO Command Line Tools',
-                                     epilog='Use -h after any sub-command for sub-command help.')
+    # pre-parse sys.argv to split between before and after '--'
+    if '--' in sys.argv:
+        idx = sys.argv.index('--')
+        sys_args = sys.argv[:idx]
+        user_args = sys.argv[idx + 1:]
+        sys.argv[:] = sys_args
+    else:
+        user_args = []
 
-    subs = parser.add_subparsers(title='Tools', metavar='')
-    for p, (parser_setup_func, cmd, help_str) in sorted(chain(_post_setup_map.items(),
-                                                              _non_post_setup_map.items())):
+    parser = argparse.ArgumentParser(description='OpenMDAO Command Line Tools',
+                                     epilog='Use -h after any sub-command for sub-command help.'
+                                     ' If using a tool on a script that takes its own command line'
+                                     ' arguments, place those arguments after a "--". For example:'
+                                     ' openmdao n2 -o foo.html myscript.py -- -x --myarg=bar')
+
+    # setting 'dest' here will populate the Namespace with the active subparser name
+    subs = parser.add_subparsers(title='Tools', metavar='', dest="subparser_name")
+    for p, (parser_setup_func, executor, help_str) in sorted(_command_map.items()):
         subp = subs.add_parser(p, help=help_str)
         parser_setup_func(subp)
-        if p in _post_setup_map:
-            subp.set_defaults(func=cmd, executor=_post_setup_exec)
-        else:
-            subp.set_defaults(executor=cmd)
+        subp.set_defaults(executor=executor)
 
-    # handle case where someone just runs `openmdao <script>`
+    if pkg_resources is None:
+        print("\npkg_resources was not found, so no plugin entry points can be loaded.\n")
+    else:
+        # now add any plugin openmdao commands
+        epdict = {}
+        for ep in pkg_resources.iter_entry_points(group='openmdao_command'):
+            cmd, module, target = split_ep(ep)
+            # don't let plugins override the builtin commands
+            if cmd in _command_map:
+                raise RuntimeError("openmdao plugin command '{}' defined in {} conflicts with "
+                                   "builtin command '{}'.".format(cmd, module, cmd))
+            elif cmd in epdict:
+                raise RuntimeError("openmdao plugin command '{}' defined in {} conflicts with a "
+                                   "another plugin command defined in {}."
+                                   .format(cmd, module, epdict[cmd][1]))
+            epdict[cmd] = (ep, module)
+
+        # sort commands by module and then by command name so commands from plugins will
+        # be grouped together.
+        for cmd, (ep, module) in sorted(epdict.items(), key=lambda x: x[1][1] + x[0]):
+            func = ep.load()
+            parser_setup_func, executor, help_str = func()
+            pkg = module.split('.', 1)[0]
+            subp = subs.add_parser(cmd, help='(%s plugin) ' % pkg + help_str)
+            parser_setup_func(subp)
+            subp.set_defaults(executor=executor)
+
+    # handle case where someone just runs `openmdao <script> [dashed-args]`
     args = [a for a in sys.argv[1:] if not a.startswith('-')]
     if not set(args).intersection(subs.choices) and len(args) == 1 and os.path.isfile(args[0]):
-        _post_setup_exec(_Options(file=[args[0]], func=None))
+        _load_and_exec(args[0], user_args)
     else:
-        options = parser.parse_args()
+        hooks.use_hooks = True
+        # we do a parse_known_args here instead of parse_args so that we can associate errors with
+        # the correct subparser.  Otherwise we would just get a top level error message without any
+        # sub-command usage info.
+        options, unknown = parser.parse_known_args()
+        if unknown:
+            msg = 'unrecognized arguments: ' + ', '.join(unknown)
+            try:
+                sub = subs.choices[options.subparser_name]
+            except KeyError:
+                parser.error(msg)
+            else:
+                print(sub.format_usage(), file=sys.stderr)
+                print(msg, file=sys.stderr)
+            parser.exit(2)
+
         if hasattr(options, 'executor'):
-            options.executor(options)
+            options.executor(options, user_args)
         else:
             print("\nNothing to do.")
 
