@@ -27,6 +27,8 @@ class OptionsDictionary(object):
         If defined, prepend this name to beginning of all exceptions.
     _read_only : bool
         If True, no options can be set after declaration.
+    _all_recordable : bool
+        Flag to determine if all options in UserOptions are recordable.
     """
 
     def __init__(self, parent_name=None, read_only=False):
@@ -44,6 +46,24 @@ class OptionsDictionary(object):
         self._parent_name = parent_name
         self._read_only = read_only
 
+        self._all_recordable = True
+
+    def __getstate__(self):
+        """
+        Return state as a dict.
+
+        Returns
+        -------
+        dict
+            State to get.
+        """
+        if self._all_recordable:
+            return self.__dict__
+        else:
+            state = self.__dict__.copy()
+            state['_dict'] = {key: val for key, val in state['_dict'].items() if val['recordable']}
+            return state
+
     def __repr__(self):
         """
         Return a dictionary representation of the options.
@@ -76,7 +96,7 @@ class OptionsDictionary(object):
             # if the default is an object instance, replace with the (unqualified) object type
             default_str = str(default)
             idx = default_str.find(' object at ')
-            if idx >= 0 and default_str[0] is '<':
+            if idx >= 0 and default_str[0] == '<':
                 parts = default_str[:idx].split('.')
                 default = parts[-1]
 
@@ -243,30 +263,30 @@ class OptionsDictionary(object):
                         typs = tuple([type_.__name__ for type_ in types])
                         self._raise("Value ({}) of option '{}' has type '{}', but one of "
                                     "types {} was expected.".format(value, name, vtype, typs),
-                                    TypeError)
+                                    exc_type=TypeError)
                     else:
                         self._raise("Value ({}) of option '{}' has type '{}', but type '{}' "
                                     "was expected.".format(value, name, vtype, types.__name__),
-                                    TypeError)
+                                    exc_type=TypeError)
 
             if upper is not None:
                 if value > upper:
                     self._raise("Value ({}) of option '{}' "
                                 "exceeds maximum allowed value of {}.".format(value, name, upper),
-                                ValueError)
+                                exc_type=ValueError)
             if lower is not None:
                 if value < lower:
                     self._raise("Value ({}) of option '{}' "
                                 "is less than minimum allowed value of {}.".format(value, name,
                                                                                    lower),
-                                ValueError)
+                                exc_type=ValueError)
 
         # General function test
         if meta['check_valid'] is not None:
             meta['check_valid'](name, value)
 
     def declare(self, name, default=_undefined, values=None, types=None, type_=None, desc='',
-                upper=None, lower=None, check_valid=None, allow_none=False):
+                upper=None, lower=None, check_valid=None, allow_none=False, recordable=True):
         r"""
         Declare an option.
 
@@ -297,6 +317,8 @@ class OptionsDictionary(object):
             General check function that raises an exception if value is not valid.
         allow_none : bool
             If True, allow None as a value regardless of values or types.
+        recordable : bool
+            If True, add to recorder
         """
         if type_ is not None:
             warn_deprecation("In declaration of option '%s' the '_type' arg is deprecated.  "
@@ -306,11 +328,11 @@ class OptionsDictionary(object):
 
         if values is not None and not isinstance(values, (set, list, tuple)):
             self._raise("In declaration of option '%s', the 'values' arg must be of type None,"
-                        " list, or tuple - not %s." % (name, values), TypeError)
+                        " list, or tuple - not %s." % (name, values), exc_type=TypeError)
 
         if types is not None and not isinstance(types, (type, set, list, tuple)):
             self._raise("In declaration of option '%s', the 'types' arg must be None, a type "
-                        "or a tuple - not %s." % (name, types), TypeError)
+                        "or a tuple - not %s." % (name, types), exc_type=TypeError)
 
         if types is not None and values is not None:
             self._raise("'types' and 'values' were both specified for option '%s'." % name)
@@ -318,6 +340,9 @@ class OptionsDictionary(object):
         if types is bool:
             values = (True, False)
 
+        if not recordable:
+            self._all_recordable = False
+
         default_provided = default is not _undefined
 
         self._dict[name] = {
@@ -330,6 +355,7 @@ class OptionsDictionary(object):
             'check_valid': check_valid,
             'has_been_set': default_provided,
             'allow_none': allow_none,
+            'recordable': recordable,
         }
 
         # If a default is given, check for validity
@@ -404,10 +430,10 @@ class OptionsDictionary(object):
         except KeyError:
             # The key must have been declared.
             msg = "Option '{}' cannot be set because it has not been declared."
-            self._raise(msg.format(name), KeyError)
+            self._raise(msg.format(name), exc_type=KeyError)
 
         if self._read_only:
-            self._raise("Tried to set read-only option '{}'.".format(name), KeyError)
+            self._raise("Tried to set read-only option '{}'.".format(name), exc_type=KeyError)
 
         self._assert_valid(name, value)
 
@@ -436,4 +462,4 @@ class OptionsDictionary(object):
             else:
                 self._raise("Option '{}' is required but has not been set.".format(name))
         except KeyError:
-            self._raise("Option '{}' cannot be found".format(name), KeyError)
+            self._raise("Option '{}' cannot be found".format(name), exc_type=KeyError)
