@@ -25,17 +25,19 @@ from openmdao.core.group import Group
 from openmdao.core.problem import Problem
 from openmdao.core.component import Component
 from openmdao.core.implicitcomponent import ImplicitComponent
-from openmdao.visualization.html_utils import read_files, write_script, DiagramWriter
+from openmdao.drivers.doe_driver import DOEDriver
+from openmdao.recorders.case_reader import CaseReader
+from openmdao.solvers.nonlinear.newton import NewtonSolver
 from openmdao.utils.class_util import overrides_method
 from openmdao.utils.general_utils import warn_deprecation, simple_warning, make_serializable
 from openmdao.utils.record_util import check_valid_sqlite3_db
 from openmdao.utils.mpi import MPI
-from openmdao.recorders.case_reader import CaseReader
-from openmdao.drivers.doe_driver import DOEDriver
+from openmdao.visualization.html_utils import read_files, write_script, DiagramWriter
 
 # Toolbar settings
 _FONT_SIZES = [8, 9, 10, 11, 12, 13, 14]
-_MODEL_HEIGHTS = [600, 650, 700, 750, 800, 850, 900, 950, 1000, 2000, 3000, 4000]
+_MODEL_HEIGHTS = [600, 650, 700, 750, 800,
+                  850, 900, 950, 1000, 2000, 3000, 4000]
 
 _IND = 4  # HTML indentation (spaces)
 
@@ -68,6 +70,7 @@ def _get_tree_dict(system, component_execution_orders, component_execution_index
     tree_dict['name'] = system.name
     tree_dict['type'] = 'subsystem'
     tree_dict['class'] = system.__class__.__name__
+    tree_dict['expressions'] = None
 
     if not isinstance(system, Group):
         tree_dict['subsystem_type'] = 'component'
@@ -76,6 +79,7 @@ def _get_tree_dict(system, component_execution_orders, component_execution_index
             tree_dict['component_type'] = 'implicit'
         elif isinstance(system, ExecComp):
             tree_dict['component_type'] = 'exec'
+            tree_dict['expressions'] = system._exprs
         elif isinstance(system, (MetaModelStructuredComp, MetaModelUnStructuredComp)):
             tree_dict['component_type'] = 'metamodel'
         elif isinstance(system, IndepVarComp):
@@ -138,6 +142,9 @@ def _get_tree_dict(system, component_execution_orders, component_execution_index
 
         if system.nonlinear_solver:
             tree_dict['nonlinear_solver'] = system.nonlinear_solver.SOLVER
+
+            if system.nonlinear_solver.SOLVER == NewtonSolver.SOLVER:
+                tree_dict['solve_subsystems'] = system._nonlinear_solver.options['solve_subsystems']
         else:
             tree_dict['nonlinear_solver'] = ""
 
@@ -200,15 +207,17 @@ def _get_viewer_data(data_source):
         root_group = data_source.model
 
         if not isinstance(root_group, Group):
-            simple_warning("The model is not a Group, viewer data is unavailable.")
+            simple_warning(
+                "The model is not a Group, viewer data is unavailable.")
             return {}
 
         driver = data_source.driver
         driver_name = driver.__class__.__name__
-        driver_type = 'doe' if isinstance(driver, DOEDriver) else 'optimization'
+        driver_type = 'doe' if isinstance(
+            driver, DOEDriver) else 'optimization'
         driver_options = {k: driver.options[k] for k in driver.options}
         driver_opt_settings = None
-        if driver_type is 'optimization' and 'opt_settings' in dir(driver):
+        if driver_type == 'optimization' and 'opt_settings' in dir(driver):
             driver_opt_settings = driver.opt_settings
 
     elif isinstance(data_source, Group):
@@ -226,7 +235,8 @@ def _get_viewer_data(data_source):
         return CaseReader(data_source, pre_load=False).problem_metadata
 
     else:
-        raise TypeError('_get_viewer_data only accepts Problems, Groups or filenames')
+        raise TypeError(
+            '_get_viewer_data only accepts Problems, Groups or filenames')
 
     data_dict = {}
     comp_exec_idx = [0]  # list so pass by ref
@@ -270,8 +280,8 @@ def _get_viewer_data(data_source):
                 edges_list = [
                     (sys_pathnames_dict[s], sys_pathnames_dict[t]) for s, t in G.edges(strong_comp)
                     if s in orders and exe_low <= orders[s] <= exe_high and t in orders and
-                        exe_low <= orders[t] <= exe_high and
-                        not (s == src and t == tgt) and t in sys_pathnames_dict
+                    exe_low <= orders[t] <= exe_high and
+                    not (s == src and t == tgt) and t in sys_pathnames_dict
                 ]
                 for vsrc, vtgtlist in iteritems(G.get_edge_data(src, tgt)['conns']):
                     for vtgt in vtgtlist:
@@ -303,7 +313,7 @@ def view_tree(*args, **kwargs):
     Parameters
     ----------
     *args : dict
-        Postional args.
+        Positional args.
     **kwargs : dict
         Keyword args.
     """
@@ -318,7 +328,7 @@ def view_model(*args, **kwargs):
     Parameters
     ----------
     *args : dict
-        Postional args.
+        Positional args.
     **kwargs : dict
         Keyword args.
     """
@@ -358,17 +368,18 @@ def n2(data_source, outfile='n2.html', show_browser=True, embeddable=False,
         is assumed.
 
     """
+    # grab the model viewer data
+    model_data = _get_viewer_data(data_source)
+
     # if MPI is active only display one copy of the viewer
     if MPI and MPI.COMM_WORLD.rank != 0:
         return
 
-    # grab the model viewer data
-    model_data = _get_viewer_data(data_source)
-    options = {}
-    options['use_declare_partial_info'] = use_declare_partial_info
+    options = {'use_declare_partial_info': use_declare_partial_info}
     model_data['options'] = options
 
-    model_data = 'var modelData = %s' % json.dumps(model_data, default=make_serializable)
+    model_data = 'var modelData = %s' % json.dumps(
+        model_data, default=make_serializable)
 
     import openmdao
     openmdao_dir = os.path.dirname(inspect.getfile(openmdao))
@@ -378,9 +389,26 @@ def n2(data_source, outfile='n2.html', show_browser=True, embeddable=False,
     style_dir = os.path.join(vis_dir, "style")
 
     # grab the libraries, src and style
-    lib_dct = {'d3': 'd3.v4.min', 'awesomplete': 'awesomplete', 'vk_beautify': 'vkBeautify'}
+    lib_dct = {'d3': 'd3.v5.min', 'awesomplete': 'awesomplete',
+               'vk_beautify': 'vkBeautify'}
     libs = read_files(itervalues(lib_dct), libs_dir, 'js')
-    src_names = 'constants', 'draw', 'legend', 'modal', 'ptN2', 'search', 'svg'
+    src_names = \
+        'modal', \
+        'utils', \
+        'SymbolType', \
+        'N2TreeNode', \
+        'ModelData', \
+        'N2Style', \
+        'N2Layout', \
+        'N2MatrixCell', \
+        'N2Legend', \
+        'N2Matrix', \
+        'N2Arrow', \
+        'N2Search', \
+        'N2Diagram', \
+        'N2UserInterface', \
+        'defaults', \
+        'ptN2'
     srcs = read_files(src_names, src_dir, 'js')
     styles = read_files(('awesomplete', 'partition_tree'), style_dir, 'css')
 
@@ -403,7 +431,8 @@ def n2(data_source, outfile='n2.html', show_browser=True, embeddable=False,
         h.insert('{{{}_lib}}'.format(k), write_script(libs[v], indent=_IND))
 
     for name, code in iteritems(srcs):
-        h.insert('{{{}_lib}}'.format(name.lower()), write_script(code, indent=_IND))
+        h.insert('{{{}_lib}}'.format(name.lower()),
+                 write_script(code, indent=_IND))
 
     h.insert('{{model_data}}', write_script(model_data, indent=_IND))
 
@@ -412,7 +441,8 @@ def n2(data_source, outfile='n2.html', show_browser=True, embeddable=False,
     group1 = toolbar.add_button_group()
     group1.add_button("Return To Root", uid="returnToRootButtonId", disabled="disabled",
                       content="icon-home")
-    group1.add_button("Back", uid="backButtonId", disabled="disabled", content="icon-left-big")
+    group1.add_button("Back", uid="backButtonId",
+                      disabled="disabled", content="icon-left-big")
     group1.add_button("Forward", uid="forwardButtonId", disabled="disabled",
                       content="icon-right-big")
     group1.add_button("Up One Level", uid="upOneLevelButtonId", disabled="disabled",
@@ -433,14 +463,18 @@ def n2(data_source, outfile='n2.html', show_browser=True, embeddable=False,
     group3 = toolbar.add_button_group()
     group3.add_button("Clear Arrows and Connections", uid="clearArrowsAndConnectsButtonId",
                       content="icon-eraser")
-    group3.add_button("Show Path", uid="showCurrentPathButtonId", content="icon-terminal")
-    group3.add_button("Show Legend", uid="showLegendButtonId", content="icon-map-signs")
-    group3.add_button("Toggle Solver Names", uid="toggleSolverNamesButtonId", content="icon-minus")
+    group3.add_button(
+        "Show Path", uid="showCurrentPathButtonId", content="icon-terminal")
+    group3.add_button("Show Legend", uid="showLegendButtonId",
+                      content="icon-map-signs")
+    group3.add_button("Toggle Solver Names",
+                      uid="toggleSolverNamesButtonId", content="icon-minus")
     group3.add_dropdown("Font Size", id_naming="idFontSize", options=_FONT_SIZES,
                         option_formatter=lambda x: '{}px'.format(x),
                         button_content="icon-text-height")
     group3.add_dropdown("Vertically Resize", id_naming="idVerticalResize",
-                        options=_MODEL_HEIGHTS, option_formatter=lambda x: '{}px'.format(x),
+                        options=_MODEL_HEIGHTS, option_formatter=lambda x: '{}px'.format(
+                            x),
                         button_content="icon-resize-vertical", header="Model Height")
 
     group4 = toolbar.add_button_group()
